/*
  filter.c - Particle filtering routines for Seal PF program
*/

#include "prepro.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#include <limits.h>
#include "global.h"
#include "util.h"
#include "nrutil.h"
#include "numeric.h"
#include "model.h"
#include "filter.h"

//This is the only global variable - declared in main.c
extern long glngSeed;

/*------------------------------------------------------------------------*/

void doPF (controlPFstruct * const pControlPF, metastruct * const pMeta)
/*
  Runs one iteration of particle filtering, from getting the priors right
    to the end of the data.
  When it gets to the end of the data, performs rejection control and
    stores the surviving particles to disk in binary or text format with suffix i
  Previous incarnation in SPlus was do.sis.by.i
  */
{

  //Structure to hold the particles
  particlestruct *pParticles;

  //Set aside memory for the particles
  pParticles = mallocParticleStruct(pControlPF->intNParticles, NAGES,
    NCOLONIES, NPARAMS);

  /*call doPFYear2 */
  doPFYear2 (pControlPF, pMeta, pParticles);

  /*call doPFYear3Plus */
  doPFYear3Plus (pControlPF, pMeta, pParticles);

  //if you integrated out psi then generate values for it
  // from the full conditional, before saving to file
  if(pControlPF->blnIntegrateOutPsi) 
    samplePsi(pControlPF, pParticles, pControlPF->intNYears);    

  /*do final RC and save smoothed estimates to file */
  processResults (pControlPF, pMeta, pParticles);

  //Free memory for the assigned particles
  freeParticleStruct(pParticles, pControlPF->intNParticles, NAGES,
    NCOLONIES, NPARAMS);
}

/*------------------------------------------------------------------------*/

void doPFYear2 (controlPFstruct * const pControlPF, metastruct * const pMeta,
                particlestruct * const pParticles)
/*
Purpose:
  Runs particle filter algorithm for first 2 years of data.  Can optionally use
    rejection control to return a set of particles that is congruent with the
    year 2 data..
  Uses year 1 data to get priors for states; runs forward to year 2; if
    pControlPF.blnRCYr1 then uses rejection control and accumulates
    pControlPF.intNParticles before returning.
  Inputs:
    pControlPF is the particle filter control structure
    pMeta is the meta structure (for recording info about the results)
    pParticles is a pointer to an the particles structure - this has been
      initialized, in the sense that memory has been set aside for it, but
      not in any other sense (ie no ID's, no params, etc)
  Outputs:
    pControlPF is untouched
    pMeta has an updated number of particles generated in Year 1
    pParticles contains the particles that are now run up to year 2
  */
{

  Int i;
  //Shortcut to number of particles in pParticles
  Int intNParticles;
  //true when rejection control finished
  boolean blnRCFinished;
  //number of rejection control iterations so far
  Int intRCIterations=0;
  //number of particles surviving one iteration of RC
  Int intNParticlesSaved;
  //number of particles created and surviving RC so far
  Int intCumNParticlesSaved=0;
  //stores the calculated ESS
  double dblESS;
  //Used to index particles that get through rejection control, if used
  Int *intParticleIndex;
  //Number of particles in the index
  Int intNParticleIndex;
  //Used in rejection control
  double dblC;
  //Temp var
  Int intParent;

  //Initialize the particle index
  intParticleIndex = ivector(1, pControlPF->intNParticles);
  if(!(pControlPF->blnRCYr1)) {
    intNParticleIndex=pControlPF->intNParticles;
    for(i=1;i<=intNParticleIndex;i++) {
      intParticleIndex[i]=i;
    }
  }

  //Add 1 to the iteration number in meta for years 1 and 2
  pMeta[1].intIterations ++;
  pMeta[2].intIterations ++;

  intNParticles = pControlPF->intNParticles;

  //blnRCFinished is true by default if not doing RC in year 1
  blnRCFinished = !(pControlPF->blnRCYr1);

  do { //until RC is finished

    intRCIterations ++;

    if (pControlPF->blnDebugFile)
      fprintf(pControlPF->pDebugFile,HEADING(Year 1 RC iteration %i),intRCIterations);

    //Give each particle a unique ID
    for(i=1;i<=intNParticles;i++) pParticles[i].intID = pMeta[1].intTotalParticles+i;
    //Increment the store of the number of particle IDs used so far
    pMeta[1].intTotalParticles+=intNParticles;
    pMeta[2].intTotalParticles+=intNParticles;

    if(pControlPF->enumInitializationMethod==fromFile) {
      //Sample particles from file
      readPriorsFromFile(pControlPF, pParticles);
    } else {
      //Give all particles equal weight
      for(i=1;i<=intNParticles;i++) pParticles[i].dblWeight = 1.0;
      //Sample from prior distributions
      initializeParams(pControlPF, pParticles);
      initializeStates(pControlPF, pParticles);
    }

    if (pControlPF->blnDebugFile)
      fprintf(pControlPF->pDebugFile,HEADING(Year 2 RC iteration %i),intRCIterations);


    if (pControlPF->blnRCYr1) {

      //Run the particle filter to project forward to year 2 and update the weights
      //Use the importance sample algorithm, rather than any tricks - as
      // no need for tricks in year 1 - the rejection control algorithm
      // will ensure unique particles are generated eventually
      importanceSample(2, pControlPF, pMeta, pParticles);

      //Apply rejection control
      dblC = getC(pParticles, pControlPF->intNParticles);
      if(pControlPF->blnDebugFile) {
        fprintf(pControlPF->pDebugFile, "Rejection control\n  RC criterion %f\n  Before ",dblC);
        getESS(pControlPF, pParticles, intParticleIndex, -1, True);
      }
      rejectionControl(pParticles, pControlPF->intNParticles,
        intParticleIndex, &intNParticleIndex,dblC);
      if(pControlPF->blnDebugFile) {
        fprintf(pControlPF->pDebugFile, "  After ");
        getESS(pControlPF, pParticles, intParticleIndex, intNParticleIndex, True);
      }
    
    } else {
    
      //Run the particle filter to project forward to year 2 and update the weights
      //No rejection control, so use whatever tricks specified by pControlPF
      filterParticles(2, pControlPF, pMeta, pParticles);
    
    }

    //Set the Parent ID of the particles to save
    intParent=intCumNParticlesSaved;
    for(i=1; i<=intNParticleIndex; i++){
      intParent ++;
      pParticles[intParticleIndex[i]].intParent = intParent;
    }
    //Save the particles pointed to by intParticleIndex
    //Year 1
    intNParticlesSaved = saveParticles(pControlPF, pParticles,
      intParticleIndex, intNParticleIndex, 1, 
      True, False, True, True, False, (intRCIterations>1), False,
      intNParticles-intCumNParticlesSaved, TEMPFILE);
    pMeta[1].intNParticlesSaved+=intNParticlesSaved;
    //Year 2
    intNParticlesSaved = saveParticles(pControlPF, pParticles,
      intParticleIndex, intNParticleIndex, 2, 
      True, True, True, True, False, (intRCIterations>1), False,
      intNParticles-intCumNParticlesSaved, TEMPFILE);
    pMeta[2].intNParticlesSaved+=intNParticlesSaved;

    //Increment the intCumNParticlesAfterRC;
    intCumNParticlesSaved+=intNParticlesSaved;

    if (pControlPF->blnDebugFile) {
///this needs changing -- ESS in year 1 is no longer number of particles
///when read in from file.  Also possibly need to change the 
///way the Year 1 particles are saved in the lines above -- at present
///the states that are being saved are the posterior in year 1 given year
///2 data -- which is inconsistent with the others...
      // note - year 1 ESS is simply the number of particles
      pMeta[1].dblESS += pControlPF->intNParticles;
      // get the ESS and write out to debug file
      dblESS = getESS(pControlPF, pParticles, intParticleIndex, intNParticleIndex, True);
      // write ESS to meta
      pMeta[2].dblESS += dblESS;
    }

    //Write out to the meta file
    writeMeta(pControlPF, pMeta, False, True);

    //If this value is more than intNParticles, then you've finished and set
    //blnFinished to true
    if(intCumNParticlesSaved>=intNParticles) blnRCFinished=True;

  } while ((!blnRCFinished)&&(intRCIterations<pControlPF->intRCYr1MaxNIterations));

  free_ivector(intParticleIndex, 1, pControlPF->intNParticles);

  if((pControlPF->blnRCYr1)&&(intCumNParticlesSaved<pControlPF->intNParticles)) {
    //Will need to allow fewer particles to be loaded in future time periods or some such...
    printf("Current code is not designed for the case when you do maximum yr1 iterations but dont get enough particles!");
    exit(1);
  }

}

/*------------------------------------------------------------------------*/
void readPriorsFromFile(controlPFstruct * const pControlPF,
                particlestruct * const pParticles)
/*
Purpose:
  Takes a sample of particles with replacement from the files indicated in the pControlPF 
  and fills the particle structure with them.  This way of getting prior particles is an
  alternative to sampling from given distributions (which is what initializeParams and
  initializeStates do.
*/
{
  //Number of particles in prior files
  Int intNPriorParticles;
  //Weights of particles in prior files
  double *dblPriorWeights;
  //Vector showing which particles get resampled
  Int *intResampleIndex;

  //Get number of particles in prior particles files
  intNPriorParticles = getNParticlesBinary(0, pControlPF->strPriorFilePrefix, "");
  //If more than the maximum number set by the user then truncate
  if ((pControlPF->intMaxPriorParticlesFromFile>0) &
    (pControlPF->intMaxPriorParticlesFromFile<intNPriorParticles))
      intNPriorParticles = pControlPF->intMaxPriorParticlesFromFile;

  //Read the weights in
  dblPriorWeights = dvector(1, intNPriorParticles);
  loadWeightsBinary(dblPriorWeights, 0, 0, 0, intNPriorParticles, pControlPF->strPriorFilePrefix, "");

  //Get the resample index
  intResampleIndex = ivector(1, pControlPF->intNParticles);
  resample(intResampleIndex, pControlPF->intNParticles, dblPriorWeights, 
     intNPriorParticles, pControlPF->enumResamplingAlgorithm, &glngSeed);
  
  //Call loadParticlesBinary to take the sample
  loadParticlesBinary(pParticles, intResampleIndex, pControlPF->intNParticles, 
    0, pControlPF->intNParticles, False, False, pControlPF->strPriorFilePrefix, "");

  free_dvector(dblPriorWeights, 1, intNPriorParticles);


  //If debugging, then print out summary states and params
  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Summary of initial parameter values from prior particle file\n");
    writeParamsSummary(pControlPF, pParticles);

    fprintf(pControlPF->pDebugFile, "Summary of initial state values from prior particle file\n");
    writeStatesSummary(pControlPF, pParticles, 1, False);
  }

}

/*------------------------------------------------------------------------*/

void doPFYear3Plus (controlPFstruct * const pControlPF, metastruct * const pMeta,
                particlestruct * const pParticles)
/*
Purpose:
  Runs particle filter algorithms for years 3 and after
  Takes prior for year 3 as input
*/
{

  Int intYear, intParticle, intParticlesSaved, *intDummy=0;
  double dblESS;

  // loop around each remaining year
  for(intYear=3;intYear<=pControlPF->intNYears;intYear++) {

    //Add 1 to the iteration number in meta for the year
    pMeta[intYear].intIterations ++;

    // load particles from file
    loadParticlesBinary(pParticles, intDummy, 0, intYear-1,
      pControlPF->intNParticles, True, pControlPF->blnIntegrateOutPsi,
      pControlPF->strResultsPrefix, TEMPFILE);

    //Reset the Parent part of the particles - used later in smoothing
    for(intParticle=1; intParticle<pControlPF->intNParticles; intParticle++)
      pParticles[intParticle].intParent = intParticle;

    //Add the number of particles to the total in meta for this year
    pMeta[intYear].intTotalParticles+=pControlPF->intNParticles;

    if (pControlPF->blnDebugFile)
      fprintf(pControlPF->pDebugFile,HEADING(Year %i),intYear);

    // call importance sampling (or whatever)
    filterParticles(intYear, pControlPF, pMeta, pParticles);

    if (pControlPF->blnDebugFile) {
      fprintf(pControlPF->pDebugFile, "Summary of total weight\n  ");
      // get the ESS and also write out to debug file
      dblESS = getESS(pControlPF, pParticles, intDummy, -1, True);
      // write ESS to meta
      pMeta[intYear].dblESS += dblESS;
    }

    //save the particles to file
    intParticlesSaved = saveParticles(pControlPF, pParticles,
      intDummy, -1, intYear, 
      True, True, True, True, False, False, False, 
      pControlPF->intNParticles, TEMPFILE);
    pMeta[intYear].intNParticlesSaved += intParticlesSaved;

    //Write out to the meta file
    writeMeta(pControlPF, pMeta, False, True);

  }
}

/*------------------------------------------------------------------------*/

void processResults (controlPFstruct * const pControlPF, metastruct * const pMeta,
                particlestruct * const pParticles)
/*
Purpose:
  Called after the filtering is complete
  Does rejection control or resampling
    on the final weights and then saves the selected
    particles to text or binary files, from which they can be easily read by e.g., R
  If it's the first iteration, the output files are created, otherwise these
    results are appended.
  It's easy writing the weights, parameter values and IDs, but to get the past
    state values (i.e., to smooth) have to thread pack through the ParentIDs
  Once it's done with the intermediate files, it deletes them
Inputs:
  pParticles contains final weights etc
Outputs:
  Files containing results, after RC
*/
{
  //An index array - holds an index to surviving particles
  Int *intIndex;
  Int intNIndex;
  double dblC;
  Int intYear, i, intParticle;
  //Index of weights for resampling
  double *dblWeights;
  double dblSumWeights;
  //Temp variable that holds whether to save states etc as compact for that year
  boolean blnCompact;

  if (pControlPF->blnDebugFile)
    fprintf(pControlPF->pDebugFile,HEADING(Final processing iteration %i),
      pMeta[pControlPF->intNYears].intIterations);

  //Initialize
  intIndex = ivector(1, pControlPF->intNParticles);

  switch(pControlPF->enumFinalYearDataReduction) {

    case(finalYearNone):
      //write out all the data
      intNIndex = pControlPF->intNParticles;
      for(i=1;i<=intNIndex;i++) intIndex[i]=i;
      break;

    case(finalYearRC):
      //call rejection control to get the index to selected particles
      if (pControlPF->dblFinalYearRCC == 0) {
        //user set the RC criterion in the control file to 0, so get the mean particle weight
        dblC = getC(pParticles, pControlPF->intNParticles);
      } else {
        //user did not use 0, so use the value they passed in
        dblC = pControlPF->dblFinalYearRCC;
      }
      if(pControlPF->blnDebugFile) {
        fprintf(pControlPF->pDebugFile, "Rejection control\n  RC criterion %f\n  Before ",dblC);
        getESS(pControlPF, pParticles, intIndex, -1, True);
      }
      rejectionControl(pParticles, pControlPF->intNParticles, intIndex, &intNIndex, dblC);
      if(pControlPF->blnDebugFile) {
        fprintf(pControlPF->pDebugFile, "  After ");
        getESS(pControlPF, pParticles, intIndex, intNIndex, True);
      }
      break;

    case(finalYearResample):
      //resample the particles
      //get the weights
      dblWeights=dvector(1, pControlPF->intNParticles);
      for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
        dblWeights[intParticle]=pParticles[intParticle].dblWeight;
      //get the sum of the weights - used in reweighting
      dblSumWeights=dsumVector(dblWeights, 1, pControlPF->intNParticles);
      //get the resample indexes
      intNIndex=pControlPF->intFinalYearNResamples;
      resample(intIndex, intNIndex, dblWeights, pControlPF->intNParticles,
         pControlPF->enumResamplingAlgorithm, &glngSeed);
      //re-weight - new weights are old weights divided by expected number
      // of resamples, the latter being old weights*num resamples/(sum old weights)
      // so that means the new weights are sum(old weights)/num resamples
      //Note - you only need to reweight particles that were sampled
      // as the others will get discarded
      dblSumWeights = dblSumWeights/intNIndex;
      for(intParticle=1;intParticle<=intNIndex;intParticle++)
        pParticles[intIndex[intParticle]].dblWeight = dblSumWeights;
      break;

    default:
      printf("Final year data reduction strategy not recognized");
      exit(1);
  }

  //write out the final year's results
  //last year is compact only if fully compact set
  blnCompact=(pControlPF->enumOutputCompact==fullyCompact);
  saveParticles(pControlPF, pParticles, intIndex, intNIndex, pControlPF->intNYears,
    pControlPF->blnOutputBinary, True, True, False, True,
    (pMeta[pControlPF->intNYears].intIterations==1) ? False : True, 
    blnCompact, 
    INT_MAX, (pControlPF->blnOutputBinary) ? "": TXTFILE);

  //Store the correct ParentIDs for the last year into intIndex
  for(i=1;i<=intNIndex;i++) {
    intIndex[i]=pParticles[intIndex[i]].intParent;
  }
  //Compact in this case if fully or partially compact
  blnCompact=(pControlPF->enumOutputCompact!=notCompact);
  for(intYear=pControlPF->intNYears-1; intYear>=1; intYear--) {
    //Load the previous year's states into oldStates and parentID into Parent
    loadParticlesBinary(pParticles, intIndex, intNIndex, intYear, 
      pControlPF->intNParticles, True, pControlPF->blnIntegrateOutPsi,
      pControlPF->strResultsPrefix, TEMPFILE);
    //write out the states
    saveParticles(pControlPF, pParticles, intIndex, -1, intYear,
      pControlPF->blnOutputBinary, False, False, False, False, 
      (pMeta[pControlPF->intNYears].intIterations==1) ? False : True, 
      blnCompact,
      intNIndex, (pControlPF->blnOutputBinary) ? "": TXTFILE);
    //Transfer the new ParentIDs into intIndex
    for(i=1;i<=intNIndex;i++) {
      intIndex[i]=pParticles[i].intParent;
    }
  }

  //delete the intermediate files
  removeParticleFiles(pControlPF, TEMPFILE);

  free_ivector(intIndex, 1, pControlPF->intNParticles);
}

/*------------------------------------------------------------------------*/

void filterParticles (Int intYear,
                       controlPFstruct * const pControlPF,
                       metastruct * const pMeta,
                       particlestruct * const pParticles)
/*
Purpose:
   Calls the appropriate algorithm to do particle filtering for one year,
    depending on pControlPF->enumTypeOfSampler.  If all observations are missing
    for the year then just projects the population forward
*/
{

  if(checkAllMissingObservations(intYear, pControlPF)) {
    //All observations missing, so just project the population forward to the next year
    if (pControlPF->blnDebugFile) {
      fprintf(pControlPF->pDebugFile, "No observations in this year, so projecting forward without filtering\n\n");
    }
    runPopulationProcess(intYear, True, pControlPF, pParticles);
  } else {
    //There are some observations, so choose the correct algorithm
    switch(pControlPF->enumTypeOfSampler) {
      case importance:   
        importanceSample(intYear, pControlPF, pMeta, pParticles);
        break;
      case APF:
        auxiliaryParticleFilter(intYear, pControlPF, pMeta, pParticles);
        break;
      case bootstrap:
        bootstrapFilter(intYear, pControlPF, pMeta, pParticles);
        break;
      default:
        printf("The controlPF file specifies a type of sampler that is not recognized by the filterParticles routine");
        exit(1);
    }
  }

}

/*------------------------------------------------------------------------*/

void importanceSample (Int intYear,
                       controlPFstruct * const pControlPF,
                       metastruct * const pMeta,
                       particlestruct * const pParticles)
/*
Purpose:
  Projects the particles forward from intYear-1 to intYear - i.e.,
    takes pParticles[].oldStates[][], uses the population dynamics model to fill
    in pParticles[].newStates[][], and updates the weights based on the observed
    pup counts in year intYear
*/
{

  //Counting var
  Int intParticle;
  //Array 1..NParticles to hold the weights in
  double *dblWeights;

  //run the population process forward
  runPopulationProcess(intYear, True, pControlPF, pParticles);

  //calculate the weights for the new states
  dblWeights=dvector(1, pControlPF->intNParticles);
  getWeights(intYear, dblWeights, pControlPF, pMeta, pParticles);

  //update the old weights with the new ones
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
      pParticles[intParticle].dblWeight*=dblWeights[intParticle];

  //free up memory used
  free_dvector(dblWeights, 1, pControlPF->intNParticles);

  if (pControlPF->blnScaleWeights) {
    //Scale the new particle weights so the mean is approx 1 (makes mean=1 on 1st iteration)
    scaleParticleWeights (intYear, pControlPF, pMeta, pParticles);
  }

}

/*------------------------------------------------------------------------*/

void auxiliaryParticleFilter (Int intYear,
                       controlPFstruct * const pControlPF,
                       metastruct * const pMeta,
                       particlestruct * const pParticles)
/*
Purpose:
  Uses the auxiliary particle filter to project the particles forward from intYear-1 to intYear 
*/
{

  //Counting var
  Int intParticle;
  //Array 1..NParticles to hold the weights in
  double *dblWeights1, *dblWeights2;
  //Vector showing which particles get resampled
  Int *intResampleIndex;

  //---  First (auxiliary) part (deterministic) ---
  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Part 1 (deterministic part) of APF\n");
  }

  //Run the population process forward deterministically 
  // to get expected pup counts
  runPopulationProcess(intYear, False, pControlPF, pParticles);

  //Get weights for these expected pup counts, given observations
  dblWeights1=dvector(1, pControlPF->intNParticles);
  getWeights(intYear, dblWeights1, pControlPF, pMeta, pParticles);

  //Multiply by the weights from the previous year
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
      dblWeights1[intParticle]*=pParticles[intParticle].dblWeight;

  //Take some power of the weights, if required
  if(!(pControlPF->dblResamplingPower==1)) {
    for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
        dblWeights1[intParticle]=pow(dblWeights1[intParticle],pControlPF->dblResamplingPower);
  }

  //Standardize these weights so they sum to N.particles
  // this is then the expected number of resample copies that will be 
  // made of each particle - and is the appropriate divisor for the 
  // final weights, below
  dstandardizeVectorToOut(dblWeights1, 1, pControlPF->intNParticles, 
    dblWeights1, (double) pControlPF->intNParticles);
  
  //Get the resample index
  intResampleIndex = ivector(1, pControlPF->intNParticles);
  resample(intResampleIndex, pControlPF->intNParticles, dblWeights1, 
     pControlPF->intNParticles, pControlPF->enumResamplingAlgorithm, &glngSeed);

  //Resample the particles appropriately, doing kernal smoothing on 
  // the parameter values
  sampleAndKSParticles(intResampleIndex, pControlPF, pParticles);

  //---  Second part (stochastic) ---
  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Part 2 (stochastic part) of APF\n");
  }

  //Run the population process forward stochastically
  runPopulationProcess(intYear, True, pControlPF, pParticles);

  //Get new set of weights
  dblWeights2=dvector(1, pControlPF->intNParticles);
  getWeights(intYear, dblWeights2, pControlPF, pMeta, pParticles);

  //Final weights are these weights divided by the primary weights
  //Note that the primary weights vector dblWeights1 was created before
  // resampling so to get the entry for particle x after resampling 
  // you need to look up intResampleIndex[x]
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
      pParticles[intParticle].dblWeight=dblWeights2[intParticle]/
        dblWeights1[intResampleIndex[intParticle]];
  
  //free up memory used
  free_dvector(dblWeights1, 1, pControlPF->intNParticles);
  free_dvector(dblWeights2, 1, pControlPF->intNParticles);
  free_ivector(intResampleIndex, 1, pControlPF->intNParticles);

  if (pControlPF->blnScaleWeights) {
    //Scale the new particle weights so the mean is approx 1 (makes mean=1 on 1st iteration)
    scaleParticleWeights (intYear, pControlPF, pMeta, pParticles);
  }

}

/*------------------------------------------------------------------------*/

void bootstrapFilter (Int intYear,
                      controlPFstruct * const pControlPF,
                      metastruct * const pMeta,
                      particlestruct * const pParticles)
/*
Purpose:
  Uses the bootstrap filter to project the particles forward from intYear-1 to intYear 
Implementation note 25/7/07:
  It would be easy to make this routine more efficient - weights are moved around when 
    they need not be (especially if resampling is according to the weights rather than
    some power of the weights), and the sampleParticles routine moves stuff like the old states
    when it need not.  To do at some point!...
*/
{

  //Counting var
  Int intParticle;
  //Array 1..NParticles to hold the weights in
  double *dblWeights;
  //Vector showing which particles get resampled
  Int *intResampleIndex;

  //Create storage space
  dblWeights=dvector(1, pControlPF->intNParticles);
  intResampleIndex = ivector(1, pControlPF->intNParticles);

  //run the population process forward
  runPopulationProcess(intYear, True, pControlPF, pParticles);

  //calculate the weights for the new states
  getWeights(intYear, dblWeights, pControlPF, pMeta, pParticles);

  //update the old weights with the new ones
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
      pParticles[intParticle].dblWeight*=dblWeights[intParticle];

  //Skip the resampling if its the last year - then you do whatever
  // processResults says
  if(intYear<pControlPF->intNYears) {

    //Store updated weights back into dblWeights, but take some power if required
    // to make the resampling probs
    if((pControlPF->dblResamplingPower==1)) {
      for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
          dblWeights[intParticle]=pParticles[intParticle].dblWeight;
    } else {
      for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
          dblWeights[intParticle]=pow(pParticles[intParticle].dblWeight,
            pControlPF->dblResamplingPower);
    }

    //Standardize these weights so they sum to N.particles
    // this is then the expected number of resample copies that will be 
    // made of each particle - and is the appropriate divisor for the 
    // final weights, below
    dstandardizeVectorToOut(dblWeights, 1, pControlPF->intNParticles, 
      dblWeights, (double) pControlPF->intNParticles);

    //Get the resample indices
    resample(intResampleIndex, pControlPF->intNParticles, dblWeights, 
       pControlPF->intNParticles, pControlPF->enumResamplingAlgorithm, &glngSeed);

    //Do the resampling - with no kernel smoothing
    sampleParticles(intResampleIndex, pControlPF, pParticles);

    //Work out the final weights -- correct the current weights for the 
    // resampling by dividing by the expected number of copies of that
    // particle.  If the power for the resampling probs is 1, then this
    // will set all the weights to 1, so do so automatically, to save
    // time
    if((pControlPF->dblResamplingPower==1)) {
      for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
        pParticles[intParticle].dblWeight=1.0;
    } else {
      for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
          pParticles[intParticle].dblWeight=pParticles[intParticle].dblWeight/
            dblWeights[intResampleIndex[intParticle]];
    }

  }

  //free up memory used
  free_ivector(intResampleIndex, 1, pControlPF->intNParticles);
  free_dvector(dblWeights, 1, pControlPF->intNParticles);

  if (pControlPF->blnScaleWeights) {
    //Scale the new particle weights so the mean is approx 1 (makes mean=1 on 1st iteration)
    scaleParticleWeights (intYear, pControlPF, pMeta, pParticles);
  }

}

/*------------------------------------------------------------------------*/

void resample(Int * const intResampleIndex, 
              Int intNResamples, 
              const double * const dblWeights, 
              Int intNWeights,
              enum resamplingAlgorithm enumResamplingAlgorithm,
              long *plngSeed)
/* Purpose: Returns a resample of size intNResamples into intResampleIndex
   according to dblWeights (length inNWeights) and using enumResamplingAlgorithm
*/

//Will call routines like residualResample etc etc
{

  switch (enumResamplingAlgorithm) {
    case randomResampling: 
      randomResample(intResampleIndex, intNResamples, 
        dblWeights, intNWeights, plngSeed);
      return;
    case residualResampling: 
      residualResample(intResampleIndex, intNResamples, 
        dblWeights, intNWeights, plngSeed);
      return;
    default: 
      printf("Resampling algorithm not recognized");
      exit(1);
  }

}

/*------------------------------------------------------------------------*/

void randomResample(Int * const intResampleIndex, 
              Int intNResamples, 
              const double * const dblWeights, 
              Int intNWeights,
              long *plngSeed)
/* Purpose: Returns a random resample of intNResamples values from
   intNWeights into intResampleIndex
   according to dblWeights
*/
{

  //Temp vector holds the standardized vals
  double *dblProbs;
  
  //Standardize the weights so they sum to 1
  dblProbs=dvector(1, intNWeights);
  dstandardizeVectorToOut(dblWeights, 1, intNWeights, dblProbs, 1.0);

  //Resample from them with prob proportional to dblProbs
  alias(intNWeights, intNResamples, dblProbs, intResampleIndex, plngSeed);

  free_dvector(dblProbs, 1, intNWeights);
}

/*------------------------------------------------------------------------*/

void residualResample(Int * const intResampleIndex, 
              Int intNResamples, 
              const double * const dblWeights, 
              Int intNWeights,
              long *plngSeed)
/* Purpose: Returns a redisual resample of intNResamples values from
   intNWeights into intResampleIndex
   according to dblWeights
*/

{

  Int intNSure; // Number of particles that will be sampled for sure
  Int intNRemaining; //Number left after sure sampling
  Int i,j; //Counting vars
  Int intPosition, intNewPosition; //Used in filling in the sure samples
  double *dblTempWeights; //Temp vector holds weights and associated stuff
  Int *intSureSample; //Vector that holds number of sure samples

  //Dimension stuff
  dblTempWeights=dvector(1, intNWeights);
  intSureSample=ivector(1, intNWeights); 

  //Standardize the weights so they sum to 1
  dstandardizeVectorToOut(dblWeights, 1, intNWeights, dblTempWeights, 1.0);

  // Determine the observations that will be sampled for sure
  intNSure = 0;
  for(i=1;i<=intNWeights;i++) {
    intSureSample[i] = (Int) floor(dblTempWeights[i]*(double) intNResamples);
    intNSure += intSureSample[i];
  }

  // Calculate the unscaled residual weights
  for(i=1;i<=intNWeights;i++) 
    dblTempWeights[i] = dblTempWeights[i]-(double)intSureSample[i]/(double)intNWeights;

  // and number remaining to randomly sample
  intNRemaining = intNResamples-intNSure;

  // Scale the residual weights so they sum to 1
  dstandardizeVectorToOut(dblTempWeights, 1, intNWeights, dblTempWeights, 1.0);

  //Resample from them with prob proportional to dblProbs
  alias(intNWeights, intNRemaining, dblTempWeights, intResampleIndex, plngSeed);

  //Now, the first NRemaining positions of intResampleIndex contain randomly
  // sampled observations using dblTempWeights, so now we want to fill in the
  // remaining intNSure observations
  if(intNSure>0){
    intPosition = intNRemaining+1;
    for(i=1;i<=intNWeights;i++) {
      intNewPosition = intPosition+intSureSample[i];
      for(j=intPosition;j<intNewPosition;j++) {
         intResampleIndex[j]=i;
      }
      intPosition = intNewPosition;
    }
  }

   //Free memory
  free_dvector(dblTempWeights, 1, intNWeights);
  free_ivector(intSureSample, 1, intNWeights);

}

/*------------------------------------------------------------------------*/

void sampleAndKSParticles(Int * const intResampleIndex, 
                          controlPFstruct * const pControlPF,
                          particlestruct * const pParticles)
/* Purpose: given the list of samples in intResampleIndex, and stuff in controlPFstruct
  like the discount factor (or could get that out myself!), 
  it reorders the values in pParticles and does kernal smoothing on the params
*/
{

  //Uncomment the line below if you want KS debugging output
  //#define DEBUG_KS

  // Matrix that holds parameters for kernal smoothing
  // 1 .. NParticles, 1 .. NModelParams (ie parameters in this model)
  double **dblThetaStar;
  // Vector of mean parameter vals
  double *dblMeanThetaStar;
  // Matrix of "shrunk" means
  double **dblMeanMat;
  // Variance-covariance matrix
  double **dblSigmaThetaStar;
  //A value - (3.0*KSdisount-1)/(2.0*KSdiscount)
  // h_sq = 1.0-a_wt*a_wt;
  double dblAWt, dblHSq;
  //Holds the weights for passing in to covariance matrix func 1.. NParticles
  double *dblWeights;
  Int intParticle; //counting var

  dblAWt = (3.0*pControlPF->dblKernelSmoothingDiscount-1.0)/
    (2.0*pControlPF->dblKernelSmoothingDiscount);
  dblHSq = 1.0-dblAWt*dblAWt;

  //Reserve space
  dblThetaStar=dmatrix(1, pControlPF->intNParticles, 1, pControlPF->intNModelParams);  
  dblMeanThetaStar=dvector(1, pControlPF->intNModelParams);  
  dblMeanMat=dmatrix(1, pControlPF->intNParticles, 1, pControlPF->intNModelParams);  
  dblSigmaThetaStar=dmatrix(1, pControlPF->intNModelParams, 1, pControlPF->intNModelParams);  
  dblWeights=dvector(1, pControlPF->intNParticles);

  //Pack params from pParticles to dblThetaStar and transform
  packThetaStar(pControlPF, pParticles, dblThetaStar);

  //Pack up weights from pParticles to a vector - note that we are using
  // the weights from the previous year, not those from the auxiliary
  // particle step here
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
      dblWeights[intParticle]=pParticles[intParticle].dblWeight;

  //Calculate the variance covariance matrix
  //Note - you don't use the ThetaStar that comes out of here - rather you
  // use the dblMeanMat - so you could return meanmat as thetastar - but
  // that might be confusing.  Another alternative (I think) is to 
  // put the meanmat shrinkage stuff into the kernal smooth routine -- but
  // then you need to be careful about following the resampling
  wt_covMat(pControlPF->intNModelParams, pControlPF->intNParticles, 
    dblAWt, dblThetaStar, dblMeanThetaStar, dblMeanMat, dblSigmaThetaStar,
    dblWeights);
  #ifdef DEBUG_KS
    writeCovMat(pControlPF, pParticles, dblAWt, dblThetaStar, 
      dblMeanThetaStar, dblMeanMat, dblSigmaThetaStar);
  #endif

  //sample the particles pointed to by intResampleIndex
  sampleParticles1(intResampleIndex, pControlPF, pParticles, dblMeanMat);

  //Kernel Smooth
  kernelSmooth(dblHSq, pControlPF->intNParticles, pControlPF->intNModelParams,
    dblSigmaThetaStar, dblMeanMat, dblThetaStar);

  //Unpack params from dblThetaStar to pParticles and back-transform
  unpackThetaStar(pControlPF, pParticles, dblThetaStar);

  //free space
  free_dmatrix(dblThetaStar, 1, pControlPF->intNParticles, 1, pControlPF->intNModelParams);
  free_dvector(dblMeanThetaStar, 1, pControlPF->intNModelParams);
  free_dmatrix(dblMeanMat, 1, pControlPF->intNParticles, 1, pControlPF->intNModelParams);
  free_dmatrix(dblSigmaThetaStar, 1, pControlPF->intNModelParams, 1, pControlPF->intNModelParams);
  free_dvector(dblWeights, 1, pControlPF->intNParticles);

  #ifdef DEBUG_KS
    #undef DEBUG_KS
  #endif
}

/*------------------------------------------------------------------------*/

void packThetaStar(controlPFstruct * const pControlPF,
                   particlestruct * const pParticles,
                   double ** const dblThetaStar)
//Takes the parameters for this model out of pParticles, transforms them
// and saves them to dblThetaStar (1 .. NParticles, 1 .. NModelParams)
//For parameters drawn from the scaled beta prior, the transformation
// involves a rescale before the logit transform
{

  //counting vars
  Int i, intParam, intParticle;
  //temp storage
  Int *intParamNumberInTheta;
  double dblTemp;
  particlestruct *pParticle;

  //Get parameter indexes == position of parameter in ThetaStar
  intParamNumberInTheta=ivector(1, pControlPF->intNModelParams);
  i=1;
  for(intParam=0;intParam<NPARAMS;intParam++) {
    if(pControlPF->intParamNumber[intParam]>0){
      intParamNumberInTheta[i]=intParam;
      i++;
    }
  }  
  
  //Go through each particle in turn, transforming and loading up ThetaStar
  for(intParticle=1; intParticle<=pControlPF->intNParticles; intParticle++) {
    pParticle=&pParticles[intParticle];
    for(intParam=1; intParam <= pControlPF->intNModelParams; intParam++) {
      //transform
      if(cblnParamIsLogit[intParamNumberInTheta[intParam]]) {
        dblTemp = pParticle->dblParams[intParamNumberInTheta[intParam]];
        //Check for scaled beta
        if(pControlPF->enumPriorDists[intParamNumberInTheta[intParam]]==SCALEDBETA){
          //Need to scale the variable before the logit, so it has [0,1] scale
          dblTemp = (dblTemp - (1-pControlPF->dblPriorParams3[intParamNumberInTheta[intParam]]-
            pControlPF->dblPriorParams4[intParamNumberInTheta[intParam]]))/
            pControlPF->dblPriorParams3[intParamNumberInTheta[intParam]];
        } 
        //logit transform
        dblTemp = log(dblTemp/(1.0-dblTemp));
      } else {
        //log transform
        dblTemp = log(pParticle->dblParams[intParamNumberInTheta[intParam]]);
      }
      //save
      dblThetaStar[intParticle][intParam]=dblTemp;
    }
  }

  free_ivector(intParamNumberInTheta, 1, pControlPF->intNModelParams);
}

/*------------------------------------------------------------------------*/

void sampleParticles(Int * const intResampleIndex, 
                     controlPFstruct * const pControlPF,
                     particlestruct * const pParticles)
/*Purpose: Given the index intResampleIndex, samples
    all the parts of pParticles and puts them in the right place
  Called by bootstrapSample routine
  Note - ignoring Tau pups as well here - assuming I won't be using 
   this algorithm with the Tau models
*/
{

  //Counting vars
  Int intParticle, intAge, intColony, intParam;
  //Used to store the contents of one part of the particles while stuff is
  //swapped round
  Int *intTempVector;
  Int **intTempStateMatrix; 
  double *dblTempVector;
  double **dblTempParamMatrix;

  //ID
  intTempVector=ivector(1, pControlPF->intNParticles);
  //Copy
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    intTempVector[intParticle]=pParticles[intParticle].intID;
  }
  //Reassign
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    pParticles[intParticle].intID=intTempVector[intResampleIndex[intParticle]];
  }

  //Parent
  //Copy
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    intTempVector[intParticle]=pParticles[intParticle].intParent;
  }
  //Reassign
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    pParticles[intParticle].intParent=intTempVector[intResampleIndex[intParticle]];
  }
  free_ivector(intTempVector,1, pControlPF->intNParticles);

  //States
  intTempStateMatrix=imatrix(1, pControlPF->intNParticles,1, NCOLONIES);
  //Old states
  for(intAge = 1; intAge<=NAGES; intAge++) {
    //Copy
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      for (intColony = 1; intColony<=NCOLONIES; intColony ++) {
        intTempStateMatrix[intParticle][intColony]=
          pParticles[intParticle].intOldStates[intColony][intAge];
      }
    }
    //Reassign
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      for (intColony = 1; intColony<=NCOLONIES; intColony ++) {
        pParticles[intParticle].intOldStates[intColony][intAge]=
          intTempStateMatrix[intResampleIndex[intParticle]][intColony];
      }
    }
  }  
  //New states
  for(intAge = 1; intAge<=NAGES; intAge++) {
    //Copy
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      for (intColony = 1; intColony<=NCOLONIES; intColony ++) {
        intTempStateMatrix[intParticle][intColony]=
          pParticles[intParticle].intNewStates[intColony][intAge];
      }
    }
    //Reassign
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      for (intColony = 1; intColony<=NCOLONIES; intColony ++) {
        pParticles[intParticle].intNewStates[intColony][intAge]=
          intTempStateMatrix[intResampleIndex[intParticle]][intColony];
      }
    }
  }  
  free_imatrix(intTempStateMatrix,1, pControlPF->intNParticles,1, NCOLONIES);

  //Params
  dblTempParamMatrix=dmatrix(1, pControlPF->intNParticles,1, pControlPF->intNModelParams);
  //Copy
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    for (intParam = 1; intParam<=pControlPF->intNModelParams; intParam ++) {
      dblTempParamMatrix[intParticle][intParam]=pParticles[intParticle].dblParams[intParam-1];
    }
  }
  //Reassign
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    for (intParam = 1; intParam<=pControlPF->intNModelParams; intParam ++) {
      pParticles[intParticle].dblParams[intParam-1]=
        dblTempParamMatrix[intResampleIndex[intParticle]][intParam];
    }
  }
  free_dmatrix(dblTempParamMatrix, 1, pControlPF->intNParticles,1, pControlPF->intNModelParams);

  //Weights
  dblTempVector=dvector(1, pControlPF->intNParticles);
  //Copy
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    dblTempVector[intParticle]=pParticles[intParticle].dblWeight;
  }
  //Reassign
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    pParticles[intParticle].dblWeight=dblTempVector[intResampleIndex[intParticle]];
  }
  free_dvector(dblTempVector, 1, pControlPF->intNParticles);

  //If you're integrating out the observation parameters, then move the Bs too
  if(pControlPF->blnIntegrateOutPsi) {
    dblTempVector=dvector(1, pControlPF->intNParticles);
    //BPrev
    //Copy
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      dblTempVector[intParticle]=pParticles[intParticle].dblBPrev;
    }
    //Reassign
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      pParticles[intParticle].dblBPrev=dblTempVector[intResampleIndex[intParticle]];
    }
    //BCurrent
    //Copy
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      dblTempVector[intParticle]=pParticles[intParticle].dblBCurrent;
    }
    //Reassign
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      pParticles[intParticle].dblBCurrent=dblTempVector[intResampleIndex[intParticle]];
    }
    free_dvector(dblTempVector, 1, pControlPF->intNParticles);
  }

}

/*------------------------------------------------------------------------*/

void sampleParticles1(Int * const intResampleIndex, 
                     controlPFstruct * const pControlPF,
                     particlestruct * const pParticles,
                     double **dblMeanMat)
/*Purpose: Given the index intResampleIndex, samples
    all the parts of pParticles and puts them in the right place - apart
    from the params, which are taken from dblThetaStar
    and weights, which do not need to be swapped
    and the new states, which also do not need to be swapped
  Called by sampleAndKSParams routine
  Note - ignoring Tau pups as well here - assumign I won't be using 
   this algorithm with the Tau models
*/
{

  //Counting vars
  Int intParticle, intAge, intColony, intParam;
  //Used to store the contents of one part of the particles while stuff is
  //swapped round
  Int *intTempVector;
  Int **intTempStateMatrix; 
  double *dblTempVector;
  double **dblTempParamMatrix;

  //ID
  intTempVector=ivector(1, pControlPF->intNParticles);
  //Copy
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    intTempVector[intParticle]=pParticles[intParticle].intID;
  }
  //Reassign
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    pParticles[intParticle].intID=intTempVector[intResampleIndex[intParticle]];
  }

  //Parent
  //Copy
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    intTempVector[intParticle]=pParticles[intParticle].intParent;
  }
  //Reassign
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    pParticles[intParticle].intParent=intTempVector[intResampleIndex[intParticle]];
  }
  free_ivector(intTempVector,1, pControlPF->intNParticles);

  //Old states
  intTempStateMatrix=imatrix(1, pControlPF->intNParticles,1, NCOLONIES);
  for(intAge = 1; intAge<=NAGES; intAge++) {
    //Copy
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      for (intColony = 1; intColony<=NCOLONIES; intColony ++) {
        intTempStateMatrix[intParticle][intColony]=
          pParticles[intParticle].intOldStates[intColony][intAge];
      }
    }
    //Reassign
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      for (intColony = 1; intColony<=NCOLONIES; intColony ++) {
        pParticles[intParticle].intOldStates[intColony][intAge]=
          intTempStateMatrix[intResampleIndex[intParticle]][intColony];
      }
    }
  }  
  free_imatrix(intTempStateMatrix,1, pControlPF->intNParticles,1, NCOLONIES);

  //Params in dblMeanMat
  dblTempParamMatrix=dmatrix(1, pControlPF->intNParticles,1, pControlPF->intNModelParams);
  //Copy
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    for (intParam = 1; intParam<=pControlPF->intNModelParams; intParam ++) {
      dblTempParamMatrix[intParticle][intParam]=dblMeanMat[intParticle][intParam];
    }
  }
  //Reassign
  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
    for (intParam = 1; intParam<=pControlPF->intNModelParams; intParam ++) {
      dblMeanMat[intParticle][intParam]=
        dblTempParamMatrix[intResampleIndex[intParticle]][intParam];
    }
  }
  free_dmatrix(dblTempParamMatrix, 1, pControlPF->intNParticles,1, pControlPF->intNModelParams);

  //If you're integrating out the observation parameters, then move the Bs too
  if(pControlPF->blnIntegrateOutPsi) {
    dblTempVector=dvector(1, pControlPF->intNParticles);
    //BPrev
    //Copy
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      dblTempVector[intParticle]=pParticles[intParticle].dblBPrev;
    }
    //Reassign
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      pParticles[intParticle].dblBPrev=dblTempVector[intResampleIndex[intParticle]];
    }
    //BCurrent
    //Copy
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      dblTempVector[intParticle]=pParticles[intParticle].dblBCurrent;
    }
    //Reassign
    for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle ++){
      pParticles[intParticle].dblBCurrent=dblTempVector[intResampleIndex[intParticle]];
    }
    free_dvector(dblTempVector, 1, pControlPF->intNParticles);
  }

}

/*------------------------------------------------------------------------*/
void kernelSmooth(double dblHSq, Int intNParticles, Int intNParams, 
  double **dblSigmaThetaStar,
  double **dblMeanMat, double **dblThetaStar)
/*Purpose: Performs kernal smoothing of params, returns results in dblThetaStar
    Inputs: 
      dblHSq is the H parameter
      intNParticles is number of particles
      intNParams is number of parameters in this model
      dblSigmaThetaStar is a 1..params, 1..params var cov matrix
      dblMeanMat is the shrunk matrix of param values 1..particles, 1..params
*/
{

  double **dblWTemp;
  double *dblDiag, *dblZVec, *dblNoiseVec;
  Int i, j, k;

  dblWTemp = dmatrix(1, intNParams, 1, intNParams);
  dblDiag = dvector(1, intNParams);
  dblZVec = dvector(1, intNParams);
  dblNoiseVec = dvector(1, intNParams);

  for(i = 1; i<=intNParams;i++) 
    for(j=1;j<=intNParams;j++)
      dblWTemp[i][j]=dblHSq*dblSigmaThetaStar[i][j];

  choldc(dblWTemp, intNParams, dblDiag);

  for(i=1; i<=intNParams; i++)
    dblWTemp[i][i] = dblDiag[i];

  for(i=1; i<=intNParticles; i++) {
    for(j=1; j<=intNParams; j++) {
      dblZVec[j] = drnorm(0, 1, &glngSeed);
      dblNoiseVec [j] = 0;
      for(k=1; k<=j; k++) 
        dblNoiseVec[j] += dblWTemp[j][k]*dblZVec[k];
      dblThetaStar[i][j] = dblMeanMat[i][j]+dblNoiseVec[j];
    }
  }

  free_dmatrix(dblWTemp, 1, intNParams, 1, intNParams);
  free_dvector(dblDiag, 1, intNParams);
  free_dvector(dblZVec, 1, intNParams);
  free_dvector(dblNoiseVec, 1, intNParams);


}

/*------------------------------------------------------------------------*/

void unpackThetaStar(controlPFstruct * const pControlPF,
                   particlestruct * const pParticles,
                   double ** const dblThetaStar)
//Takes the parameters for this model out of dblThetaStar (1 .. NParticles, 1 .. NModelParams)
// back-transforms them, and saves them to pParticles
{

  //counting vars
  Int i, intParam, intParticle;
  //temp storage
  Int *intParamNumberInTheta;
  double dblTemp;
  particlestruct *pParticle;

  //Get parameter indexes == position of parameter in ThetaStar
  intParamNumberInTheta=ivector(1, pControlPF->intNModelParams);
  i=1;
  for(intParam=0;intParam<NPARAMS;intParam++) {
    if(pControlPF->intParamNumber[intParam]>0){
      intParamNumberInTheta[i]=intParam;
      i++;
    }
  }  
  
  //Go through each row of ThetaStar, backtransform and load into the
  // appropriate pParticle
  for(intParticle=1; intParticle<=pControlPF->intNParticles; intParticle++) {
    pParticle=&pParticles[intParticle];
    for(intParam=1; intParam <= pControlPF->intNModelParams; intParam++) {
      //back-transform
      if(cblnParamIsLogit[intParamNumberInTheta[intParam]]) {
        //inverse logit
        dblTemp = exp(dblThetaStar[intParticle][intParam]);
        dblTemp = dblTemp/(1.0+dblTemp);
        //Check for scaled beta
        if(pControlPF->enumPriorDists[intParamNumberInTheta[intParam]]==SCALEDBETA){
          //Need to scale the variable so it has the appropriate [(1-param3-param4),(1-param4)] scale
          dblTemp = (dblTemp*pControlPF->dblPriorParams3[intParamNumberInTheta[intParam]]) + 
            (1-pControlPF->dblPriorParams3[intParamNumberInTheta[intParam]]-
            pControlPF->dblPriorParams4[intParamNumberInTheta[intParam]]);
        } 
      } else {
        //exponentiate
        dblTemp = exp(dblThetaStar[intParticle][intParam]);
      }
      //save
      pParticle->dblParams[intParamNumberInTheta[intParam]]=dblTemp;
    }
  }

  free_ivector(intParamNumberInTheta, 1, pControlPF->intNModelParams);
}

/*------------------------------------------------------------------------*/

double getC(particlestruct * const pParticles, Int intNParticles)
/*
Purpose: Gets the rejection control criterion -- currently just the mean
  of the weights
*/
{
  double dblC;
  Int i;

  dblC = 0.0;
  for (i=1; i<= intNParticles; i++)
    dblC+=pParticles[i].dblWeight/(double)intNParticles;
  return(dblC);
}

/*------------------------------------------------------------------------*/

void rejectionControl(particlestruct * const pParticles, Int intNParticles,
                     Int * const intIndex, Int * const pintNIndex,
                     double dblC)
/*
Purpose:
  Implements the rejection control algorithm of Liu 2001, page 44
  Inputs:
    pParticles contains the particles [1..intNParticles] to do RC on.
    intNParticles is the number of weights
    intIndex is an empty vector [1..intNWeights]
    pintNIndex is a pointer to an empty integer
    dlbC is the rejection control criterion
  Outputs:
    intIndex contains the numbers of the particles that passed rejection control
    pintNIndex contains the number of these particles
    dblWeights contains the new weights, as some of the particles that passed will have had their weights
      adjusted
*/
{
  Int i;

  //temp var
  particlestruct *pParticle;

  *pintNIndex = 0;
  for(i=1; i<=intNParticles; i++) {
    pParticle = &pParticles[i];
    if(pParticle->dblWeight>=dblC) {
      //Accept the particle
      *pintNIndex = *pintNIndex + 1;
      intIndex[*pintNIndex]=i;
    } else {
      //Decide whether to accept the particle
      if(drunif(0,dblC,&glngSeed)<=pParticle->dblWeight) {
        //Accept, so update the weight to C
        pParticle->dblWeight = dblC;
        *pintNIndex = *pintNIndex + 1;
        intIndex[*pintNIndex]=i;
      }
    }
  }

}


/*------------------------------------------------------------------------*/

void scaleParticleWeights (Int intYear,
                           controlPFstruct * const pControlPF,
                           metastruct * const pMeta,
                           particlestruct * const pParticles)
/*
Purpose:
  Scales the weights in pParticles by dividing them by pMeta[intYear].dblWeightScaling3
  If the value in dblWeightScaling3 is zero or less then works out the value to use
    (this happens on the first iteration) as the mean of the particle weights (so setting
    the weights to 1.0 on the first iteration and approx 1.0 on the other iterations).
*/
{
  Int intParticle, intNParticles;
  double dblMean=0.0;

  intNParticles = pControlPF->intNParticles;

  //if scaling not yet set
  if(pMeta[intYear].dblWeightScaling3 == (double) MISSINGWEIGHT) {
    //work out the mean weight
    for (intParticle = 1; intParticle<=intNParticles; intParticle ++) {
      dblMean += pParticles[intParticle].dblWeight/(double)intNParticles;
    }
    //set it as the scaling
    pMeta[intYear].dblWeightScaling3 = (dblMean>0.0) ? dblMean : 1.0;
  }

  //do the scaling
  for(intParticle=1;intParticle<=intNParticles;intParticle++)
      pParticles[intParticle].dblWeight /= pMeta[intYear].dblWeightScaling3;

}
