/*
  global.h - Global header file
  Contains constants and data structures used throughout the program
*/

#define VERSION 28 // Version of SealPF -- increment when writing a new version
#define NPARAMS 12 // Number of parameters
#define NAGES 7//46 // Number of ages of seal 7 in UK and Canada
#define NCOLONIES 4 //Number of breeding colonies -- if you change this then also need
                    // to change NPARAMS, strParamNames,
                    // cblnParamIsLogit, paramNames
#define NGPARAMS NPARAMS-NCOLONIES //Number of parameters that are not beta params
#define PFEMALE 0.5
#define TAU 6 //Number of years before density dependence kicks in for model 7
              //Also, age at first breeding for model 12 (6, typically)

//TAU_N commented out - dens dep is (n_0,t-1 - n_0,tau-1)
//TAU_N not commented out but TAU_USE_Y commented out - dens dep is (n_0,t-1 - TAU_N)
//TAU_N not commented out and TAU_USE_Y not commented out - dens dep is (y_0,t-1 - TAU_N)
//#define TAU_N 1313
//#define TAU_USE_Y

#define STABLE_AGE_YEARS 50 // Number of years to project forward to get stable age structure

// Maximum length of strings
#define MAXSTR 128
// 1st column of data in the control file
#define CONTROL_FILE_INPUT_COLUMN 40
// Default name for file containing control info (can be changed by inputting a name on the command line
#define DEFAULT_CONTROL_FILE "ControlPF.txt"
//Used to produce headings in the debug file
#define HEADING(x) "\n***********************************************************\n" #x "\n***********************************************************\n\n"

//See note in main.c for what this is for
//#define SQRTPSI
#define PSI_IS_PRECISION

#define MISSINGWEIGHT -1.0
#define VERY_SMALL_NUMBER 1E-100
#define LARGE_NUMBER 10000000 //Used when a large number is needed!

//All integers here are declared as Int, so by having typedef long Int I can
// turn all integers into longs; by having typedef int Int I can keep them
// as integers.  Note - I need to be able to set some as negative so don't use
// unsigned integers!
typedef int Int;

//Add some rudimentary logical support!
typedef enum {False, True} boolean;
//time.h needed for the controlPF struct, and is often not included in the .c files
#include <time.h>

//Names of the parameters
static const char *cstrParamNames[NPARAMS] = {"sa","sjmax","amax","psi",
  "movedd","movedist","movesf","rho","beta1","beta2","beta3","beta4"};
//True if logit transform when kernal smooth; false if log transform
static const boolean cblnParamIsLogit[NPARAMS] = {True, True, True, False,
  False, False, False, False, False, False, False};
enum paramNames {sa, sjmax, amax, psi,
  movedd, movedist, movesf, rho, beta1, beta2, beta3, beta4};

//Names of prior distributions
enum priorDists {FIXED,UNIFORM,GAMMA,BETA,SCALEDBETA,CARRYINGCAPACITY};

//used in saving results
#define TEMPFILE ".tmp"
#define TXTFILE ".txt"

/* Models:
 0 nodd no density dependence
 1 ddsurv1 density dependent juv survival (Beverton holt), no movement
 2 ddfec1 density dependent fecundity (Beverton holt), no movement
 3 ddsurv2 density dependent juv survival (Beverton holt), with movement according to movementModel
 4 ddfec2 density dependent fecundity (Beverton holt), with movement according to movementModel
 5 ddsurv3 density dependent juv survival (extended Beverton holt), with movement according to movementModel
 6 ddfec3 density dependent fecundity (extended Beverton holt), with movement according to movementModel
 7 ddsurvTakis density dependent juv survival (Beverton holt) - but only after year Tau, no movement
 8 ddemigrTakis density dependent emmigration (adult survival - Beverton Holt) - but only after year Tau, no movement
     Note - this is used for single colony or region-level analysis - so emmigration means to other colonies or regions
 9 ddsurvDFO denstiy dependent juv survival (Beverton holt), females 4+ breed, with a 2 parameter breeding curve,
     and restricted movement
10 ddsurv4 density dependent juv survival (extended BH), no movement
11 ddfec4 density dependent fecundity (extended BH), no movment
12 ddsurvsen density dependent juv survival (extended BH), no movement, adult senescence according to Gompertz-Makeham 
     mortality function, where the alpha in that function is the movedd parameter and the beta (which I label delta 
	 in my notes) is the movedist parameter (I'm just re-using parameters to save memory)
*/
#define NMODELS 13
enum model {nodd,ddsurv1,ddfec1,ddsurv2,ddfec2,ddsurv3,ddfec3,ddsurvTakis,ddemigrTakis,ddsurvDFO,ddsurv4,ddfec4,ddsurvsen};
static const boolean cblnMovementInModel[NMODELS] = {False, False, False, True, True, True, True,False, False, True, False, False, False};
static const boolean cblnDDSurvInModel[NMODELS] = {False, True, False, True, False, True, False, True, False, True, True, False, True};
static const boolean cblnDDFecInModel[NMODELS] = {False, False, True, False, True, False, True, False, False, False, False, True, False};
static const boolean cblnDDEmigrInModel[NMODELS] = {False, False, False, False, False, False, False, False, True, False, False, False, False};
static const boolean cblnExtendedDDInModel[NMODELS] = {False, False, False, False, False, True, True, False, False, False, True, True, True};
static const boolean cblnTauInModel[NMODELS] = {False, False, False, False, False, False, False, True, True, False, False, False, False};
static const boolean cblnSenescenceInModel[NMODELS] = {False, False, False, False, False, False, False, False, False, False, False, False, True};

/* Movement models:
 0 model used in ANZJS paper
 1 model used in scos03, 04 and 05 briefing papers
 2 model suggested by Carmen Fernandez for the SSM2 paper
 3 model used for 2007 DFO analysis (same as 2 but without distance param)
*/
enum movementModel {anzjs,scos03,carmen,dfo};

/* Type of sampler used to update the particles within each year:
  0 simple importance sampling
  1 auxiliary particle filter (1 year look-ahead) with kernal smoothing
  2 bootstrap filter
*/
enum typeOfSampler {importance, APF, bootstrap};

/* Resampling algorithm:
  0 simple random
  1 residual resampling
*/
enum resamplingAlgorithm {randomResampling, residualResampling}; //stratified to follow

/* Initialization methods:
 0 method used in ANZJS paper
 1 Takis' stable age start method
 2 Sample particles from a file of priors
 3 init using survival - like 0, but uses survival for last age instead of fecundity
     designed for use only with model 12
*/
enum initializationMethod {anzjsInit,takisInit,fromFile,survivalInit};

/* Final year data reduction method:
 0 none
 1 rejection control
 2 resampling
*/
enum finalYearDataReduction {finalYearNone, finalYearRC, finalYearResample};

/* Harvest strategy for projecting forward:
 0 no harvest
 1 proportion of population harvested
 2 fixed numbers to be harvested (i.e., quotas), read from file
 3 fixed specificed total, plus specification of what proportion are pups
     (other age groups are harvested according to their year strengths)
*/
enum harvestStrategy {harvestNone, harvestProp, harvestFixed, harvestFixedTot};


/* Whether and how to compact results
  0 not compact
  1 fully compact (sum all ages except first and last, store integer values as int and floats as float)
  2 compact for all except the last year
*/
enum outputCompact {notCompact, fullyCompact, partlyCompact};

/* Defines the structure used to store information about each particle */
struct {
  Int intID; // Unique ID of particle
  Int intParent; // Points to position of the parent particle in the previous years file
  double dblWeight;
  Int **intOldStates; //1 .. colonies x 1 .. ages
  Int **intNewStates; //1 .. colonies x 1 .. ages
  Int *intTauPups; //1 .. colonies - used for Takis' tau models - stores the
                   // estimated pup numbers in year tau-1
  double *dblParams; // 0 ... NPARAMS - 1 *** Note - base 0, not 1!! ***
  double dblBPrev; //Used when integrating out observation error
  double dblBCurrent; //ditto
} typedef particlestruct;

/* Defines the structure used to store control info and input data */
struct  {
  /* What to do */
  boolean blnFitModel;
  /* About the data */
  Int intNYears;
  Int intFirstYear;
  boolean blnKnownSEs;
  double dblSEInflationFactor;
  /* About the algorithm */
  Int intNParticles;
  Int intNIterations;
  boolean blnRCYr1;
  Int intRCYr1MaxNIterations;
  enum typeOfSampler enumTypeOfSampler;
  double dblKernelSmoothingDiscount;
  enum resamplingAlgorithm enumResamplingAlgorithm;
  double dblResamplingPower;
  boolean blnScaleWeights;
  enum finalYearDataReduction enumFinalYearDataReduction;
  double dblFinalYearRCC;
  Int intFinalYearNResamples;
  /* About the model */
  enum model enumModel;
  enum movementModel enumMovementModel;
  Int intNModelParams;
  Int intParamNumber[NPARAMS];
  enum initializationMethod enumInitializationMethod;
  boolean blnStochasticInitialization;
  Int intAdultAgeThatMoves;
  double dblStateInflate;
  boolean blnStochasticPopulationProcess;
  boolean blnIntegrateOutPsi;
  /* Output and misc */
  long lngSeed;
  boolean blnEcho;
  boolean blnDebugFile;
  char strResultsPrefix[MAXSTR];
  boolean blnOverwriteExisting;
  boolean blnOutputBinary;
  enum outputCompact enumOutputCompact;
  char strDebugFile[MAXSTR];
  FILE *pDebugFile;
  /* Priors */
  boolean blnReadPriorsFromFile;
  char strPriorFilePrefix[MAXSTR];
  Int intMaxPriorParticlesFromFile;
  enum priorDists enumPriorDists[NPARAMS];
  double dblPriorParams1[NPARAMS];
  double dblPriorParams2[NPARAMS];
  double dblPriorParams3[NPARAMS]; /* Some distributions have 3 params */
  double dblPriorParams4[NPARAMS]; /* Some distributions have 4 params */
  
  /* Data */
  /* Matrix of pup counts: 1..NYears x 1..NColonies */
  Int **intPupCounts;
  /* Matrix of pup count SEs: 1..NYears x 1..NColonies */
  Int **intPupCountSE;
  /* Matrix of pup removals: 1..NYears x 1..NColonies */
  Int **intPupRemovals;
  /* Matrix of juv removals: 1..NYears x 1..NColonies */
  Int **int1PlusRemovals;
  /* Matrix of adult removals: 1..NYears x 1..NColonies */
  Int **intAllAgeRemovals;
  /* Square matrix of distances between colonies: 1..Ncolonies x 1..NColonies */
  double **dblColonyDistances;

  /* Misc */
  time_t timeStart;
  time_t timeFinish;
  /* Harvest */
  enum harvestStrategy enumHarvestStrategy;
  double dblHarvestPropMale;
  double dblHarvestPropPop;
  double dblHarvestPropPup;
  double dblHarvestProp1Plus;
  double dblHarvestPropAllAge;
  Int intHarvestTotal;

} typedef controlPFstruct ;

/* Defines the structure used to store information about the results */
struct {
  //These are all vectors that run from 1..NYears of data
  Int intYear;
  Int intIterations;
  Int intTotalParticles;
  Int intNParticlesSaved; //after rejection control
  double dblESS;
  double dblWeightScaling1; //used to scale year weights on log scale -- set first time getWeights is called
  double dblWeightScaling2; //used to scale year weights after exp -- set first time getWeights is called
  double dblWeightScaling3; //used to scale the overall weight -- set in scaleParticleWeights
} typedef metastruct;

