/*
  main.c - Main file for SealPF program

  Seal population dynamics model estimation function
  7 seal stages - pups, age 1, ..., age 6+
  Can fit models with density dependent survival and movement, density independent productivity -
    see global.h for a list of possible models
  Estimation by various particle filtering algorithms

  Author: Len Thomas, CREEM, University of St Andrews

  Program structure:
  - main - contains main sub and some housekeeping stuff (read the input files, etc)
  - filter - contains the particle filtering subroutines
  - model - contains the pop dynamics modelling subroutines, including
      sampling from the prior, etc
  - numeric - contains numerical utility functions, written by LJT, KBN or
      Numerical Recipes
  - nrutil - contains numerical recipe utility functions (dimension arrays, etc)
  - util - contains misc utility functions
  - global.h - contains the global consts, data structure definitions, etc.

  History:
  - Versions 1-15.5 as sis.seal in SPlus by KBN and LJT Aug 2001-July 2004
  - Version 16 as sealPF in C by LJT using some code by KBN and some Numerical Recipes
  - Version 17 adds support for density dependent fecundity
  - Version 18 adds the C. Fernandez movement model from the StateSpace2 paper Aug 25 2005
               minor fixes to gamma and beta generation functions Sept 2005
  - Version 19 adds known harvest, known observation SE on some data and other
               enhancements to enable it to run with Canadian grey seal data.  Jan-March 06
               adds ability to define psi as a precision or variance parameter
  - Version 20 adds more features of use in Canadian grey seal data.
               adds auxiliary particle filter and kernel smoothing (for year 3 onwards)
               July 2006.
  - Version 21 additional work for 2007 Canadian grey seal contract. Jan-March 2007.
  - Version 22 adds ability to use ANZJS-type start when no ddf or dds. May 2007.
  - Version 23 adds weighted kernel smoothing; APF where resampling step is a power
               of the likelihood weight; a more compact output.
               Version used for SCOS 2007. June 2007.
  - Version 24 Used for DFO 2007 seals analysis.  Sept-Dec 2007.
  - Version 25 Adds ability to read in binary output files, and project population
               forward into the future. Jan-March 2008.
               Added FixedTotal harvest strategy Nov 2008.
  - Version 26 Used for DFO2010 seal analysis.  Nov 2010.
               Changed harvest files to Pup, 1plus and AllAge.  The latter two are non-selective
			         in that they harvest animals proportional to their frequency in the population.
  - Version 27 adds scaled beta priors.  Used in this form for SCOS2012. June 2012.
  - Version 28 (August 2015):
     - amends scaled beta priors so they can have 0 support above the scaled beta as well as below;
	 - adds model 12, allowing for adult survival senescence.

  ToDo:
  - Make it so that if not cblnTauInModel we don't create the intTauPups for each particle
  - For deterministic projection forward, it would be nice if integers weren't used for
      population states, but doubles instead.  For the APF all we really want returned in the
      look-ahead stage is the number of pups, so that means creating an dblNonIntegerPups
      vector for each particle, and then re-engineering runPopulationProcess so that it can
      work with doubles for each population stage as well as integers.  Or could store the new
      population numbers as doubles rather than integers in general.  Only copy them to integers
      when moving new numbers to old numbers -- that might be easier?

  Inputs:
  - file ControlPF.txt containing control info, unless passed in as the first argument
      on the command line
  - file (name set in ControlPF) containing observations of pup counts
  - file (name set in ControlPF) containing distances between colonies

  Notes/Wish list:
  - Currently number of parameters is hard-coded in - would be good to make more general so
    that colonies could be added without recompiling code (as with SPlus code)
  - Currently there is one random number seed value and it is stored as a global variable
    (for performance reasons).  This is not ideal for two reasons: (1) global variables are
    evil; (2) more importantly
    it will not work when parallelizing each particle to a separate thread.  So, in the future
    will be better to move to storing seed within the particle structure, so having one
    per particle.  The one in controlPF would just be the initial one.  Haven't done that
    yet because it means having one more long value per particle, so more memory required.
  - Currently uses the same storage for parameters whether using a simple or a complex model
    (ie stores all parameters whether being used or not).  Clearly, room to improve efficiency
    here.
  - To increase efficiency, I transform the psi parameter to sqrt(psi) right after generating
    it[, and then back-transform just before saving the final results - not currently doing this bit!].
	This saves taking
    sqrt(psi) loads of times throughout the code - but it does mean if you query the
    value of psi as the code is running it will look odd!  This behaviour can be turned off
    by commenting out the %define SQRTPSI in global.h
  - if PSI_IS_PRECISION is defined in global.h then psi is a precision parameter, if not it's
    a variance(CV) parameter.  Would be better to make this an option in ControlPF.
*/

//prepro.h contains definitions required for Microsoft compiler
#include "prepro.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
//global.h contains the global constants and data structure definitions
#include "global.h"
#include "util.h"
#include "nrutil.h"
#include "filter.h"
#include "project.h"
#include "model.h"
//main.h contains definitions of the subs in main.c - so that the subs
// can be written here out of order to increase readability (e.g., main sub first)
#include "main.h"

//This is the only global variable - see implementation note, above
long glngSeed;

/*------------------------------------------------------------------------*/

int main (int argc, char *argv[])
/* Main program */
{
  /* This is the variable that contains information used to control the PF algorithm
     and information about the model, such as priors, data, etc */
  controlPFstruct controlPF;
  /* This is the variable that contains information about the results, such as total
     number of iterations, etc */
  metastruct *pMeta=0;
  Int intIteration;

  /* Save the start time */
  time(&controlPF.timeStart);

  /* Read the input files into the control structure */
  readInputFiles(&controlPF,argc,argv);

  /* Initialize the output stuff */
  initDatabase(&controlPF, &pMeta);

  if(controlPF.blnFitModel) {
    /* Run the particle filter intNIterations times */
    for(intIteration=1;intIteration<=controlPF.intNIterations; intIteration++) {
      if(controlPF.blnEcho) {
        printf("Running particle filter iteration %i of %i\n",
          intIteration,controlPF.intNIterations);
      }
      if(controlPF.blnDebugFile) {
        fprintf(controlPF.pDebugFile,HEADING(Running particle filter iteration %i of %i),
          intIteration,controlPF.intNIterations);
      }
      doPF(&controlPF, pMeta);
    }
  } else {
    /* Run the forward simulation */
    doProjection(&controlPF);
  }

  /* Save the end time */
  time(&controlPF.timeFinish);

  /* Shut down the output stuff */
  uninitDatabase(&controlPF, &pMeta);

  return(0);
}

/*------------------------------------------------------------------------*/

void readInputFiles (controlPFstruct * const pControlPF, int argc, char * const argv[])
/*
  Purpose: Reads the control, pup count and distance files
  Inputs:
    pControlPF - pointer to the controlPF structure
    argc - number of command line arguments used when starting the program
    argv[] - pointer to command line arguments
  */
{

  char strControlFileName[MAXSTR];
  char strPupCountFileName[MAXSTR];
  char strColonyDistanceFileName[MAXSTR];
  char strPupCountSEFileName[MAXSTR];
  char strPupRemovalsFileName[MAXSTR];
  char str1PlusRemovalsFileName[MAXSTR];
  char strAllAgeRemovalsFileName[MAXSTR];
  Int intStartAt; // First line of the pup count file to start reading at
  boolean blnSkipFirstColumn; //If true first column of pup count file is the year

  /* Get control file name*/
  if(argc == 1) {
    /* No command line arguments passed in, so use default name for control file */
    strcpy(strControlFileName, DEFAULT_CONTROL_FILE);
  } else {
    /* First command line argument is the control file */
    strcpy(strControlFileName, argv[1]);
  }

  /* Read it in to pControlPF */
  readControlFile(pControlPF, strControlFileName, strPupCountFileName,
    strColonyDistanceFileName, strPupCountSEFileName, strPupRemovalsFileName,
    str1PlusRemovalsFileName, strAllAgeRemovalsFileName, &intStartAt,
    &blnSkipFirstColumn);

  /* Read in the data files */

  if(pControlPF->blnFitModel) {
    pControlPF->intPupCounts = getCountFileToMatrix(pControlPF, strPupCountFileName,
      intStartAt, blnSkipFirstColumn);
    if (pControlPF->blnKnownSEs)
      pControlPF->intPupCountSE = getCountFileToMatrix(pControlPF, strPupCountSEFileName,
        intStartAt, blnSkipFirstColumn);
  } else {
    //Need to initialize the pup count matrix in any case, as it's used later
    // to pass around within population projection functions
    pControlPF->intPupCounts = imatrix(1,pControlPF->intNYears,1,NCOLONIES);
  }
  if(pControlPF->enumHarvestStrategy==harvestFixed) {
    pControlPF->intPupRemovals = getCountFileToMatrix(pControlPF, strPupRemovalsFileName,
      intStartAt, blnSkipFirstColumn);
    pControlPF->int1PlusRemovals = getCountFileToMatrix(pControlPF, str1PlusRemovalsFileName,
      intStartAt, blnSkipFirstColumn);
    pControlPF->intAllAgeRemovals = getCountFileToMatrix(pControlPF, strAllAgeRemovalsFileName,
      intStartAt, blnSkipFirstColumn);
  }

  /* Read in the distance file */
  if(strlen(strColonyDistanceFileName)>0) {
    readColonyDistanceFile(pControlPF, strColonyDistanceFileName);
  } else {
    //OK for dfo movement model - otherwise throw an error
    if (!(pControlPF->enumMovementModel==dfo)) {
      printf("Colony distance file required for the specified movement model, and file name not provided.\n");
      exit(1);
    }
  }

}

/*------------------------------------------------------------------------*/

void readControlFile (controlPFstruct * const pControlPF,
                      const char * const strControlFileName,
                      char * const strPupCountFileName,
                      char * const strColonyDistanceFileName,
                      char * const strPupCountSEFileName,
                      char * const strPupRemovalsFileName,
                      char * const str1PlusRemovalsFileName,
                      char * const strAllAgeRemovalsFileName,
                      Int * const intStartAt,
                      boolean * const blnSkipFirstColumn)
/* Purpose: Reads the contents of the control file at strControlFileName
     Uses this to initialize controlPFstruct.  Also sets the strPupCountFileName,
     strColonyDistanceFileName, strPupCountSEFileName, strPupRemovalFileName,
     strJuvRemovalFileName and strAdultRemovalFileName,
     intStartAt, which is the year to start reading pup count data at, and
     blnSkipFirstColumn which says if the pup count and other files first column
       is the year
*/
{

  FILE *pControlFile;
  char strTemp[MAXSTR];
  Int i;
  boolean blnTemp;

  /* Open control file*/
  pControlFile = fopen(strControlFileName,"r");
  if (pControlFile == NULL) {
      printf("Can't open control file '%s'\n",strControlFileName);
      exit(1);
  }

  /*Skip first 3 lines */
  SkipLines(pControlFile,3);

  /* What to do */
  SkipLines(pControlFile, 2);
  pControlPF->blnFitModel = GetBooleanFromControlFile(pControlFile);
  /* Check the next line is the opposite */
  blnTemp = GetBooleanFromControlFile(pControlFile);
  if (pControlPF->blnFitModel&blnTemp) {
    printf("You cannot fit model and simulate forward in the same run - do as two separate runs\n");
    exit(1);
  }

  if(pControlPF->blnFitModel) {

    //This code is relevant to fitting models with a particle filter

    /* About the data */
    SkipLines(pControlFile,2);
    CopyStringFromControlFile(pControlFile, strPupCountFileName);
    pControlPF->intFirstYear = GetIntFromControlFile(pControlFile);
    *intStartAt = GetIntFromControlFile(pControlFile) - pControlPF->intFirstYear + 1;
    pControlPF->intNYears = GetIntFromControlFile(pControlFile);
    *blnSkipFirstColumn = GetBooleanFromControlFile(pControlFile);
    CopyStringFromControlFile(pControlFile, strColonyDistanceFileName);
    CopyStringFromControlFile(pControlFile, strPupCountSEFileName);
    pControlPF->blnKnownSEs = (strlen(strPupCountSEFileName)>0);
    CopyStringFromControlFile(pControlFile, strPupRemovalsFileName);
    CopyStringFromControlFile(pControlFile, str1PlusRemovalsFileName);
    CopyStringFromControlFile(pControlFile, strAllAgeRemovalsFileName);
    pControlPF->dblHarvestPropMale = GetDoubleFromControlFile(pControlFile);
    //if there is removal data, then treat as fixed harvest strategy
    if ((strlen(strPupRemovalsFileName)>0) &
        (strlen(str1PlusRemovalsFileName)>0) &
        (strlen(strAllAgeRemovalsFileName)>0)) {
      pControlPF->enumHarvestStrategy = harvestFixed;
    } else {
      //otherwise no harvest
      pControlPF->enumHarvestStrategy = harvestNone;
    }

    /* About the algorithm */
    SkipLines(pControlFile,2);
    pControlPF->intNParticles = GetIntFromControlFile(pControlFile);
    pControlPF->intNIterations = GetIntFromControlFile(pControlFile);
    pControlPF->blnRCYr1 = GetBooleanFromControlFile(pControlFile);
    pControlPF->intRCYr1MaxNIterations = GetIntFromControlFile(pControlFile);
    pControlPF->enumTypeOfSampler = GetIntFromControlFile(pControlFile);
    pControlPF->dblKernelSmoothingDiscount = GetDoubleFromControlFile(pControlFile);
    pControlPF->enumResamplingAlgorithm = GetIntFromControlFile(pControlFile);
    pControlPF->dblResamplingPower = GetDoubleFromControlFile(pControlFile);
    pControlPF->blnScaleWeights = GetBooleanFromControlFile(pControlFile);
    pControlPF->enumFinalYearDataReduction = GetIntFromControlFile(pControlFile);
    pControlPF->dblFinalYearRCC = GetDoubleFromControlFile(pControlFile);
    pControlPF->intFinalYearNResamples = GetIntFromControlFile(pControlFile);
    //Translate into number to take each year
    pControlPF->intFinalYearNResamples =
      (Int) floor((double)pControlPF->intFinalYearNResamples /
                  (double)pControlPF->intNIterations);
    //Max end resamples is min of the above number and NParticles
    if(pControlPF->intNParticles<pControlPF->intFinalYearNResamples)
      pControlPF->intFinalYearNResamples = pControlPF->intNParticles;

    /* About the model*/
    SkipLines(pControlFile,2);
    pControlPF->enumModel = GetIntFromControlFile(pControlFile);
    pControlPF->enumMovementModel = GetIntFromControlFile(pControlFile);
    pControlPF->intAdultAgeThatMoves = GetIntFromControlFile(pControlFile);
    pControlPF->enumInitializationMethod = GetIntFromControlFile(pControlFile);
    pControlPF->blnStochasticInitialization = GetBooleanFromControlFile(pControlFile);
    pControlPF->dblStateInflate = GetDoubleFromControlFile(pControlFile);
    pControlPF->blnStochasticPopulationProcess = GetBooleanFromControlFile(pControlFile);
    pControlPF->blnIntegrateOutPsi = GetBooleanFromControlFile(pControlFile);
    pControlPF->dblSEInflationFactor = GetDoubleFromControlFile(pControlFile);
    if ((!pControlPF->blnKnownSEs)&(!(pControlPF->dblSEInflationFactor==1.0)))  {
      printf("Pup count SE variance inflation factor will be ignored as SE file not specified.\n");
    }

    /* Output and misc */
    SkipLines(pControlFile,2);
    pControlPF->lngSeed = GetLongFromControlFile(pControlFile);
    pControlPF->blnEcho = GetBooleanFromControlFile(pControlFile);
    pControlPF->blnDebugFile = GetBooleanFromControlFile(pControlFile);
    CopyStringFromControlFile(pControlFile, pControlPF->strResultsPrefix);
    pControlPF->blnOverwriteExisting = GetBooleanFromControlFile(pControlFile);
    pControlPF->blnOutputBinary = GetBooleanFromControlFile(pControlFile);
    pControlPF->enumOutputCompact = GetIntFromControlFile(pControlFile);
    CopyStringFromControlFile(pControlFile, pControlPF->strDebugFile);
    //Add the prefix to to Debug File
    strcpy(strTemp, pControlPF->strResultsPrefix);
    strcat(strTemp, pControlPF->strDebugFile);
    strcpy(pControlPF->strDebugFile, strTemp);

    /* Priors */
    SkipLines(pControlFile,2);
    /* Priors from file - this part is in the file in any case */
    CopyStringFromControlFile(pControlFile, pControlPF->strPriorFilePrefix);
    pControlPF->intMaxPriorParticlesFromFile = GetIntFromControlFile(pControlFile);

    /* Priors from parameters */
    if(!(pControlPF->enumInitializationMethod==fromFile)) {
      //Not reading prior particles from file, so read in prior parameter info
      SkipLines(pControlFile, 2);
      for (i=0;i<NPARAMS;i++) {
        SkipLines(pControlFile,1);
        CopyStringFromControlFile(pControlFile, strTemp);
        pControlPF->enumPriorDists[i]=getPriorDistEnum(strTemp);
        pControlPF->dblPriorParams1[i]=GetDoubleFromControlFile(pControlFile);
        pControlPF->dblPriorParams2[i]=GetDoubleFromControlFile(pControlFile);
        if(pControlPF->enumPriorDists[i]==SCALEDBETA){
          /* Has third and fourth parameters */
          pControlPF->dblPriorParams3[i]=GetDoubleFromControlFile(pControlFile);
          pControlPF->dblPriorParams4[i]=GetDoubleFromControlFile(pControlFile);
		  //Check the sum of them is less than 1!
		  if(pControlPF->dblPriorParams3[i]+pControlPF->dblPriorParams4[i]>1.0) {
            printf("Scaled beta specified, but the sum of the third and forth parameters is more than 1.\n");
            exit(1);
		  }
        }
      }
      /* Check you're using a gamma prior on observation error if you're
         trying to integrate it out */
      if (pControlPF->blnIntegrateOutPsi) {
        if (!(pControlPF->enumPriorDists[psi]==GAMMA))  {
          printf("You asked to integrate out the observation process, but did not specify a gamma prior on psi.\n");
          exit(1);
        }
#if defined SQRTPSI
        // if you define SQRTPSI put out a warning
        printf("Having SQRTPSI on doesn't make sense if you're integrating out the observation process; final parameter estiamtes are not sqrt transformed.\n");
#endif
#ifndef PSI_IS_PRECISION
        // if you define SQRTPSI put out a warning
        printf("Psi must be the precision parameter if you're integrating out the observation process.\n");
        exit(1);
#endif

        //Also check you're using the ANZJS init method
        if (!pControlPF->enumInitializationMethod==anzjsInit) {
            printf("Cannot currently integrate out observation process with anything other than the ANZJS initialization method.\n");
            exit(1);
        }
      }
    } //End of stuff to do with reading prior param info

  } else {

    //This code is relevant to projecting forward from prior files
    SkipLines(pControlFile,2);
    pControlPF->intNYears = GetIntFromControlFile(pControlFile);
    pControlPF->intNParticles = GetIntFromControlFile(pControlFile);

    /* About the model*/
    SkipLines(pControlFile,2);
    pControlPF->enumModel = GetIntFromControlFile(pControlFile);
    pControlPF->enumMovementModel = GetIntFromControlFile(pControlFile);
    pControlPF->intAdultAgeThatMoves = GetIntFromControlFile(pControlFile);
    CopyStringFromControlFile(pControlFile, strColonyDistanceFileName);
    //Init method set automatically
    pControlPF->enumInitializationMethod = fromFile;
    pControlPF->blnStochasticPopulationProcess = GetBooleanFromControlFile(pControlFile);

    /* Harvest strategy */
    SkipLines(pControlFile,2);
    pControlPF->enumHarvestStrategy = GetIntFromControlFile(pControlFile);
    if (pControlPF->enumHarvestStrategy==harvestProp)  {
      printf("The proportional harvest strategy has not yet been implemented in code.\n");
      exit(1);
    }
    pControlPF->dblHarvestPropMale=GetDoubleFromControlFile(pControlFile);
    //Proportional strategy
    SkipLines(pControlFile,1);
    pControlPF->dblHarvestPropPop=GetDoubleFromControlFile(pControlFile);
    pControlPF->dblHarvestPropAllAge=GetDoubleFromControlFile(pControlFile);
    pControlPF->dblHarvestPropPup=GetDoubleFromControlFile(pControlFile);
    pControlPF->dblHarvestProp1Plus=1-pControlPF->dblHarvestPropPup;
    if(pControlPF->enumHarvestStrategy == harvestProp) {
      //check the proportions add up etc
      if ((pControlPF->dblHarvestPropPop>1)|(pControlPF->dblHarvestPropPop<0)) {
          printf("Harvest proportion must be between 0 and 1.\n");
          exit(1);
      }
      if ((pControlPF->dblHarvestPropAllAge>1)|(pControlPF->dblHarvestPropAllAge<0)) {
          printf("Harvest proportions for all age animals must be between 0 and 1.\n");
          exit(1);
      }
      if ((pControlPF->dblHarvestPropPup>1)|(pControlPF->dblHarvestPropPup<0)) {
          printf("Harvest proportions for pups must be between 0 and 1.\n");
          exit(1);
      }
    }
    //Fixed strategy
    SkipLines(pControlFile, 1);
    *intStartAt = 1;
    *blnSkipFirstColumn = GetBooleanFromControlFile(pControlFile);
    CopyStringFromControlFile(pControlFile, strPupRemovalsFileName);
    CopyStringFromControlFile(pControlFile, str1PlusRemovalsFileName);
    CopyStringFromControlFile(pControlFile, strAllAgeRemovalsFileName);
    //Fixed total strategy
	SkipLines(pControlFile, 1);
    if(pControlPF->enumHarvestStrategy == harvestFixedTot) {
      pControlPF->intHarvestTotal = GetIntFromControlFile(pControlFile);
      pControlPF->dblHarvestPropPup = GetDoubleFromControlFile(pControlFile);
      pControlPF->dblHarvestProp1Plus = 1-pControlPF->dblHarvestPropPup;
      pControlPF->dblHarvestPropAllAge = 0;
	} else {
      //Dont' read in if not fixed tot strategy as otherwise you overwrite proppup and prop1plus
	  SkipLines(pControlFile, 2);
	}

    /* Output and misc */
    SkipLines(pControlFile,2);
    pControlPF->lngSeed = GetLongFromControlFile(pControlFile);
    pControlPF->blnEcho = GetBooleanFromControlFile(pControlFile);
    pControlPF->blnDebugFile = GetBooleanFromControlFile(pControlFile);
    CopyStringFromControlFile(pControlFile, pControlPF->strResultsPrefix);
    pControlPF->blnOverwriteExisting = GetBooleanFromControlFile(pControlFile);
    pControlPF->blnOutputBinary = GetBooleanFromControlFile(pControlFile);
    pControlPF->enumOutputCompact = GetIntFromControlFile(pControlFile);
    CopyStringFromControlFile(pControlFile, pControlPF->strDebugFile);
    //Add the prefix to to Debug File
    strcpy(strTemp, pControlPF->strResultsPrefix);
    strcat(strTemp, pControlPF->strDebugFile);
    strcpy(pControlPF->strDebugFile, strTemp);

    /* Priors from file - this part is in the file in any case */
    SkipLines(pControlFile, 2);
    CopyStringFromControlFile(pControlFile, pControlPF->strPriorFilePrefix);
    pControlPF->enumResamplingAlgorithm = GetIntFromControlFile(pControlFile);
    pControlPF->intMaxPriorParticlesFromFile = GetIntFromControlFile(pControlFile);

  }

  fclose(pControlFile);
}

/*------------------------------------------------------------------------*/

enum priorDists getPriorDistEnum(char *strPriorDist)
/* Purpose: Takes distribution for prior on parameter as a string and returns
     it as an enumeration. Called by readControlFile */
{
  switch (strPriorDist[0]) {
    case 'f': return FIXED;
    case 'u': return UNIFORM;
    case 'g': return GAMMA;
    case 'b': return BETA;
    case 's': return SCALEDBETA;
    case 'c': return CARRYINGCAPACITY;
    default: printf("Unknown prior distribution\n"); exit(1);
  }
}

/*------------------------------------------------------------------------*/

int **getCountFileToMatrix (controlPFstruct * const pControlPF,
                      char * strCountFileName,
                      const Int intStartAt,
                      const boolean blnSkipFirstColumn)
/* Purpose: Creates and returns a pointer to a matrix of size
     1..NColonies x 1..NYears.  The matrix is filled with the
     contents of strCountFileName
   Inputs:
     pControlPF is a pointer to a var of type controlPFstruct
     strCountFileName is the name of the file containing pup counts
       This file has an arbitrary first line (this line is skipped)
       Each subsequent line should have NCOLONIES Int values in it
     intCounts is a pointer to the array of counts to be filled
     intStartAt - this is the line to start at (+1 for 1st line skipped)
     blnSkipFirstColumn - if true then skip the first column
   Note - stops at line intStartAt+pControlPF->intNYears
   Note - the format of the matrix is with colonies as the rows and years as
     the columns, while the format of the file is to have years as rows and colonies
     as columns
*/
{

  FILE *pPupCountFile;
  Int i;
  Int j;
  Int skip;
  Int **intCountMatrix;

  /* Open pup count file file*/
  pPupCountFile = fopen(strCountFileName,"r");
  if (pPupCountFile == NULL) {
      printf("Can't open file '%s'\n",strCountFileName);
      exit(1);
  }

  /* Initialize count data matrix */
  intCountMatrix = imatrix(1,pControlPF->intNYears,1,NCOLONIES);

  /* Skip first intStartAt lines */
  SkipLines(pPupCountFile,intStartAt);
  for(i=1;i<=pControlPF->intNYears;i++) {
    /* Optionally, skip the number in the first column */
    if (blnSkipFirstColumn) fscanf(pPupCountFile,"%i",&skip);
    /* Read all NCOLONIES columns after this */
    for (j=1;j<=NCOLONIES;j++) {
      fscanf(pPupCountFile,"%i",&intCountMatrix[i][j]);
    }
  }

  return intCountMatrix;
}

/*------------------------------------------------------------------------*/

void readColonyDistanceFile (controlPFstruct * const pControlPF,
                             char * strColonyDistanceFileName)
/* Purpose: Reads the colony distance data from strColonyDistanceFileName
     into the matrix controlPFstruct->dblColonyDistances, which it
     also initializes as 1..NColonies x 1..NColonies
     Skips the first line (which can contain the colony names)
*/
{

  FILE *pColonyDistanceFile;
  Int i;
  Int j;

  /* Open colony distance file*/
  pColonyDistanceFile = fopen(strColonyDistanceFileName,"r");
  if (pColonyDistanceFile == NULL) {
      printf("Can't open colony distance file '%s'\n",strColonyDistanceFileName);
      exit(1);
  }

  /* Initialize colony distance data matrix */
  pControlPF->dblColonyDistances = dmatrix(1,NCOLONIES,1,NCOLONIES);

  /* Skip first line */
  SkipLines(pColonyDistanceFile,1);
  for(i=1;i<=NCOLONIES;i++) {
    for (j=1;j<=NCOLONIES;j++) {
      fscanf(pColonyDistanceFile,"%lf",&pControlPF->dblColonyDistances[i][j]);
    }
  }
}

/*------------------------------------------------------------------------*/

void initDatabase (controlPFstruct * const pControlPF, metastruct **pMeta)
/* Initializes the output stuff, and does other misc housekeeping tasks */
/* For the moment that is just initializing meta and opening the debug results file */
{
  Int i;

  if(pControlPF->blnFitModel) {
    //Initialize Meta
    *pMeta = metavector(1, pControlPF ->intNYears);
    for(i=1;i<=pControlPF->intNYears;i++) {
      (*pMeta)[i].intYear=pControlPF->intFirstYear+i-1;
      (*pMeta)[i].intIterations=0;
      (*pMeta)[i].intTotalParticles=0;
      (*pMeta)[i].intNParticlesSaved=0;
      (*pMeta)[i].dblESS=0.0;
      (*pMeta)[i].dblWeightScaling1=(double) MISSINGWEIGHT;
      (*pMeta)[i].dblWeightScaling2=(double) MISSINGWEIGHT;
      (*pMeta)[i].dblWeightScaling3=(double) MISSINGWEIGHT;
    }
    if (!(pControlPF->blnOverwriteExisting)) {
      //Read meta values from file, if the file exists - if not, you've already
      // initialized the values of meta to null values so no problem
      readMeta(pControlPF, *pMeta);
    }
  }

  /* Set the number of parameters for this model and parameter numbers */
  initializeParamNumber(pControlPF);

  //Open the debug file
  if (pControlPF->blnDebugFile) {
    pControlPF->pDebugFile = fopen(pControlPF->strDebugFile,
      (pControlPF->blnOverwriteExisting) ? "wt" : "a+t");
    if (pControlPF->pDebugFile == NULL) {
        printf("Can't open debug output file '%s'\n",pControlPF->strDebugFile);
        exit(1);
    }
  } else {
      pControlPF->pDebugFile = NULL;
  }

  /* Print a startup message */
  if (pControlPF->blnEcho) {
    printf("Running SealPF version %i, compiled on %s at %s\n",VERSION,__DATE__, __TIME__);
    printf("Program started %s",ctime(&pControlPF->timeStart));
  }
  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile,
      "Running SealPF version %i, compiled on %s at %s\n",VERSION,__DATE__, __TIME__);
    fprintf(pControlPF->pDebugFile,
      "Program started %s\n",ctime(&pControlPF->timeStart));
  }

  //Check the random seed
  if (pControlPF->lngSeed >= 0) {
    //If a non negative value entered, set according to system clock
    glngSeed = (long) -time(0);
    if (pControlPF->blnDebugFile)
      fprintf(pControlPF->pDebugFile,
        "Random number seed set from system clock, to %ld\n",glngSeed);
  } else {
    //Else set as requested
    glngSeed = pControlPF->lngSeed;
    if (pControlPF->blnDebugFile)
      fprintf(pControlPF->pDebugFile,
        "Random number seed set from command file, to %ld\n",glngSeed);
  }

}

/*------------------------------------------------------------------------*/

void uninitDatabase (controlPFstruct * const pControlPF, metastruct **pMeta)
/* Shuts down the output stuff, and does other misc housekeeping tasks */
{

  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, HEADING(Finished));
  }

  if(pControlPF->blnFitModel) {
    writeMeta(pControlPF, *pMeta, True, False);
    free_metavector(*pMeta, 1, pControlPF->intNYears);
  }

  //Free up data matricies
  if(pControlPF->blnFitModel) {
    free_imatrix(pControlPF->intPupCounts,1,pControlPF->intNYears,1,NCOLONIES);
    if (pControlPF->blnKnownSEs)
      free_imatrix(pControlPF->intPupCountSE,1,pControlPF->intNYears,1,NCOLONIES);
  }
  if(pControlPF->enumHarvestStrategy==harvestFixed) {
    free_imatrix(pControlPF->intPupRemovals,1,pControlPF->intNYears,1,NCOLONIES);
    free_imatrix(pControlPF->int1PlusRemovals,1,pControlPF->intNYears,1,NCOLONIES);
    free_imatrix(pControlPF->intAllAgeRemovals,1,pControlPF->intNYears,1,NCOLONIES);
  }

  //Write program terminating messages and close debug file
  if (pControlPF->blnEcho) {
    printf("Program terminated at %s", ctime(&pControlPF->timeFinish));
    printf("Execution time %s\n", cdifftime(pControlPF->timeFinish, pControlPF->timeStart));
  }
  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile,
      "\nProgram terminated at %sExecution time %s\n",
      ctime(&pControlPF->timeFinish),
      cdifftime(pControlPF->timeFinish, pControlPF->timeStart));
    fclose(pControlPF->pDebugFile);
  }

}

/*------------------------------------------------------------------------*/

