/*
  model.c - Population dynamics model routines for Seal PF program

  Contains routines that set the priors on the particles, and project
    the particles forward according to the popualtion dynamics model

*/

#include "prepro.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#include <limits.h>
#include "global.h"
#include "util.h"
#include "nrutil.h"
#include "numeric.h"
#include "model.h"

//This is the only global variable - declared in main.c
extern long glngSeed;

/*------------------------------------------------------------------------*/

void initializeParamNumber(controlPFstruct * const pControlPF)
/* Purpose: fills pControlPF->intParamNumber vector with
     the number to index each parameter by when packing them
     up into the thetastar matrix for kernal smoothing, etc
     Puts zeros in for parameters not used in this model,
     or those that are fixed.
     Also sets pControlPF->intNModelParams
*/
{

  // True if parameter is in the model;
  boolean blnIsParamInModel [NPARAMS];
  Int i, intParam;

  //Initialize
  for(intParam=0;intParam<NPARAMS;intParam++) 
    blnIsParamInModel [intParam]= False;

  //Sa,sjmax,amx,psi - yes, unless fixed
  blnIsParamInModel[sa]= !(pControlPF->enumPriorDists[sa]==FIXED);  
  blnIsParamInModel[sjmax]= !(pControlPF->enumPriorDists[sjmax]==FIXED);  
  blnIsParamInModel[amax]= !(pControlPF->enumPriorDists[amax]==FIXED);  
  blnIsParamInModel[psi]= !(pControlPF->enumPriorDists[psi]==FIXED);  
  //Movement parameters - depends on if movement in model and fixed
  if (cblnMovementInModel[pControlPF->enumModel]) {
    blnIsParamInModel[movedd]= !(pControlPF->enumPriorDists[movedd]==FIXED);  
    blnIsParamInModel[movedist] = !(pControlPF->enumPriorDists[movedist]==FIXED);  
    blnIsParamInModel[movesf] = !(pControlPF->enumPriorDists[movesf]==FIXED);  
  }
  //Senescence parameters (these are re-used movement model parameters actually)
  if(cblnSenescenceInModel[pControlPF->enumModel]) {
    blnIsParamInModel[movedd]= !(pControlPF->enumPriorDists[movedd]==FIXED);  
    blnIsParamInModel[movedist] = !(pControlPF->enumPriorDists[movedist]==FIXED);  
  }
  //Rho - extended dens dep
  // or the new dfo model
  if ((pControlPF->enumModel==ddsurvDFO)|(cblnExtendedDDInModel[pControlPF->enumModel]))
    blnIsParamInModel[rho]= !(pControlPF->enumPriorDists[rho]==FIXED);  
  //beta
  if (cblnDDSurvInModel[pControlPF->enumModel]||cblnDDFecInModel[pControlPF->enumModel]) {
    for (i=0;i<NCOLONIES;i++) {
      blnIsParamInModel[beta1+i] = !(pControlPF->enumPriorDists[beta1+i]==FIXED);  
    }
  }

  //Save param numbers and intNModelParams
  i=0;
  for(intParam=0;intParam<NPARAMS;intParam++) {
    if(blnIsParamInModel[intParam]) {
      i++;
      pControlPF->intParamNumber[intParam]=i;
    } else {
      pControlPF->intParamNumber[intParam]=0;
    }
  }
  pControlPF->intNModelParams = i;

}

/*------------------------------------------------------------------------*/

void initializeParams(controlPFstruct * const pControlPF,
                      particlestruct * const pParticles)
/* Purpose: Sample from prior distribution on parameters to each pParticle.  If 
     pControlPF->blnStochasticInitilization is false, sets all values to the mean of the
     priors
   Inputs:
     pPFstruct contains the observations in intPupCounts (1..NColonies x 1..NYears)
   Outputs:
     pParticles contains the particles with initialized parameter values in dblParams (0..NParams-1)
   Notes:
     - If SQRTPSI defined (see global.h), psi parameter is sqrt transformed after generating it,
        for increased efficiency
     - Could improve efficiency here by not initializing parameters that are not used in a particlar
        model
*/
{
  Int intParticle; //index for particle loop
  Int intParam; // index for parameter loop
  double *pdblParam; //Pointer to current parameter value
  boolean blnStochasticInitialization; // shortcut to pControlPF->blnStochasticInitialization
  //These are used for the carrying capacity prior
  double dblInvRho, dblCC, dblTemp, dblLambda, dblLambda2;
  
  blnStochasticInitialization = pControlPF->blnStochasticInitialization;

  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++) {

    //Initialize dblParam - now points to first element in current particles param vector
    // note, if dblParams was base 1, then you'd need &pParticles[intParticle].dblParams[1]
    pdblParam=pParticles[intParticle].dblParams;
    for(intParam=0;intParam<NPARAMS;intParam++) {
      switch(pControlPF->enumPriorDists[intParam]) {
      
      case FIXED:
        *pdblParam=pControlPF->dblPriorParams1[intParam];
        break;
      
      case UNIFORM:
        if(blnStochasticInitialization) {
          //Stochastic init
          *pdblParam=drunif(pControlPF->dblPriorParams1[intParam],
            pControlPF->dblPriorParams2[intParam],
            &glngSeed);
        } else {
          //Fixed initialization - mean is mid point
          *pdblParam = (pControlPF->dblPriorParams1[intParam] + 
            pControlPF->dblPriorParams2[intParam])/2.0;
        }
        break;
      
      case GAMMA:
        if(blnStochasticInitialization) {
          //Stochastic init
          *pdblParam=dgamdev(pControlPF->dblPriorParams1[intParam],
            pControlPF->dblPriorParams2[intParam],
            &glngSeed);
        } else {
          //Fixed initialization - mean is param1*param2
          *pdblParam = pControlPF->dblPriorParams1[intParam] * 
            pControlPF->dblPriorParams2[intParam];
        }
        break;
      
      case BETA:
        if(blnStochasticInitialization) {
          //Stochastic init
          *pdblParam=drbeta(pControlPF->dblPriorParams1[intParam],
            pControlPF->dblPriorParams2[intParam],
            &glngSeed);
        } else {
          //Fixed initialization - mean is param1/(param1+param2)
          *pdblParam = pControlPF->dblPriorParams1[intParam] / 
            (pControlPF->dblPriorParams1[intParam] + pControlPF->dblPriorParams2[intParam]);
        }
        break;

      case SCALEDBETA:
        //Scaled beta uses params 1 and 2 to get a beta RV, but then scales this so that
		// it only occupies part of the range [0,1].  It uses param3 to say how much
		// of the range is occupied by the beta RV, and param4 to say how much of the range
		// above the beta RV is 0.  The amount of the range below the beta RV that is 0 is
		// given by 1-param3-param4
		// For example, if param3 was 0.4 and param4 0.1, we would have 0 mass between 0 and 0.5,
		// the beta RV between 0.5 and 0.9 and 0 mass between 0.9 and 1.0
		if(blnStochasticInitialization){
          *pdblParam=(drbeta(pControlPF->dblPriorParams1[intParam],
            pControlPF->dblPriorParams2[intParam],&glngSeed)*pControlPF->dblPriorParams3[intParam])+
            (1-pControlPF->dblPriorParams3[intParam]-pControlPF->dblPriorParams4[intParam]);
        } else {
          //Fixed initialization - mean is (param1/(param1+param2))*param3 + (1-param3-param4)
          *pdblParam = (pControlPF->dblPriorParams3[intParam]*pControlPF->dblPriorParams1[intParam] / 
            (pControlPF->dblPriorParams1[intParam] + pControlPF->dblPriorParams2[intParam])) + 
            (1-pControlPF->dblPriorParams3[intParam]-pControlPF->dblPriorParams4[intParam]);
        }
        break;

      case CARRYINGCAPACITY:
        //Check it's a beta parameter we're interested in 
        if(intParam<NGPARAMS) {
          printf("Can only specify a carrying capacity prior on a beta parameter!\n");
          exit(1);
        }
        //check it's a density dependent model
        if (!(cblnDDSurvInModel[pControlPF->enumModel]||cblnDDFecInModel[pControlPF->enumModel])) {
          printf("Cannot specify a carrying capacity prior if there is no density dependence model!\n");
          exit(1);
        }
        //work out inverse of rho parameter, for the extended density dep models
        if (cblnExtendedDDInModel[pControlPF->enumModel]) { 
          dblInvRho = 1/pParticles[intParticle].dblParams[rho];
        } else {
          dblInvRho = 1;
        }
        //Get the carrying capacity, sampled from a gamma
        if(blnStochasticInitialization) {
          //Stochastic init
          dblCC=dgamdev(pControlPF->dblPriorParams1[intParam],
            pControlPF->dblPriorParams2[intParam],
            &glngSeed);
        } else {
          //Fixed initialization - mean is param1*param2
          dblCC = pControlPF->dblPriorParams1[intParam] * 
            pControlPF->dblPriorParams2[intParam];
        }
        //The following will only work if you have set all the other parameters for this particle first!
        //Transform from carrying capacity into the beta parameter
        if(pControlPF->enumModel==ddsurvDFO) {
          //DFO model
          dblLambda = (double) PFEMALE * pParticles[intParticle].dblParams[sjmax] *
                       pow(pParticles[intParticle].dblParams[sa], 3.0);
          dblLambda2 = getA(pControlPF->enumModel, 1, pParticles[intParticle].dblParams, 0, 5);
          dblLambda2 += getA(pControlPF->enumModel, 1, pParticles[intParticle].dblParams, 0, 6) * 
                       pParticles[intParticle].dblParams[sa];
          dblLambda2 += getA(pControlPF->enumModel, 1, pParticles[intParticle].dblParams, 0, 7) * 
                       pow(pParticles[intParticle].dblParams[sa], 2.0)/
                       (1-pParticles[intParticle].dblParams[sa]);
          dblLambda *= dblLambda2;
          dblTemp = dblLambda - 1;
          if(dblTemp<0) {
            *pdblParam = VERY_SMALL_NUMBER;
          } else {
            *pdblParam = dblTemp / dblCC;
          }

        } else {
          //Any other model
          dblLambda = ((double) PFEMALE * pParticles[intParticle].dblParams[sjmax] * 
                       pow(pParticles[intParticle].dblParams[sa], (double) (NAGES - 2))) /
                      (1 - pParticles[intParticle].dblParams[sa]);
          if(cblnDDSurvInModel[pControlPF->enumModel]) {
            // Dens dep juv survival
            // { [(PFEMALE*alpha*phij*phia^5) / (1-phia) - 1]^(1/rho) } / N at carrying capacity
            // for both density dependent juvenile survival and density dependent fecundity
            dblTemp = dblLambda * pParticles[intParticle].dblParams[amax] - 1;
            //Sometimes the above is <0, in which case set the beta to a very small number
            if(dblTemp<0) {
              *pdblParam = VERY_SMALL_NUMBER;
            } else {
              *pdblParam = (pow(dblTemp, dblInvRho)) / dblCC;
            }
          } else {
            // Dens dep fecundity
            dblTemp = dblLambda * pParticles[intParticle].dblParams[amax] - 1;
            if (dblTemp<0) {
              *pdblParam = VERY_SMALL_NUMBER;
            } else {
              *pdblParam = (pow(dblTemp, dblInvRho)) / (dblCC * dblLambda);
            }
          }
        }
        break;

      default:
        printf("Incorrect prior distribution for parameter %s\n",cstrParamNames[intParam]);
        exit(1);
      }

#if defined SQRTPSI
      //Sqrt transform psi
      if(intParam==psi)
        *pdblParam=sqrt(*pdblParam);
#endif

      //Increment pointer
      pdblParam ++;
    }

  } //end of loop by particle

  //If debugging, then prInt out summary stats
  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Summary of initial parameter values\n");
    writeParamsSummary(pControlPF, pParticles);
  }
}

/*------------------------------------------------------------------------*/

void initializeStates(controlPFstruct * const pControlPF,
                      particlestruct * const pParticles)
/* Purpose: Generate year 1 states for each pParticle, given that the
     particles are already initialized with priors on the parameters
     Driver routine that calls either the ANZJS method or Takis' stable age method
   Inputs:
     pPFstruct contains the observations in intPupCounts [1..NColonies] [1..NYears]
     pParticles contains the particles with initialized parameter values in dblParams (1 .. NParams)
   Outputs:
     pParticles contains initialized states in intOldStates [1 .. ages] [1 .. colonies]
*/
{

  switch(pControlPF->enumInitializationMethod) {
    case anzjsInit:
      initializeStatesAnzjs(pControlPF, pParticles);
      break;

    case takisInit:
      initializeStatesTakis(pControlPF, pParticles);
      break;

    case survivalInit:
      initializeStatesFromSurvival(pControlPF, pParticles);
      break;

    default:
      printf("Inititilization method ",pControlPF->enumInitializationMethod," not recognized.\n");
      exit(1);
  }

}

/*------------------------------------------------------------------------*/

void initializeStatesAnzjs(controlPFstruct * const pControlPF,
                      particlestruct * const pParticles)
/* Purpose: Generate year 1 states for each pParticle, given that the
     particles are already initialized with priors on the parameters
     If pControlPF->blnStochasticInitilization is false, sets states at their expected
       value
   Inputs:
     pPFstruct contains the observations in intPupCounts [1..NColonies] [1..NYears]
     pParticles contains the particles with initialized parameter values in dblParams (1 .. NParams)
   Outputs:
     pParticles contains initialized states in intOldStates [1 .. ages] [1 .. colonies]
   Implementation note:
     Not sure if this version can tolerate missing observations in the first year.

*/
{
  //temp and shortcut vars
  //temp vars
  double dblState, dblA, dblSj, dblSa;
  Int intGuess;
  //used in conjunction with pControlPF->dblStateInflate to smear out the starting values
  double dblStateDeflate;
  //stores pControlPF->dblStateInflate
  double dblStateInflate;
  //stores sqrtPsi
  double dblSqrtPsi;
  //1..NCOLONIES vector stores pup counts as a double
  double *dblPupCount;
  //1..NCOLONIES vector stores pup count SEs as a double
  double *dblPupCountSE;
  //1 .. ages vector - used only when no ddsurv or ddfec
  double *dblStableAgeStructure;
  // stores pControlPF->blnStochasticInitialization
  boolean blnStochasticInitialization;
  //stores pControlPF->blnIntegrateOutPsi
  boolean blnIntegrateOutPsi;
  //counters
  Int intParticle, intColony, intAge, intYear;
  //stores max pup count for colonies where initial value is missing 
  Int *intMaxPupCount;
  //temprary storage for the particle
  particlestruct *pParticle;

  //Won't work with the senescence model
  if (pControlPF->enumModel == ddsurvsen){
    printf("This state initialization method is not designed for use with the survival senescence model.\n");
    exit(1);
  }



  //load pup counts and SEs into a double vector to save time later
  dblPupCount=dvector(1,NCOLONIES);
  dblPupCountSE=dvector(1,NCOLONIES);
  //also initialize max pup count to -1
  intMaxPupCount=ivector(1,NCOLONIES); 
  for(intColony=1; intColony<=NCOLONIES; intColony++){
    //load pup count
    dblPupCount[intColony] = (double) pControlPF->intPupCounts[1][intColony];
    intMaxPupCount[intColony] = -1;
    if (dblPupCount[intColony]<0) {
      //pup count is missing, so set for now to the max value
      for (intYear=1; intYear<=pControlPF->intNYears; intYear++){
        if(pControlPF->intPupCounts[intYear][intColony]>=0)
          if(pControlPF->intPupCounts[intYear][intColony]>intMaxPupCount[intColony])
            intMaxPupCount[intColony]=pControlPF->intPupCounts[intYear][intColony];
      }
      if(intMaxPupCount[intColony]<0) {
        //all are missing
        printf("Error in initializeStates - one or more colony has all missing pup counts.\n");
        exit(1);
      }

    }
    //load pup count SE
    if(pControlPF->blnKnownSEs) { 
      //May well be a missing value - but that's ok - see below
      dblPupCountSE[intColony]= (double) pControlPF->intPupCountSE[1][intColony];
    } else {
      //Set as missing
      dblPupCountSE[intColony]=-1.0;
    }
  }

  //store stochastic init
  blnStochasticInitialization = pControlPF->blnStochasticInitialization;
  //store integrate out obs process
  blnIntegrateOutPsi = pControlPF->blnIntegrateOutPsi;

  //set state deflate - using this method keeps the same mean in runif
  dblStateInflate = pControlPF->dblStateInflate;
  if(dblStateInflate>=2) {
    printf("Factor to inflate initial states by must be < 2");
    exit(1);
  }
  dblStateDeflate = 2-dblStateInflate;

  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle++) {
    pParticle=&pParticles[intParticle];

    //Initialize dblBPrev
    if(blnIntegrateOutPsi) 
      pParticle->dblBPrev = 1/pControlPF->dblPriorParams2[psi];

#if defined SQRTPSI
    //sqrt has been done already
    dblSqrtPsi = pParticle->dblParams[psi];
#else
    //sqrt needs to be done here
    dblSqrtPsi=sqrt(pParticle->dblParams[psi]);
#endif

    //if no density dependent survival or fecundity, then work out the
    // stable age structure, as you'll need that to work out the number of
    // breeders
    if(!((cblnDDSurvInModel[pControlPF->enumModel])|(cblnDDFecInModel[pControlPF->enumModel]))) {
      dblStableAgeStructure=dvector(1,NAGES);
      //Work out the stable age structure for this particle
      getStableAgeStructure(dblStableAgeStructure, pParticles[intParticle].dblParams[sa],
          pParticles[intParticle].dblParams[sjmax],pParticles[intParticle].dblParams[amax]);
    }

    for(intColony=1; intColony<=NCOLONIES; intColony++) {

      //if there was a missing pup count, then intMaxPupCount is set for this colony.
      //Then, fill in the dblPupCount by sampling now from a uniform
      //with bounds [0, max pup count in any year for that colony]
      if(intMaxPupCount[intColony]>=0) {
        if(blnStochasticInitialization) {
          dblPupCount[intColony]=dran1(&glngSeed)*(double)intMaxPupCount[intColony];
        } else {
          dblPupCount[intColony]=0.5*(double)intMaxPupCount[intColony];
        }
      }

      for(intAge=1;intAge<=NAGES;intAge++) {

         //Get first cut on true numbers
        switch (intAge) {

          case 1: //Pups
            if(dblPupCount[intColony]==0) {
              //If number of pups observed was zero, then set to zero here
              dblState=0.0;
            } else{
              //"Invert" the observation process by sampling from a random normal with
              // mean=observation and stdev=observation/sqrtpsi
              if(blnStochasticInitialization) {
                //if SE not known, use psi to set it
                if(dblPupCountSE[intColony]<0) {
#if defined PSI_IS_PRECISION
        				  dblPupCountSE[intColony]=dblPupCount[intColony]/dblSqrtPsi;
#else
        				  dblPupCountSE[intColony]=dblPupCount[intColony]*dblSqrtPsi;
#endif
                }
                //And sample a value for the state
				        dblState=drnorm(dblPupCount[intColony],dblPupCountSE[intColony], &glngSeed);
              } else {
                dblState=dblPupCount[intColony];
              }
              if(blnIntegrateOutPsi) 
                // Increment dblBPrev with 0.5*sum [(m0/y0 - 1)^2 ] (using notation from
                // Carmen's Dec 5th note)
                pParticle->dblBPrev += 0.5*pow(dblState/dblPupCount[intColony] - 1.0, 2.0);
            }
            break;

          case 2: //Age 1 adults
            //Apply the juvenile survival rate to the number of pups
            dblSj=getSj(pControlPF->enumModel, 1, intColony,
              &pParticle->dblParams[0], 
              pParticle->intOldStates[intColony][1],
              pControlPF->intPupCounts[1][intColony],
              pParticle->intTauPups[intColony]);
            if(blnStochasticInitialization) {
              dblState=drbinom(pParticle->intOldStates[intColony][1],
                (dblSj*(double) PFEMALE), &glngSeed);
            } else {
              dblState = (double) pParticle->intOldStates[intColony][1] * 
                dblSj * (double) PFEMALE;
            }
            break;

          case NAGES: //Age NAGES adults
            //Calculate Sa (may be age specific) - note this is not exactly right, as some animals will
            // be age intAge-1 and some NAGES, but it will be close.  Not used in the current 
            // senescence model in any case, since it is a density dependent survival model
            dblSa=getSa(pControlPF->enumModel, 1, intColony,
              &pParticle->dblParams[0], 
              pParticle->intOldStates[intColony][1],
              pControlPF->intPupCounts[1][intColony],
              pParticle->intTauPups[intColony],intAge-1);
            dblA=pParticle->dblParams[amax];//Even for the DFO model this is correct
            if((cblnDDSurvInModel[pControlPF->enumModel])) {
              //No dens dep fec, so easy! - work out number of females not breeding ...
              if(blnStochasticInitialization) {
                //... from a negative binomial
                dblState = drnbinom((double) pParticle->intOldStates[intColony][1],
                  dblA,&glngSeed);
              } else {
                //.. or just use the expectation
                dblState = (double) pParticle->intOldStates[intColony][1] * (1-dblA);
              }
              //need to add in the number of females who would have bred - one per pup
              dblState += (double) pParticle->intOldStates[intColony][1];
            } else {
              if((cblnDDFecInModel[pControlPF->enumModel])) {
                //Ah - not so easy.  See function getInitial... for details.
                intGuess = getInitialBreedersUnderDensDepFecundity(dblA, 
                  pParticle->dblParams[NGPARAMS+intColony-1],
                  (cblnExtendedDDInModel[pControlPF->enumModel]) ? pParticle->dblParams[rho] : 1.0,
                  dblSa,
                  pParticle->intOldStates[intColony][1],
                  pParticle->intOldStates[intColony][NAGES-1]);
                //Now you have the expected number of adults given alpha, beta and pups
                if(blnStochasticInitialization) {
                  //... sample from a Poisson
                  dblState = drpois((double) intGuess, &glngSeed);
                } else {
                  // or just use the expectation
                  dblState = (double) intGuess;
                }
              } else {
                //Stable age structure will already have been worked out, so...
                //... use this to work out an expected number alive
                dblState = (double) pParticle->intOldStates[intColony][1] /
                  dblStableAgeStructure[1]*dblStableAgeStructure[NAGES];
                if(blnStochasticInitialization) {
                  //... if stochastic, sample from a Poisson with this as mean
                  dblState = drpois(dblState, &glngSeed);
                }
              }
            }
            break;

          default: //The other ages
            dblSa=getSa(pControlPF->enumModel, 1, intColony,
              &pParticle->dblParams[0], 
              pParticle->intOldStates[intColony][1],
              pControlPF->intPupCounts[1][intColony],
              pParticle->intTauPups[intColony],intAge-1);
            //Apply the adult survival rate to the number of the previous age category
            if(blnStochasticInitialization) {
              dblState=drbinom(pParticle->intOldStates[intColony][intAge-1],
                dblSa, &glngSeed);
            } else {
              dblState=(double) pParticle->intOldStates[intColony][intAge-1] * 
                dblSa;
            }

        }

        if(dblState<0) {
          //Where this generates a value <0, set to 0
          pParticle->intOldStates[intColony][intAge]=0;
        } else {
          //Otherwise, "smear" this out a bit, by sampling from a uniform
          // centered on the value...
          if (blnStochasticInitialization) 
            dblState=drunif(dblState*dblStateDeflate,
              dblState*dblStateInflate, &glngSeed);
          //... and round
          pParticle->intOldStates[intColony][intAge]=iround(dblState);
        }

      } //intAge
    } //intColony
  } //intParticle

  //free pupcount and pupcountSE vector
  free_dvector(dblPupCount, 1, NCOLONIES);
  free_dvector(dblPupCountSE, 1, NCOLONIES);
  free_ivector(intMaxPupCount, 1, NCOLONIES);
  if(!((cblnDDSurvInModel[pControlPF->enumModel])|(cblnDDFecInModel[pControlPF->enumModel]))) {
    //free StableAge vector
    free_dvector(dblStableAgeStructure, 1, NAGES);
  }

  //If debugging, then prInt out summary stats
  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Summary of initial state values\n");
    writeStatesSummary(pControlPF, pParticles, 1, False);
  }
}

/*------------------------------------------------------------------------*/
void initializeStatesTakis(controlPFstruct * const pControlPF,
                      particlestruct * const pParticles)
/* Purpose: Generate year 1 states for each pParticle, given that the
     particles are already initialized with priors on the parameters
     Uses the stable age structure method, as agreed between Takis and Len
       in email of 26 June 2006
     If pControlPF->blnStochasticInitilization is false, sets states at their expected
       value
   Inputs:
     pPFstruct contains the observations in intPupCounts [1..NColonies] [1..NYears]
     pParticles contains the particles with initialized parameter values in dblParams (1 .. NParams)
   Outputs:
     pParticles contains initialized states in intOldStates [1 .. ages] [1 .. colonies]
   Implementation note:
     Current version cannot tolerate any missing observations in the first year.
*/
{

  //temp and shortcut vars
  //temp vars
  double dblState;
  //used in conjunction with pControlPF->dblStateInflate to smear out the starting values
  double dblStateDeflate;
  //stores pControlPF->dblStateInflate
  double dblStateInflate;
  //stores sqrtPsi
  double dblSqrtPsi;
  //1..NCOLONIES vector stores pup counts as a double
  double *dblPupCount;
  // stores pControlPF->blnStochasticInitialization
  boolean blnStochasticInitialization;
  //counters
  Int intParticle, intColony, intAge, intAge2, intYear, intMaxPupCount;
  //1 .. ages
  double *dblStableAgeStructure;
  //Estimated total population size
  Int intN;
  //Stores temporary numbers for ages 2..NAGES
  Int *intTempState;
  //used for sampling from multinomial
  double dblProbLeft;

  //Can only use this approach for density independent models, or models where
  // density independence doesn't kick in until later
  switch(pControlPF->enumModel) {
    //list OK models here
    case nodd: 
    case ddsurvTakis:
    case ddemigrTakis:
      break;
    //anything else causes a stop
    default:
      printf("Cannot use stable age initialization method with model",
        pControlPF->enumModel, ".\n");
      exit(1);
  }

  //load pup counts into a double vector to save time later
  dblPupCount=dvector(1,NCOLONIES);
  for(intColony=1; intColony<=NCOLONIES; intColony++){
    dblPupCount[intColony] = (double) pControlPF->intPupCounts[1][intColony];
    if (dblPupCount[intColony]<0) {
      //pup count is missing, so use a random uniform between zero and the
      //  minimum number observed in any year
      //First, find minimum number in any year...
      intMaxPupCount = -1;
      for (intYear=1; intYear<=pControlPF->intNYears; intYear++){
        if(pControlPF->intPupCounts[intYear][intColony]>=0)
          if(pControlPF->intPupCounts[intYear][intColony]>intMaxPupCount)
            intMaxPupCount=pControlPF->intPupCounts[intYear][intColony];
      }
      if(intMaxPupCount<0) {
        //all are missing
        printf("Error in initializeStates - one or more colony has all missing pup counts.\n");
        exit(1);
      }
      //...then muliply by a random uniform 0-1
      dblPupCount[intColony]=dran1(&glngSeed)*(double)intMaxPupCount;
    }
  }

  //store stochastic init
  blnStochasticInitialization = pControlPF->blnStochasticInitialization;

  //set state deflate - using this method keeps the same mean in runif
  dblStateInflate = pControlPF->dblStateInflate;
  if(dblStateInflate>=2) {
    printf("Factor to inflate initial states by must be < 2");
    exit(1);
  }
  dblStateDeflate = 2-dblStateInflate;

  //Initialize other arrays we'll need
  dblStableAgeStructure=dvector(1,NAGES);
  //Note this one doesn't include pups!
  intTempState=ivector(2,NAGES);

  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle++) {

#if defined SQRTPSI
    //sqrt has been done already
    dblSqrtPsi = pParticles[intParticle].dblParams[psi];
#else
    //sqrt needs to be done here
    dblSqrtPsi=sqrt(pParticles[intParticle].dblParams[psi]);
#endif

    //Work out the stable age structure for this particle
    getStableAgeStructure(dblStableAgeStructure, pParticles[intParticle].dblParams[sa],
        pParticles[intParticle].dblParams[sjmax],pParticles[intParticle].dblParams[amax]);

    for(intColony=1; intColony<=NCOLONIES; intColony++) {

      for(intAge=1;intAge<=NAGES;intAge++) {

         //Get first cut on true numbers
        switch (intAge) {

          case 1: //Pups
            //"Invert" the observation process by sampling from a random normal with
            // mean=observation and stdev=observation/sqrtpsi
            if(blnStochasticInitialization) {
#if defined PSI_IS_PRECISION
				dblState=drnorm(dblPupCount[intColony],dblPupCount[intColony]/dblSqrtPsi, &glngSeed);
#else
				dblState=drnorm(dblPupCount[intColony],dblPupCount[intColony]*dblSqrtPsi, &glngSeed);
#endif
            } else {
              dblState=dblPupCount[intColony];
            }
            break;

          case 2: //Age 1 adults
            //Work out total population size
            intN = iround((double) pParticles[intParticle].intOldStates[intColony][1]/
              dblStableAgeStructure[1]);
            //Subtract off pups, to get number of adults
            intN = intN-pParticles[intParticle].intOldStates[intColony][1];
            if(intN<0) intN = 0;
            //Get preliminary numbers for the other ages by...
            dblProbLeft = 1.0 - dblStableAgeStructure[1];
            for(intAge2=2;intAge2<NAGES;intAge2++) {
              if(dblProbLeft>0.0) {
                if(blnStochasticInitialization) {
                  // ... applying successive binomails
                  intTempState[intAge2] = (Int) drbinom(intN, 
                    dblStableAgeStructure[intAge2]/
                    dblProbLeft, &glngSeed);
                } else {
                  // ... or just taking the expected outcome
                  intTempState[intAge2] = iround((double) intN *
                    dblStableAgeStructure[intAge2]/dblProbLeft);
                }
              } else {
                intTempState[intAge2] = 0;
              }
              intN -= intTempState[intAge2];
              dblProbLeft -= dblStableAgeStructure[intAge2];
            }
            //The rest go to the last age group
            intTempState[NAGES] = intN;
            //No break here as you want to go on for age 1 adults         

          default: //Age 1 adults and older ages
            dblState = (double) intTempState[intAge];

        }

        if(dblState<0) {
          //Where this generates a value <0, set to 0
          pParticles[intParticle].intOldStates[intColony][intAge]=0;
        } else {
          //Otherwise, "smear" this out a bit, by sampling from a uniform
          // centered on the value...
          if (blnStochasticInitialization) 
            dblState=drunif(dblState*dblStateDeflate,
              dblState*dblStateInflate, &glngSeed);
          //... and round
          pParticles[intParticle].intOldStates[intColony][intAge]=iround(dblState);
        }

      } //intAge
    } //intColony
  } //intParticle

  //free up arrays
  free_dvector(dblPupCount, 1, NCOLONIES);
  free_dvector(dblStableAgeStructure, 1, NAGES);
  free_ivector(intTempState,2, NAGES);

  //If debugging, then prInt out summary stats
  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Summary of initial state values\n");
    writeStatesSummary(pControlPF, pParticles, 1, False);
  }

}

/*------------------------------------------------------------------------*/

void getStableAgeStructure(double * const dblStableAgeStructure, 
                          double dblSa, double dblSj, double dblAlpha)
/* Purpose: Returns the proportion of the population in each age class given
     an exponential growth model
   Inputs:
     dblStableAgeStructure - uninitialized vector 1...ages
     dblSa, dblSj, dblAlpha are model params
   Outputs:
     pParticles contains initialized vector containing proportion in each age class
   Implementation note:
     Currently quite a cheezy way of doing this -- just runs forward lots of years
       from an aribitrary start point.  Evenually want to calc this by getting
       the dominant eigenvector -- but that meant linking in LAPACK code which
       will make it hard to compile on multiple machines.
*/
{

  //Counting vars
  Int intAge, intYear;
  //Temp var used to store total population size at the end;
  double dblTotal;
  //Used to store the accumulating states for each particle
  double *dblStates;
  dblStates=dvector(1,NAGES);

  //Initialize with 1s
  for(intAge=1;intAge<=NAGES;intAge++) {
    dblStates[intAge]=1.0;
  }

  //Run forwards for STABLE_AGE_YEARS
  for (intYear=1;intYear<=STABLE_AGE_YEARS;intYear++) {
    //expected breeding adults
    dblStates[NAGES]=(dblStates[NAGES]+dblStates[NAGES-1])*dblSa;
    //expected nonbreeding adults
    for(intAge=(NAGES-1);intAge>=3;intAge--) {
       dblStates[intAge]=dblStates[intAge-1]*dblSa;
    }
    //expected 1st year adults
    dblStates[2]=dblStates[1]*0.5*dblSj;
    //expected pups
    dblStates[1]=dblStates[NAGES]*dblAlpha;
  }

  //Now work out the sum...
  dblTotal=0.0;
  for(intAge=1;intAge<=NAGES;intAge++) {
    dblTotal+=dblStates[intAge];
  } 
  //... and divide by the sum to get the stable age structure
  for(intAge=1;intAge<=NAGES;intAge++) {
    dblStableAgeStructure[intAge] = dblStates[intAge]/dblTotal;
  } 

  free_dvector(dblStates,1,NAGES);

}

/*------------------------------------------------------------------------*/

void initializeStatesFromSurvival(controlPFstruct * const pControlPF,
                      particlestruct * const pParticles)
/* Purpose: Generate year 1 states for each pParticle, given that the
     particles are already initialized with priors on the parameters
     If pControlPF->blnStochasticInitilization is false, sets states at their expected
       value
     Starts with pups and then just works up each age, using the number at the previous age and
       the survival rate to get the expectation.  In other words, it's the same as the ANZJS 
       version, except that in that version the breeding animals are done using fecundity; here
       I use survival.  I tried this a while back and it didn't work well when there were few states
       and just one age breeding; I'm using it this time when there are many ages, so that the 
       last age category will have almost noone in it, and when many adult ages breed, so it's hard
       to apportion fecundity.
   Inputs:
     pPFstruct contains the observations in intPupCounts [1..NColonies] [1..NYears]
     pParticles contains the particles with initialized parameter values in dblParams (1 .. NParams)
   Outputs:
     pParticles contains initialized states in intOldStates [1 .. ages] [1 .. colonies]

*/
{
  //temp and shortcut vars
  //temp vars
  double dblState, dblSj, dblSa;
  double dblGuess;
  //used in conjunction with pControlPF->dblStateInflate to smear out the starting values
  double dblStateDeflate;
  //stores pControlPF->dblStateInflate
  double dblStateInflate;
  //stores sqrtPsi
  double dblSqrtPsi;
  //1..NCOLONIES vector stores pup counts as a double
  double *dblPupCount;
  //1..NCOLONIES vector stores pup count SEs as a double
  double *dblPupCountSE;
  // stores pControlPF->blnStochasticInitialization
  boolean blnStochasticInitialization;
  //stores pControlPF->blnIntegrateOutPsi
  boolean blnIntegrateOutPsi;
  //counters
  Int intParticle, intColony, intAge, intYear;
  //stores max pup count for colonies where initial value is missing 
  Int *intMaxPupCount;
  //temprary storage for the particle
  particlestruct *pParticle;

  //Only designed for the senescence model
  if (!(pControlPF->enumModel == ddsurvsen)){
    printf("This state initialization method can only be used with the survival senescence model.\n");
    exit(1);
  }

  //load pup counts and SEs into a double vector to save time later
  dblPupCount=dvector(1,NCOLONIES);
  dblPupCountSE=dvector(1,NCOLONIES);
  //also initialize max pup count to -1
  intMaxPupCount=ivector(1,NCOLONIES); 
  for(intColony=1; intColony<=NCOLONIES; intColony++){
    //load pup count
    dblPupCount[intColony] = (double) pControlPF->intPupCounts[1][intColony];
    intMaxPupCount[intColony] = -1;
    if (dblPupCount[intColony]<0) {
      //pup count is missing, so set for now to the max value
      for (intYear=1; intYear<=pControlPF->intNYears; intYear++){
        if(pControlPF->intPupCounts[intYear][intColony]>=0)
          if(pControlPF->intPupCounts[intYear][intColony]>intMaxPupCount[intColony])
            intMaxPupCount[intColony]=pControlPF->intPupCounts[intYear][intColony];
      }
      if(intMaxPupCount[intColony]<0) {
        //all are missing
        printf("Error in initializeStates - one or more colony has all missing pup counts.\n");
        exit(1);
      }

    }
    //load pup count SE
    if(pControlPF->blnKnownSEs) { 
      //May well be a missing value - but that's ok - see below
      dblPupCountSE[intColony]= (double) pControlPF->intPupCountSE[1][intColony];
    } else {
      //Set as missing
      dblPupCountSE[intColony]=-1.0;
    }
  }

  //store stochastic init
  blnStochasticInitialization = pControlPF->blnStochasticInitialization;
  //store integrate out obs process
  blnIntegrateOutPsi = pControlPF->blnIntegrateOutPsi;

  //set state deflate - using this method keeps the same mean in runif
  dblStateInflate = pControlPF->dblStateInflate;
  if(dblStateInflate>=2) {
    printf("Factor to inflate initial states by must be < 2");
    exit(1);
  }
  dblStateDeflate = 2-dblStateInflate;

  for (intParticle = 1; intParticle<=pControlPF->intNParticles; intParticle++) {
    pParticle=&pParticles[intParticle];

    //Initialize dblBPrev
    if(blnIntegrateOutPsi) 
      pParticle->dblBPrev = 1/pControlPF->dblPriorParams2[psi];

#if defined SQRTPSI
    //sqrt has been done already
    dblSqrtPsi = pParticle->dblParams[psi];
#else
    //sqrt needs to be done here
    dblSqrtPsi=sqrt(pParticle->dblParams[psi]);
#endif

    for(intColony=1; intColony<=NCOLONIES; intColony++) {

      //if there was a missing pup count, then intMaxPupCount is set for this colony.
      //Then, fill in the dblPupCount by sampling now from a uniform
      //with bounds [0, max pup count in any year for that colony]
      if(intMaxPupCount[intColony]>=0) {
        if(blnStochasticInitialization) {
          dblPupCount[intColony]=dran1(&glngSeed)*(double)intMaxPupCount[intColony];
        } else {
          dblPupCount[intColony]=0.5*(double)intMaxPupCount[intColony];
        }
      }

      for(intAge=1;intAge<=NAGES;intAge++) {

         //Get first cut on true numbers
        switch (intAge) {

          case 1: //Pups
            if(dblPupCount[intColony]==0) {
              //If number of pups observed was zero, then set to zero here
              dblState=0.0;
            } else{
              //"Invert" the observation process by sampling from a random normal with
              // mean=observation and stdev=observation/sqrtpsi
              if(blnStochasticInitialization) {
                //if SE not known, use psi to set it
                if(dblPupCountSE[intColony]<0) {
#if defined PSI_IS_PRECISION
        				  dblPupCountSE[intColony]=dblPupCount[intColony]/dblSqrtPsi;
#else
        				  dblPupCountSE[intColony]=dblPupCount[intColony]*dblSqrtPsi;
#endif
                }
                //And sample a value for the state
				        dblState=drnorm(dblPupCount[intColony],dblPupCountSE[intColony], &glngSeed);
              } else {
                dblState=dblPupCount[intColony];
              }
              if(blnIntegrateOutPsi) 
                // Increment dblBPrev with 0.5*sum [(m0/y0 - 1)^2 ] (using notation from
                // Carmen's Dec 5th note)
                pParticle->dblBPrev += 0.5*pow(dblState/dblPupCount[intColony] - 1.0, 2.0);
            }
            break;

          case 2: //Age 1 adults
            //Apply the juvenile survival rate to the number of pups
            dblSj=getSj(pControlPF->enumModel, 1, intColony,
              &pParticle->dblParams[0], 
              pParticle->intOldStates[intColony][1],
              pControlPF->intPupCounts[1][intColony],
              pParticle->intTauPups[intColony]);
            if(blnStochasticInitialization) {
              dblState=drbinom(pParticle->intOldStates[intColony][1],
                (dblSj*(double) PFEMALE), &glngSeed);
            } else {
              dblState = (double) pParticle->intOldStates[intColony][1] * 
                dblSj * (double) PFEMALE;
            }
            break;

          case NAGES: //Age NAGES adults
            //Calculate Sa (may be age specific) - note this is not exactly right, as some animals will
            // be age intAge-1 and some NAGES, but it will be close.
            //Number in last age class is approximated by number in age class before * Sa/(1-Sa)
            dblSa=getSa(pControlPF->enumModel, 1, intColony,
              &pParticle->dblParams[0], 
              pParticle->intOldStates[intColony][1],
              pControlPF->intPupCounts[1][intColony],
              pParticle->intTauPups[intColony],intAge-1);
            dblGuess=(double) pParticle->intOldStates[intColony][intAge-1]*dblSa/(1-dblSa);
            if(blnStochasticInitialization) {
              dblState=drpois((double) dblGuess, &glngSeed);;
            } else {
              dblState=dblGuess;
            }
            break;

          default: //The other ages
            dblSa=getSa(pControlPF->enumModel, 1, intColony,
              &pParticle->dblParams[0], 
              pParticle->intOldStates[intColony][1],
              pControlPF->intPupCounts[1][intColony],
              pParticle->intTauPups[intColony],intAge-1);
            //Apply the adult survival rate to the number of the previous age category
            if(blnStochasticInitialization) {
              dblState=drbinom(pParticle->intOldStates[intColony][intAge-1],
                dblSa, &glngSeed);
            } else {
              dblState=(double) pParticle->intOldStates[intColony][intAge-1] * 
                dblSa;
            }

        }

        if(dblState<0) {
          //Where this generates a value <0, set to 0
          pParticle->intOldStates[intColony][intAge]=0;
        } else {
          //Otherwise, "smear" this out a bit, by sampling from a uniform
          // centered on the value...
          if (blnStochasticInitialization) 
            dblState=drunif(dblState*dblStateDeflate,
              dblState*dblStateInflate, &glngSeed);
          //... and round
          pParticle->intOldStates[intColony][intAge]=iround(dblState);
        }

      } //intAge
    } //intColony
  } //intParticle

  //free pupcount and pupcountSE vector
  free_dvector(dblPupCount, 1, NCOLONIES);
  free_dvector(dblPupCountSE, 1, NCOLONIES);
  free_ivector(intMaxPupCount, 1, NCOLONIES);

  //If debugging, then prInt out summary stats
  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Summary of initial state values\n");
    writeStatesSummary(pControlPF, pParticles, 1, False);
  }
}

/*------------------------------------------------------------------------*/
void runPopulationProcess(Int intYear, boolean blnIsStochastic, 
                          controlPFstruct * const pControlPF,
                          particlestruct * const pParticles)
/*
Purpose:
  Runs the population dynamics model from year intYear-1 to intYear
  If blnIsStochastic then generates new values at random, otherwise
    uses the expected values
  If blnNonIntegerPups and blnIsStochastic then the expected value of pups
    is first generated as a real value and stored in pParticles->dblNonIntegerPups,
    before being rounded and stored in the usual pParticles->intNewStates[,1].
    This real value is used in the auxiliary particle filter.
  If year is Tau-1 then stores the estimated pup values into pParticles<-intTauPups
*/

{
  Int intParticle, intColony, intAge;
  Int intTempPups;
  //temp vars
  double dblSj, dblA, dblSa;
  //Shortcuts
  Int enumModel;
  boolean blnDensDepSj, blnDensDepA, blnDensDepSa, blnSenescenceSa, blnMovement;
  particlestruct *pParticle;

  enumModel = pControlPF->enumModel;
  blnDensDepSj = cblnDDSurvInModel[enumModel];
  blnDensDepA = cblnDDFecInModel[enumModel];
  blnDensDepSa = cblnDDEmigrInModel[enumModel];
  blnSenescenceSa = cblnSenescenceInModel[enumModel];
  blnMovement = cblnMovementInModel[enumModel];



#if !defined TAU_N
  //If it's the right model and year, save estimated pup numbers to intTauPups
  if((cblnTauInModel[enumModel])&(intYear==TAU-1)){
    for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++) {
      pParticle=&pParticles[intParticle];
      for(intColony=1;intColony<=NCOLONIES;intColony++) {
        pParticle->intTauPups[intColony]=pParticle->intOldStates[intColony][1];
      }
    }
  }
#endif

  //Even if stochastic pop process was passed in, if controlPFstruct says no, then
  // make it deterministic!
  if (!(pControlPF->blnStochasticPopulationProcess)) blnIsStochastic = False;

  //Harvest subprocess
  if (!(pControlPF->enumHarvestStrategy == harvestNone)) {
  //###Temporary code ### - for DFO pop projection analysis
  //    if(intYear<=5) 
    runHarvestSubProcess (intYear, blnIsStochastic, pControlPF, pParticles);
    //If debugging, then print out summary of stats
    if (pControlPF->blnDebugFile) {
      fprintf(pControlPF->pDebugFile, "Summary of state values after harvest\n",intYear);
      writeStatesSummary(pControlPF, pParticles, intYear, False);
  }

  }

  //Survival and age incrementation subprocess
  //Note - do for each particle before doing the movement subprocess so that
  // we can produce summary statistics for the movement subprocess alone
  if (pControlPF->blnDebugFile)
    fprintf(pControlPF->pDebugFile, "Survival and ageing sub-process\n");
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++) {
    pParticle=&pParticles[intParticle];
    for(intColony=1;intColony<=NCOLONIES;intColony++) {
      //Age 1 adults
      if (blnDensDepSj) {
        dblSj = getSj(enumModel, intYear, intColony, pParticle->dblParams, 
          pParticle->intOldStates[intColony][1],
          pControlPF->intPupCounts[intYear-1][intColony],
          pParticle->intTauPups[intColony]);
      } else {
        dblSj = pParticle->dblParams[sjmax];
      }
      if(blnIsStochastic) {
        pParticle->intNewStates[intColony][2] = (Int) drbinom(pParticle->intOldStates[intColony][1],
          ((double) PFEMALE*dblSj), &glngSeed);
      } else {
        pParticle->intNewStates[intColony][2] = iround((double) pParticle->intOldStates[intColony][1] *
          ((double) PFEMALE*dblSj));
      }
      //Age 2 adults (ie 3rd in stats array) to NAGES adults - except last year's NAGES adults, which are dealt with below
      //Note the numbers here are pre-movement
      if(blnDensDepSa|blnSenescenceSa) {
        //Get the survival for age 2 animals -- i.e., survival from age 1 to age 2 (hence age=1 passed to the getSa function)
        // (age only matters for the Senescence model)
        dblSa = getSa(enumModel, intYear, intColony,
          pParticle->dblParams, pParticle->intOldStates[intColony][1], 
          pControlPF->intPupCounts[intYear-1][intColony],
          pParticle->intTauPups[intColony], 1);
      } else {
        dblSa = pParticle->dblParams[sa];
      }
      for(intAge=3;intAge<=NAGES;intAge++) {
        if(blnIsStochastic) {
          pParticle->intNewStates[intColony][intAge]= (Int) drbinom(pParticle->intOldStates[intColony][intAge-1],
            dblSa, &glngSeed);
        } else {
          pParticle->intNewStates[intColony][intAge]= iround((double) pParticle->intOldStates[intColony][intAge-1] *
            dblSa);
        }
        //If it's a senescence model, need to get survival for the next age of animal
        if(blnSenescenceSa) {
          dblSa = getSa(enumModel, intYear, intColony,
            pParticle->dblParams, pParticle->intOldStates[intColony][1], 
            pControlPF->intPupCounts[intYear-1][intColony],
            pParticle->intTauPups[intColony], intAge-1);
        }
      }
      //NAGES adults from last year
      if(blnIsStochastic) {
        pParticle->intNewStates[intColony][NAGES] += 
          (Int) drbinom(pParticle->intOldStates[intColony][NAGES], dblSa, &glngSeed);
      } else {
        pParticle->intNewStates[intColony][NAGES] += 
          iround((double) (pParticle->intOldStates[intColony][NAGES])*dblSa);
      }
    }
  }

  //Movement subprocess
  if(blnMovement) runMovementSubProcess (intYear, blnIsStochastic, pControlPF, pParticles);

  //Breeding subprocess
  //Note - must come after movement in case I specify that age 6 is the age that moves, rather than
  // age 5, for example.  In the former case, the number of adults available to breed will depend on movement
  if (pControlPF->blnDebugFile)
    fprintf(pControlPF->pDebugFile, "Breeding sub-process\n");
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++) {
    pParticle=&pParticles[intParticle];
    for(intColony=1;intColony<=NCOLONIES;intColony++) {

      switch(enumModel) {
        case ddsurvDFO:
          //dfo Model
          intTempPups=0;
          //Go through all ages that can breed, and accumulate pups
          for(intAge=4;intAge<=NAGES;intAge++) {
            dblA = getA(enumModel, intColony, pParticle->dblParams, pParticle->intOldStates[intColony][intAge], intAge);
            if(blnIsStochastic) {
              intTempPups += (Int) drbinom(pParticle->intNewStates[intColony][intAge], dblA, &glngSeed);
            } else {
              intTempPups += iround((double) pParticle->intNewStates[intColony][intAge] * dblA);
            }
          }
          pParticle->intNewStates[intColony][1] = intTempPups;
          break;
        case ddsurvsen:
          //senescence Model - breeding starts at age TAU
          intTempPups=0;
          //Go through all ages that can breed, and accumulate pups
          for(intAge=(TAU+1);intAge<=NAGES;intAge++) {
            dblA = getA(enumModel, intColony, pParticle->dblParams, pParticle->intOldStates[intColony][intAge], intAge);
            if(blnIsStochastic) {
              intTempPups += (Int) drbinom(pParticle->intNewStates[intColony][intAge], dblA, &glngSeed);
            } else {
              intTempPups += iround((double) pParticle->intNewStates[intColony][intAge] * dblA);
            }
          }
          pParticle->intNewStates[intColony][1] = intTempPups;
          break;
        default:
          //models other than the above
          if (blnDensDepA) {
            dblA = getA(enumModel, intColony, pParticle->dblParams, pParticle->intOldStates[intColony][NAGES], NAGES);
          } else {
            dblA = pParticle->dblParams[amax];
          }
          if(blnIsStochastic) {
            pParticle->intNewStates[intColony][1] = (Int) drbinom(pParticle->intNewStates[intColony][NAGES],
              dblA, &glngSeed);
          } else {
            pParticle->intNewStates[intColony][1] = iround((double) pParticle->intNewStates[intColony][NAGES] *
              dblA);
          }
      }
    }
  }

  //If debugging, then print out summary of stats
  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Summary of state values in year %i\n",intYear);
    writeStatesSummary(pControlPF, pParticles, intYear, True);
  }
}

/*------------------------------------------------------------------------*/

void runHarvestSubProcess (Int intYear, boolean blnIsStochastic,
                          controlPFstruct * const pControlPF,
                          particlestruct * const pParticles)
/*
Purpose:
  Driver function for the various harvest options
*/
{

  switch(pControlPF->enumHarvestStrategy) {
    case harvestNone:   
      printf("Harvest routine has been called when no harvest specified.");
      exit(1);
    case harvestFixed:
      runFixedHarvestSubProcess (intYear, blnIsStochastic, pControlPF, pParticles);
      break;
    case harvestProp:
      runPropHarvestSubProcess (intYear, blnIsStochastic, pControlPF, pParticles);
      break;
    case harvestFixedTot:
      runFixedTotHarvestSubProcess (intYear, blnIsStochastic, pControlPF, pParticles);
      break;
    default:
      printf("The controlPF file specifies a type of sampler that is not recognized by the filterParticles routine");
      exit(1);
  }
}


/*------------------------------------------------------------------------*/

void runFixedHarvestSubProcess (Int intYear, boolean blnIsStochastic,
                                controlPFstruct * const pControlPF,
                                particlestruct * const pParticles)
/*
Purpose:
  Runs the fixed harvest strategy
*/
{
  Int intParticle, intColony, intAge;
  Int intTempHarvest, intTempHarvest2, intTempHarvest3, intTempHarvest4;
  double *dblPropAllAges, *dblProp1Plus, dblTotalPop, dblTotalPop2;
  particlestruct *pParticle;

  //Initialize
  dblPropAllAges = dvector(1, NAGES);
  dblProp1Plus = dvector(1, NAGES-1);

  //Write some debug output
  if(pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Harvest subprocess - just after previous year's breeding season\n");
    for(intColony=1;intColony<=NCOLONIES;intColony++) {
      fprintf(pControlPF->pDebugFile, "  Colony %i harvest: %i Pups, %i 1Plus, %i AllAges\n",
        intColony,
        pControlPF->intPupRemovals[intYear-1][intColony],
        pControlPF->int1PlusRemovals[intYear-1][intColony],
        pControlPF->intAllAgeRemovals[intYear-1][intColony]);
    }
  }

  //Do the removals for each particle
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++) {
    pParticle=&pParticles[intParticle];

    for(intColony=1;intColony<=NCOLONIES;intColony++) {

      //Work out the proportion of animals in each age class
      //Do the 1Plus group first
      //Start by working out the total population size
      dblTotalPop = 0.0;
      for(intAge=2;intAge<=NAGES;intAge++)
        dblTotalPop += (double) pParticle->intOldStates[intColony][intAge];
      //Use this to work out proportion of 1Plus animals
      for(intAge=2;intAge<=NAGES;intAge++)
        dblProp1Plus[intAge-1] = (double) pParticle->intOldStates[intColony][intAge]/(double) dblTotalPop;
      //Then do the AllAges group
      //for this, you need to add include males and pups
      dblTotalPop2 = (dblTotalPop / (1.0-pControlPF->dblHarvestPropMale)) + (double) pParticle->intOldStates[intColony][1];
      //Now can work out the proportion of AllAges animals
      dblPropAllAges[1] = (double) pParticle->intOldStates[intColony][1]/dblTotalPop2;
      for(intAge=2;intAge<=NAGES;intAge++)
        dblPropAllAges[intAge] = (double) pParticle->intOldStates[intColony][intAge] /
          ((1.0-pControlPF->dblHarvestPropMale)*(double) dblTotalPop2);

      //Removal of pups
      intTempHarvest=pControlPF->intPupRemovals[intYear-1][intColony];
      if(intTempHarvest>0) {
        pParticle->intOldStates[intColony][1]-=intTempHarvest;
        //Don't allow negative pups!
        if(pParticle->intOldStates[intColony][1]<0) pParticle->intOldStates[intColony][1]=0;
      }

      //Removal of 1Plus animals
      //Need to account for males in harvest - so multiply harvest
      //  numbers by proportion female
      intTempHarvest=iround((double) pControlPF->int1PlusRemovals[intYear-1][intColony] *
        (1.0-(double) pControlPF->dblHarvestPropMale));
      if((intTempHarvest>0)&(dblTotalPop>0.0)){
        intTempHarvest3=0;
        for(intAge=2;intAge<NAGES;intAge++){
          intTempHarvest2 = iround((double)intTempHarvest * dblProp1Plus[intAge-1]);
          intTempHarvest3+=intTempHarvest2;
          pParticle->intOldStates[intColony][intAge]-=intTempHarvest2;
          //Negative seals not allowed
          if(pParticle->intOldStates[intColony][intAge]<0)
            pParticle->intOldStates[intColony][intAge]=0;
        }
        //The last category gets what's left over
        pParticle->intOldStates[intColony][NAGES]-=(intTempHarvest-intTempHarvest3);
        //Don't allow negative numbers
        if(pParticle->intOldStates[intColony][NAGES]<0)
          pParticle->intOldStates[intColony][NAGES]=0;
      }

      //Removal of all ages
      intTempHarvest=iround((double) pControlPF->intAllAgeRemovals[intYear-1][intColony]);
      if((intTempHarvest>0)&(dblTotalPop2>0.0)){
        intTempHarvest3=0;
        for(intAge=1;intAge<NAGES;intAge++){
          intTempHarvest4 = iround((double)intTempHarvest * dblPropAllAges[intAge]);
          //Work out actual harvest
          if(intAge==1) {
            intTempHarvest2 = intTempHarvest4;
          } else {
            //Need to multiply by proportion female
            intTempHarvest2 = iround((double)intTempHarvest4 * 
              (1.0-(double) pControlPF->dblHarvestPropMale));
          }
          intTempHarvest3+=intTempHarvest4;
          pParticle->intOldStates[intColony][intAge] -= intTempHarvest2;
          //Don't allow negative seals
          if(pParticle->intOldStates[intColony][intAge]<0)
            pParticle->intOldStates[intColony][intAge]=0;
        }
        //The last category gets what's left over
        pParticle->intOldStates[intColony][NAGES]-=iround((intTempHarvest-intTempHarvest3)*(1.0-(double) pControlPF->dblHarvestPropMale));
        //Don't allow negative numbers
        if(pParticle->intOldStates[intColony][NAGES]<0)
          pParticle->intOldStates[intColony][NAGES]=0;
      }

    }
  }

  //Free space
  free_dvector(dblPropAllAges, 1, NAGES);
  free_dvector(dblProp1Plus, 1, NAGES);

}
                          
/*------------------------------------------------------------------------*/

void runPropHarvestSubProcess (Int intYear, boolean blnIsStochastic,
                                controlPFstruct * const pControlPF,
                                particlestruct * const pParticles)
/*
Purpose:
  Runs the harvest strategy where numbers taken are proportional to numbers
  there
*/
{

//TO DO!


}
                          
/*------------------------------------------------------------------------*/

void runFixedTotHarvestSubProcess (Int intYear, boolean blnIsStochastic,
                                controlPFstruct * const pControlPF,
                                particlestruct * const pParticles)
/*
Purpose:
  Runs the fixed total harvest strategy, in which a fixed number of animals are removed,
  with a fixed proportion specified as pups and the rest assigned according to the
  age class strengths.
*/
{
  double dblTotalPop;
  double *dblTotalPopByColony, *dblTotal1PlusByColony;
  Int intParticle, intColony, intAge;
  Int *intTotalHarvestByColony, **intTotalHarvestByColonyAge;
  Int intTempHarvest;
  particlestruct *pParticle;

  //Write some debug output
  if(pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Harvest subprocess - just after previous year's breeding season\n");
    fprintf(pControlPF->pDebugFile, "  Target total harvest: %i with proportion %6.4f pups\n",
      pControlPF->intHarvestTotal, pControlPF->dblHarvestPropPup);
  }

  dblTotalPopByColony = dvector(1, NCOLONIES);
  dblTotal1PlusByColony = dvector(1, NCOLONIES);
  intTotalHarvestByColony = ivector(1, NCOLONIES);
  intTotalHarvestByColonyAge = imatrix(1, NCOLONIES, 1, NAGES);

  //Do the removals for each particle
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++) {
    pParticle=&pParticles[intParticle];
    
    //Work out total population size and 1+ year old population size for each colony
    dblTotalPop=0.0;
    for(intColony=1;intColony<=NCOLONIES;intColony++) {
      dblTotal1PlusByColony[intColony]=0.0;
      for(intAge=2;intAge<=NAGES;intAge++) {
        dblTotal1PlusByColony[intColony]+=(double)pParticle->intOldStates[intColony][intAge];
      }
      dblTotalPopByColony[intColony]=dblTotal1PlusByColony[intColony] + 
        (double) pParticle->intOldStates[intColony][1];
      dblTotalPop += dblTotalPopByColony[intColony];
    }

    //if zero, no harvest possible
    if(dblTotalPop>0.0) {    

      //Assign total harvest to each colony
      intTempHarvest=0;
      for(intColony=1;intColony<=NCOLONIES-1;intColony++) {
        intTotalHarvestByColony[intColony]=iround(dblTotalPopByColony[intColony]/
          dblTotalPop*(double)pControlPF->intHarvestTotal);
        if(intTotalHarvestByColony[intColony]<0) intTotalHarvestByColony[intColony]=0;
        intTempHarvest += intTotalHarvestByColony[intColony];
      }    
      //Last one is the remaining number to be harvested
      intTotalHarvestByColony[NCOLONIES]=pControlPF->intHarvestTotal-intTempHarvest;
      if(intTotalHarvestByColony[NCOLONIES]<0) intTotalHarvestByColony[NCOLONIES]=0;

      //Assign to each age within each colony
      for(intColony=1;intColony<=NCOLONIES;intColony++) {
        //Pups
        intTotalHarvestByColonyAge[intColony][1]=iround((double)intTotalHarvestByColony[intColony]*
          pControlPF->dblHarvestPropPup);
        intTotalHarvestByColony[intColony]-=intTotalHarvestByColonyAge[intColony][1];
        intTempHarvest=0;
        //The other ages except the last
        for(intAge=2;intAge<=NAGES-1;intAge++) {
          intTotalHarvestByColonyAge[intColony][intAge]=iround((double)intTotalHarvestByColony[intColony]*
            (double)pParticle->intOldStates[intColony][intAge]/dblTotal1PlusByColony[intColony]);
          if(intTotalHarvestByColonyAge[intColony][intAge]<0) intTotalHarvestByColonyAge[intColony][intAge]=0;
          intTempHarvest+=intTotalHarvestByColonyAge[intColony][intAge];
        }
        //The last one
        intTotalHarvestByColonyAge[intColony][NAGES]=intTotalHarvestByColony[intColony]-intTempHarvest;
        if(intTotalHarvestByColonyAge[intColony][NAGES]<0) intTotalHarvestByColonyAge[intColony][NAGES]=0;
      }

      //Now actually do the removals
      for(intColony=1;intColony<=NCOLONIES;intColony++) {
        //Pups
        pParticle->intOldStates[intColony][1]-=intTotalHarvestByColonyAge[intColony][1];
        //Don't allow negative numbers
        if(pParticle->intOldStates[intColony][1]<0) 
          pParticle->intOldStates[intColony][1]=0;
        //Age 1+ females.  Need to correct for the proportion harvested that are male
        for(intAge=2;intAge<=NAGES;intAge++) {
          pParticle->intOldStates[intColony][intAge]-=
            iround((double)intTotalHarvestByColonyAge[intColony][intAge]*
                   (1.0-pControlPF->dblHarvestPropMale));
          //Don't allow negative numbers
          if(pParticle->intOldStates[intColony][intAge]<0) 
            pParticle->intOldStates[intColony][intAge]=0;
        }
      }

    } //0 population size
  } //loop over particles

  free_dvector(dblTotalPopByColony, 1, NCOLONIES);
  free_dvector(dblTotal1PlusByColony, 1, NCOLONIES);
  free_ivector(intTotalHarvestByColony, 1, NCOLONIES);
  free_imatrix(intTotalHarvestByColonyAge, 1, NCOLONIES, 1, NAGES);

}

/*------------------------------------------------------------------------*/

void runMovementSubProcess (Int intYear, boolean blnIsStochastic,
                          controlPFstruct * const pControlPF,
                          particlestruct * const pParticles)
/*
Purpose:
  Implements the movement model to move seals between colonies
  Notation here is from the ANZJS and SCOS papers -- note that 
   in the ssmSeal.pdf precursor to the SSM paper, outlining Carmen's movement
   model, the r's and i's are the other way round.  Here r is the colony
   you're moving from and i is the colony you're moving too.
*/
{

  //counting vars
  Int intParticle, intColony, intColonyI, intColonyR;
  //Shortcuts
  enum model enumModel;
  enum movementModel enumMovementModel;
  boolean blnDensDepSj, blnDensDepA;
  particlestruct *pParticle;
  Int intAgeThatMoves;
  double **dblColonyDistances;
  //Theta and rho from the ANZJS paper
  double *dblTheta, *dblRho, dblSumTheta;
  //p from Carmen's model
  double *dblP;
  //Stores the total number of seals of moving age (used in Carmen's model)
  Int intTotalThatCanMove;
  //Stores the number of seals in the colonies after redistribution
  Int *intNAfterMovement;
  //Stores the numbers actually moving from a colony to another colony
  Int intNMoving;
  //Stores the number left to move from a colony to the other colonies
  Int intNLeftToMove;
  //Amount of the probability that makes up rho that's left in remaining colonies
  double dblProbLeft;

  //Value of the density dependent parameter (sj or a) for each colony
  double *dblDDepParam; // 1 .. NCOLONIES
  //max of (difference in dd param between colony i - colony r) or zero
  double **dblDiffDDepParam; // [i=1 .. NCOLONIES][r=1 .. NCOLONIES]

  //Initialize
  enumModel = pControlPF->enumModel;
  enumMovementModel = pControlPF->enumMovementModel;
  blnDensDepSj = cblnDDSurvInModel[enumModel];
  blnDensDepA = cblnDDFecInModel[enumModel];
  //+1 here is for the pups
  intAgeThatMoves = pControlPF->intAdultAgeThatMoves + 1;
  dblDDepParam = dvector(1, NCOLONIES);
  dblDiffDDepParam = dmatrix(1, NCOLONIES, 1, NCOLONIES);
  dblColonyDistances = pControlPF->dblColonyDistances;
  intNAfterMovement = ivector(1, NCOLONIES);
  dblTheta = dvector(1, NCOLONIES);
  dblRho = dvector(1, NCOLONIES);
  dblP = dvector(1, NCOLONIES);

  //Check whether movement is required
  if (!cblnMovementInModel[enumModel]) return;
  if (!(blnDensDepSj||blnDensDepA)) return; //no movement without density dependence

  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Movement sub-process\n  Before movement:\n");
    writeStateSummaryForOneAge(pControlPF, pParticles, intYear, intAgeThatMoves, True);
  }

  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++) {
    pParticle=&pParticles[intParticle];

    //Retrieve density dep params
    if(blnDensDepSj) {
      //Density dependent survival
      for(intColony=1;intColony<=NCOLONIES;intColony++) {
        dblDDepParam[intColony]=getSj(enumModel, intYear, 
          intColony, pParticle->dblParams, 
          pParticle->intOldStates[intColony][1],
          pControlPF->intPupCounts[intYear][intColony],
          pParticle->intTauPups[intColony]);
      }
    } else {
      //Density dependent fecundity
      for(intColony=1;intColony<=NCOLONIES;intColony++) {
        dblDDepParam[intColony]=getA(enumModel,
          intColony, pParticle->dblParams, pParticle->intOldStates[intColony][NAGES], NAGES);
      }
    }

    //Work out the difference in dens dep params between colonies (set to 0 if negative)
    for(intColonyI=1;intColonyI<=NCOLONIES;intColonyI++) {
      for(intColonyR=1;intColonyR<=NCOLONIES;intColonyR++) {
        dblDiffDDepParam[intColonyI][intColonyR] =
          DMAX(dblDDepParam[intColonyI]-dblDDepParam[intColonyR], 0.0);
      }
    }

    //Reset the NAfter movement and the P's
    intTotalThatCanMove = 0;
    for(intColony=1;intColony<=NCOLONIES;intColony++) {
      intNAfterMovement[intColony]=0;
      dblP[intColony]=0.0;
    }

    //For each colony R, work out where moving seals from that colony go for the SCOS and ANZJS
    // models; for the Carmen model, work out the un-normalized p's.
    //ColonyR is the source, ColonyI is the destination
    for(intColonyR=1;intColonyR<=NCOLONIES;intColonyR++) {

      //Work out thetas
      dblSumTheta = 0.0;
      switch (enumMovementModel) {
        case anzjs:
          for(intColonyI=1;intColonyI<=NCOLONIES;intColonyI++) {
            if(intColonyR==intColonyI) {
              dblTheta[intColonyI]=exp(pParticle->dblParams[movesf]);
            } else {
              dblTheta[intColonyI]=exp(pParticle->dblParams[movedd]*dblDiffDDepParam[intColonyI][intColonyR] -
                pParticle->dblParams[movedist]*dblColonyDistances[intColonyI][intColonyR]);
            }
            dblSumTheta += dblTheta[intColonyI];
          }
          break;
        case scos03:
          for(intColonyI=1;intColonyI<=NCOLONIES;intColonyI++) {
            if(intColonyR==intColonyI) {
              dblTheta[intColonyI]=pParticle->dblParams[movesf];
            } else {
              dblTheta[intColonyI]=pParticle->dblParams[movedd]*dblDiffDDepParam[intColonyI][intColonyR]/
                exp(pParticle->dblParams[movedist]*dblColonyDistances[intColonyI][intColonyR]);
            }
            dblSumTheta += dblTheta[intColonyI];
          }
          break;
        case carmen:
          //Same as scos03 but with site faithfulness param fixed at 1.0
          for(intColonyI=1;intColonyI<=NCOLONIES;intColonyI++) {
            if(intColonyR==intColonyI) {
              dblTheta[intColonyI]=1.0;
            } else {
              dblTheta[intColonyI]=pParticle->dblParams[movedd]*dblDiffDDepParam[intColonyI][intColonyR]/
                exp(pParticle->dblParams[movedist]*dblColonyDistances[intColonyI][intColonyR]);
            }
            dblSumTheta += dblTheta[intColonyI];
          }
          break;

        case dfo:
          //Same as carmen but with distance part removed
          for(intColonyI=1;intColonyI<=NCOLONIES;intColonyI++) {
            if(intColonyR==intColonyI) {
              dblTheta[intColonyI]=1.0;
            } else {
              dblTheta[intColonyI]=pParticle->dblParams[movedd]*dblDiffDDepParam[intColonyI][intColonyR];
            }
            dblSumTheta += dblTheta[intColonyI];
          }
          break;

      } // end switch (enumMovementModel)

      //Convert to rhos (ie normalize)
      if(dblSumTheta>0.0) {
        //If thetas sum to > 0 then normalize thetas to get the Rs
        for(intColonyI=1;intColonyI<=NCOLONIES;intColonyI++) {
          dblRho[intColonyI]=dblTheta[intColonyI]/dblSumTheta;
        }
      } else {
        //If the thetas sum to zero then all seals stay in colony R
        for(intColonyI=1;intColonyI<=NCOLONIES;intColonyI++) {
          dblRho[intColonyI]=(intColonyI==intColonyR) ? 1.0:0.0;
        }
      }

      switch (enumMovementModel) {
        case anzjs:
        case scos03:
          //Redistribute seals from colony R
          intNLeftToMove = pParticle->intNewStates[intColonyR][intAgeThatMoves];
          dblProbLeft = 1.0;
          //Assign the first NCOLONIES-1 colonies by...
          for(intColonyI=1;intColonyI<NCOLONIES;intColonyI++) {
            if(dblProbLeft>0.0) {
              if(blnIsStochastic) {
                // ... applying successive binomials
                intNMoving =(Int) drbinom(intNLeftToMove, dblRho[intColonyI]/dblProbLeft, &glngSeed);
              } else {
                // ... or rounding the expected outcome
                intNMoving = iround(((double)intNLeftToMove)*dblRho[intColonyI]/dblProbLeft);
              }
            } else {
              intNMoving = 0;
            }
          
            intNAfterMovement[intColonyI]+=intNMoving;
            intNLeftToMove -= intNMoving;
            dblProbLeft -=dblRho[intColonyI];
          }
          //The rest go to the last colony
          intNAfterMovement[NCOLONIES]+=intNLeftToMove;
          break;

        case carmen:
        case dfo:
          //Add to the current value of p for each colony
          intNLeftToMove = pParticle->intNewStates[intColonyR][intAgeThatMoves];
          for(intColonyI=1;intColonyI<=NCOLONIES;intColonyI++) {
            dblP[intColonyI] += dblRho[intColonyI]*(double)intNLeftToMove;
          }
          //Add to the total number to move
          intTotalThatCanMove += intNLeftToMove;
          break;

      } // end switch (enumMovementModel)

    }

    switch (enumMovementModel) {
      case anzjs:
      case scos03:

        //For the other movement models,
        // now you've done all the colonies, update the particle states
        for(intColony=1;intColony<=NCOLONIES;intColony++) {
          pParticle->intNewStates[intColony][intAgeThatMoves]=intNAfterMovement[intColony];
        }
        break;


      case carmen:
      case dfo:

        //For the carmen and dfo movement models, now is when you actually move the seals!
        //First check there are some to move!
        if(intTotalThatCanMove>0){
          //Then, normalize the P's
          for(intColony=1;intColony<=NCOLONIES;intColony++) {
            dblP[intColony]/=(double)intTotalThatCanMove;
          }
          //And initialize the number to move and the probability denominator
          intNLeftToMove = intTotalThatCanMove;
          dblProbLeft = 1.0;
          //Now redistribute the seals
          //Assign the first NCOLONIES-1 colonies by...
          for(intColony=1;intColony<NCOLONIES;intColony++) {
            if(dblProbLeft>0.0) {
              if(blnIsStochastic) {
                // ... applying successive binomials
                intNMoving =(Int) drbinom(intNLeftToMove, dblP[intColony]/dblProbLeft, &glngSeed);
              } else {
                // ... or rounding the expected outcome
                intNMoving = iround(((double)intNLeftToMove)*dblP[intColony]/dblProbLeft);
              }
            } else {
               intNMoving = 0;
            }
        
            //Move the seals
            pParticle->intNewStates[intColony][intAgeThatMoves]=intNMoving;
            //Decrement the number left to move...
            intNLeftToMove -= intNMoving;
            //... and the denominator for the movement probability
            dblProbLeft -=dblP[intColony];
          }
          //The rest go to the last colony
          pParticle->intNewStates[NCOLONIES][intAgeThatMoves]=intNLeftToMove;
        } //End of case carmen and case dfo
    }
  }

  free_dvector(dblTheta, 1, NCOLONIES);
  free_dvector(dblRho, 1, NCOLONIES);
  free_dvector(dblP, 1, NCOLONIES);
  free_ivector(intNAfterMovement, 1, NCOLONIES);
  free_dvector(dblDDepParam, 1, NCOLONIES);
  free_dmatrix(dblDiffDDepParam, 1, NCOLONIES, 1, NCOLONIES);

  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "  After movement:\n");
    writeStateSummaryForOneAge(pControlPF, pParticles, intYear, intAgeThatMoves, True);
  }

}

/*------------------------------------------------------------------------*/

double getA(Int enumModel, Int intColony, double * const dblParams, Int intAdults, Int intAge)
/* Purpose:
     Returns value of fecundity parameter a, given model, colony, parameters,
       true pups, and age for some models.
     Has special sections for the DFO model and the survival senescence model;
       all other models are dealt with at the end.
*/
{
  double dblA;
  double dblTemp;

  //DFO model
  if(enumModel==ddsurvDFO) {
    //DFO model has A depending on age
    if(intAge==NAGES) {
      dblA=dblParams[amax];
    } else {
      //Hard-wired in - if 3 yr adult or less, doesn't breed
      // - note Age 4 is a 3 year old adult
      if(intAge<=4) {
        dblA=0.0;
      } else {
         //work out age - note subtract 5.5 rather than 4.5 as age 5
         // is a 4 year old adult, etc
         dblTemp = dblParams[rho]*((double) intAge - 5.5);
         dblTemp = exp(dblTemp);
         dblTemp = dblTemp/(1+dblTemp);
         dblA = dblParams[amax]*dblTemp;
      }
    }
    return(dblA);
  }

  if(enumModel==ddsurvsen) {
    //Animals of age TAU-1 don't breed
    // Note if intAge=TAU then this is a TAU-1 year old adult (because intAge=1 is pups)
    if(intAge<=TAU) {
      dblA=0.0;
    } else {
      //no density dep fecundity
      dblA=dblParams[amax];
    }
    return(dblA);
  }

  //All other models
  if(cblnDDFecInModel[enumModel]) {
    //density dependent fecundity
    if(cblnExtendedDDInModel[enumModel]) {
      //extended beverton-holt
      dblA=dblParams[amax]/(1+pow(dblParams[NGPARAMS+intColony-1]*intAdults, dblParams[rho]));
    } else {
      //ordinary beverton-holt
      dblA=dblParams[amax]/(1+dblParams[NGPARAMS+intColony-1]*intAdults);
    }
  } else {
    //no density dep fecundity
    dblA=dblParams[amax];
  }
  return(dblA);

}

/*------------------------------------------------------------------------*/

double getSj(Int enumModel, Int intYear, Int intColony, double * const dblParams, 
  Int intPups, Int intPupData, Int intTauPups)
/* Purpose:
     Returns value of juvenile survival parameter, Sj, given model, year, colony, parameters,
       true pups (intPups),  observed pups (intPupData) or pups in year Tau-1 (intTauPups) 
*/
{
  double dblSj;

  if(cblnDDSurvInModel[enumModel]) {
	  if(!cblnTauInModel[enumModel]){
      //density dependent juvenile survival
      if(cblnExtendedDDInModel[enumModel]) {
        //extended beverton-holt
        dblSj=dblParams[sjmax]/(1+pow(dblParams[NGPARAMS+intColony-1]*intPups, dblParams[rho]));
      } else {
        //ordinary beverton-holt
        dblSj=dblParams[sjmax]/(1+dblParams[NGPARAMS+intColony-1]*intPups);
      }
    } else {
      //models with tau in them
#if defined TAU_N
#if defined TAU_USE_Y
      if((intYear>TAU)&(intPupData>TAU_N)) {
        //ordinary beverton-holt - but with TAU_N subtracted from pup data
        dblSj=dblParams[sjmax]/(1+dblParams[NGPARAMS+intColony-1]*(intPupData - TAU_N));
#else
      if((intYear>TAU)&(intPups>TAU_N)) {
        //ordinary beverton-holt - but with TAU_N subtracted from estimates of true pup numbers
        dblSj=dblParams[sjmax]/(1+dblParams[NGPARAMS+intColony-1]*(intPups - TAU_N));
#endif
#else
      if((intYear>TAU)&(intPups>intTauPups)) {
        //ordinary beverton-holt - but with intTauPups subtracted from estimates of true pup numbers
        dblSj=dblParams[sjmax]/(1+dblParams[NGPARAMS+intColony-1]*(intPups - intTauPups));
#endif
      } else {
        //no density dependence
        dblSj=dblParams[sjmax];
      }
    }  
  } else {
    //no density dependence
    dblSj=dblParams[sjmax];
  }
  return(dblSj);
}

/*------------------------------------------------------------------------*/

double getSa(Int enumModel, Int intYear, Int intColony, double * const dblParams, 
  Int intPups, Int intPupData, Int intTauPups, Int intAge)
/* Purpose:
     Returns value of adult survival parameter, Sa, given model, year, colony, parameters,
       true pups (intPups), observed pups (intPupData), or estimated pups in year tau-1
       (intTauPups)
	 Designed to deal with the DDEmigration model, where adult survival is density dependent,
	   optionally after year Tau
	 Also designed to deal with the Senescence model, where adult survival declines with age
	   according to parameters alpha (the movedd parameter) and delta (the movedist parameter)
	 Note that these two are assumed to be mutually exclusive.  If neither of these, then assumed
	   to be a constant adult survival model.
*/
{
  double dblSa;

  //Deal with the DDEmigr model
  if(cblnDDEmigrInModel[enumModel]) {
	  if(!cblnTauInModel[enumModel]){
      //density dependent adult survival
      if(cblnExtendedDDInModel[enumModel]) {
        //extended beverton-holt
        dblSa=dblParams[sa]/(1+pow(dblParams[NGPARAMS+intColony-1]*intPups, dblParams[rho]));
      } else {
        //ordinary beverton-holt
        dblSa=dblParams[sa]/(1+dblParams[NGPARAMS+intColony-1]*intPups);
      }
    } else {
      //models with tau in them
#if defined TAU_N
#if defined TAU_USE_Y
      if((intYear>TAU)&(intPupData>TAU_N)) {
        //ordinary beverton-holt - but with TAU_N subtracted from pup data
        //rather than estimated true pups
        dblSa=dblParams[sa]/(1+dblParams[NGPARAMS+intColony-1]*(intPupData - TAU_N));
#else
      if((intYear>TAU)&(intPups>TAU_N)) {
        //ordinary beverton-holt - but with TAU_N subtracted from estimates of true pup numbers
        //rather than estimated true pups
        dblSa=dblParams[sa]/(1+dblParams[NGPARAMS+intColony-1]*(intPups - TAU_N));
#endif
#else
      if((intYear>TAU)&(intPups>intTauPups)) {
        //ordinary beverton-holt - but with intTauPups subtracted from estimates of true pup numbers
        //rather than estimated true pups
        dblSa=dblParams[sa]/(1+dblParams[NGPARAMS+intColony-1]*(intPups - intTauPups));
#endif
      } else {
        //no density dependence
        dblSa=dblParams[sa];
      }
    }
    return(dblSa);
  }

  //Deal with the Senescence model
  if(cblnSenescenceInModel[enumModel]) {
    //Gompertz-Makeham-type model
    dblSa= dblParams[sa]*exp(dblParams[movedd]/dblParams[movedist] * 
      (exp (dblParams[movedist]* (double) intAge) - exp(dblParams[movedist]* (double) (intAge+1) )));
    return(dblSa);
  }

  //Still here -- that means no density dependence in adult survival
  dblSa= dblParams[sa];
  return(dblSa);

}

/*------------------------------------------------------------------------*/

#define APOS 0
#define BETAPOS 1
#define RHOPOS 2
#define NUMPUPSPOS 3
#define NUMOTHERPARAMS 4

float diffPredPups (float sngNumAdults, 
                double *dblOtherParams,
                Int intNumOtherParams)
/*
Purpose:
  Used in getInitialBreedersUnderDensDepFecundity.  
  Called from there by the Brent minimization routine -- returns the difference between
  the actual number of pups observed and the number predicted by
  Alpha * NumAdults / (1 + pow((Beta * NumAdults), Rho))
Inputs:
  sngNumAdults is the number of adults to try
  sngOtherParams is an array [0 ... itnNumOtherParams-1] with elements
    APOS - Alpha
    BETAPOS - Beta
    RHOPOS - Rho
    NUMPUPSPOS - Num pups
Output:
  Abs(difference between predicted number of pups and sngOtherParams[NUMPUPSPOS])
*/
{
  
  double dblNumAdults, dblPredPups, dblDiff;

  dblNumAdults = (double) sngNumAdults;
  dblPredPups = dblOtherParams[APOS] * dblNumAdults /
    (1 + pow(dblOtherParams[BETAPOS] * dblNumAdults, dblOtherParams[RHOPOS]));
  dblDiff = fabs(dblPredPups - dblOtherParams[NUMPUPSPOS]);
  return((float) dblDiff);
}

/*------------------------------------------------------------------------*/

Int getInitialBreedersUnderDensDepFecundity(double dblA, double dblBeta, 
                                            double dblRho, double dblSa, 
                                            Int intPups, Int intSubAdults) 
//Purpose: Title says it all
//  First approach was to use phi_a/(1-phi_a) times the number
//   of age 5's.  But this doesn't work at all -- get far too many age 6+s.
//   Second approach is to use the alpha we have, like we do when it's ddsurv.
//   We know that n6+s is given by n_pup = alpha*n_6+ / (1+(beta*n_6+)^rho)
//   We know n_pup, alpha, beta and rho, so we can solve for n_6+.
//   Uses Brent's algorithm to get an initial value, then polishes it
{
  
  Int intMax;
  double dblOtherParams[NUMOTHERPARAMS];
  float sngAx, sngA1x, sngBx, sngCx, sngC1x, sngNBreeders;
  float sngLastDiff, sngIncrement, sngThisDiff;

  const Int intMaxAdults = 1000000;
  const float sngEPS = (float) 1.0e-6;

  //get a maximum feasible value for number of adults
  if (dblSa > 0) {
    intMax = iround( (double) intSubAdults * dblSa / (1-dblSa));
    intMax = (intMax > intMaxAdults) ? intMaxAdults : intMax;
    intMax = (intMax < intPups) ? intPups : intMax;
  } else {
    intMax = intMaxAdults;
  }

  //Load params into OtherParams array - used in diffPredPups
  dblOtherParams[APOS] = dblA;
  dblOtherParams[BETAPOS] = dblBeta;
  dblOtherParams[RHOPOS] = dblRho;
  dblOtherParams[NUMPUPSPOS] = (double) intPups;

  //Set starting values
  sngAx = (float) intPups;
  sngA1x = (float) intPups+1;
  sngCx = (float) intMax;
  sngC1x = (float) intMax-1;
  //Check the max is really more than the min!
  if(sngA1x>sngCx) return(intPups);
  //Check you have bracketing - ie that 
  // the difference in predicted pups is really less at Ax than Ax+1
  // and Cx than Cx-1
  if(diffPredPups(sngAx, dblOtherParams, NUMOTHERPARAMS) < 
    diffPredPups(sngA1x, dblOtherParams, NUMOTHERPARAMS)) return (intPups);
  if(diffPredPups(sngCx, dblOtherParams, NUMOTHERPARAMS) < 
    diffPredPups(sngC1x, dblOtherParams, NUMOTHERPARAMS)) return (intMax);
  //Set the inside value
  sngBx = (diffPredPups(sngA1x, dblOtherParams, NUMOTHERPARAMS) < 
    diffPredPups(sngC1x, dblOtherParams, NUMOTHERPARAMS)) ? sngA1x : sngC1x;
  //Call Brent to get an initial solution
  brent(sngAx, sngBx, sngCx, diffPredPups, sngEPS, &sngNBreeders, dblOtherParams, NUMOTHERPARAMS);
  sngNBreeders = (float) iround((double) sngNBreeders);

  //If this initial solution isn't within 1 of the minimum, find the minimum
  sngLastDiff = diffPredPups(sngNBreeders, dblOtherParams, NUMOTHERPARAMS);
  //See if a better value is at larger sngNBreeders
  sngIncrement = (sngLastDiff > diffPredPups(sngNBreeders + (float) 1.0, dblOtherParams, NUMOTHERPARAMS)) ? 
    (float) 1.0 : (float) - 1.0;
  do {
    sngNBreeders += sngIncrement;
    sngThisDiff = diffPredPups(sngNBreeders, dblOtherParams, NUMOTHERPARAMS);
    if (sngThisDiff < sngLastDiff) {
      sngLastDiff = sngThisDiff;
    } else {
      sngNBreeders -= sngIncrement;
      break;
    }
  } while ((sngNBreeders < (float) intMax) && (sngNBreeders > (float) intPups));

  return((int) sngNBreeders);

}

/*------------------------------------------------------------------------*/

boolean checkAllMissingObservations (Int intYear,
                                  controlPFstruct * const pControlPF)
/* 
  Returns true if all observations are missing, false otherwise
*/
{

  Int intColony;
  Int intNMissingValues=0;

  //Check for missing values.
  for(intColony=1; intColony<=NCOLONIES; intColony++){
    if (pControlPF->intPupCounts[intYear][intColony]<0) intNMissingValues ++;
  }
  if(intNMissingValues==NCOLONIES) {
    return(True);
  } else {
    return(False);
  }

}

/*------------------------------------------------------------------------*/
void getWeights (Int intYear,
                 double * dblWeights,
                 controlPFstruct * const pControlPF,
                 metastruct * const pMeta,
                 particlestruct * const pParticles)
/* Wrapper function - either gets weights for non integrated observation
   error, or for observation error */
{
    if (pControlPF->blnIntegrateOutPsi) {
      getIntegratedWeights(intYear, dblWeights, pControlPF, pMeta, pParticles);
    } else {
      getNonIntegratedWeights(intYear, dblWeights, pControlPF, pMeta, pParticles);
    }
}

/*------------------------------------------------------------------------*/

void getIntegratedWeights (Int intYear,
                 double * dblWeights,
                 controlPFstruct * const pControlPF,
                 metastruct * const pMeta,
                 particlestruct * const pParticles)
/*
Purpose:
  Returns weights, given the true values in pParticles[].intNewStates
    and observations for intYear in pControlPF.  Weights are calculated
    integrated over the psi parameter.
  If it's the first time that weights have been calculated for this year (as
    evidenced by pMeta[intYear].dblWeightScaling1<0) then calculate a scaling.
Inputs:
  - intYear is the year to look at state values and observations from
  - dblWeights is a vector 1..NParticles to put weights into
  - pControlPF is the control strucutre
  - pMeta is the meta structure
  - pParticles is the particle structure
Output:
  - dblWeights has the weights in it
  - pMeta[intYear].dblWeightScaling1 and dblWeightScaling2 may be updated
Notes:
  - Weights in pParticles are not touched
  - This function is in model.c because it uses the observation model
      to come up with the weights
*/
{
  Int intParticle, intColony;
  Int intNMissingValues=0;
  //shortcuts
  //Used in calculating the weights -- Carmen starts 1st year=0, so subtract
  // 1 from the years used here (which start with 1st year=1)
  double dblThisYear, dblLastYear;
  //1..NCOLONIES vector stores pup counts as a double
  double *dblPupCount;
  double *dblTruePup;
  //parameters from gamma prior on psi param
  double dblPsi1, dblPsi2;
  //set to true if any observed state is zero
  boolean blnZeroObs=False;
  //set to true if any true state for a particle is zero
  boolean blnAnyZeroPups;
  particlestruct *pParticle;

  dblTruePup=dvector(1,NCOLONIES);
  dblPsi1 = pControlPF->dblPriorParams1[psi];
  dblPsi2 = pControlPF->dblPriorParams2[psi];
  dblThisYear = (double) (intYear - 1);
  dblLastYear = (double) (intYear - 2);

  //Load pup counts into a double vector to save time later.
  //Also, check for missing values.
  dblPupCount=dvector(1,NCOLONIES);
  for(intColony=1; intColony<=NCOLONIES; intColony++){
    dblPupCount[intColony] = (double) pControlPF->intPupCounts[intYear][intColony];
    if (dblPupCount[intColony]<0) intNMissingValues ++;
    if (dblPupCount[intColony]==0) blnZeroObs=True;
  }
  if(intNMissingValues==NCOLONIES) {
    //All values are missing, so weights are 1 for all particles
    for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
      dblWeights[intParticle]=1.0;
    return;
  }
  //Temporary code
  if(intNMissingValues>0) {
    printf("Current version not designed to deal with missing observations when psi integrated out\n");
    exit(1);
  }
  if(blnZeroObs) {
    printf("Current version not designed to deal with zero pup counts when psi integrated out\n");
    exit(1);
  }
  if(pControlPF->blnKnownSEs) {
    printf("Current version not designed to deal with known SEs when psi integrated out\n");
    exit(1);
  }

  //For each particle
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++) {
    pParticle=&pParticles[intParticle];

    //Load true pups into a vector to save time later
    //Also, check for zeros
    blnAnyZeroPups = False;
    for(intColony=1;intColony<=NCOLONIES;intColony++) {
      dblTruePup[intColony] = (double) pParticle->intNewStates[intColony][1];
      if(dblTruePup[intColony]==0) blnAnyZeroPups = True;
    }

    //This stuff implements eqn 9 from Carmen's Dec 5 2005 notes,
    //also lines 424 onwards of ken's newseal_intPsi2_RCX.c code
    // wts= prod(1/[n_r])*betastar_prev^(f1+2*t)/betastar_curr^{f1+2*(t+1)}
    //Start by trapping pups=0 will cause a divide by zero error
    if (blnAnyZeroPups) {
      // in that case, set to 0
      dblWeights[intParticle]=0.0;
    } else {
      dblWeights[intParticle]=1.0;
      for(intColony=1;intColony<=NCOLONIES;intColony++)
        dblWeights[intParticle] *= 1.0/dblTruePup[intColony];
      dblWeights[intParticle] *= pow(pParticle->dblBPrev, dblPsi1+2.0*dblLastYear);
      pParticle->dblBCurrent = pParticle->dblBPrev;
      for(intColony=1;intColony<=NCOLONIES;intColony++)
        pParticle->dblBCurrent += 0.5 * pow(dblPupCount[intColony]/dblTruePup[intColony]-1.0,2.0);
      dblWeights[intParticle] /= pow(pParticle->dblBCurrent, dblPsi1+2.0*dblThisYear);

      // Check for null or infinity
      if(isnan(dblWeights[intParticle])|isinf(dblWeights[intParticle])) 
        dblWeights[intParticle]=0.0;
    }

  }

  if (pControlPF->blnScaleWeights) {

    //If doing scaling...
    //...check if you have to create the scaling
    if(pMeta[intYear].dblWeightScaling1 == (double) MISSINGWEIGHT)
      //Try using the mean of the particles
      pMeta[intYear].dblWeightScaling1 = dmeanVector(dblWeights, 1,pControlPF->intNParticles);
    //...do the scaling
    for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
      dblWeights[intParticle]/=pMeta[intYear].dblWeightScaling1;
    //...no second scaling

  }

  free_dvector(dblPupCount, 1, NCOLONIES);
  free_dvector(dblTruePup, 1, NCOLONIES);

  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Summary of weights from data in year %i\n",intYear);
    writeDVectorSummary(pControlPF, dblWeights, 1, pControlPF->intNParticles);
  }

}

/*------------------------------------------------------------------------*/

void getNonIntegratedWeights (Int intYear,
                 double * dblWeights,
                 controlPFstruct * const pControlPF,
                 metastruct * const pMeta,
                 particlestruct * const pParticles)
/*
Purpose:
  Returns weights, given the true values in pParticles[].intNewStates
    and observations for intYear in pControlPF.  Each particles weight
    is calculated according to the psi parameter
  If it's the first time that weights have been calculated for this year (as
    evidenced by pMeta[intYear].dblWeightScaling1<0) then calculate a scaling.
Inputs:
  - intYear is the year to look at state values and observations from
  - dblWeights is a vector 1..NParticles to put weights into
  - pControlPF is the control strucutre
  - pMeta is the meta structure
  - pParticles is the particle structure
Output:
  - dblWeights has the weights in it
  - pMeta[intYear].dblWeightScaling1 and dblWeightScaling2 may be updated
Notes:
  - Weights in pParticles are not touched
  - This function is in model.c because it uses the observation model
      to come up with the weights
*/
{
  Int intParticle, intColony;
  Int intNMissingValues=0;
  double dblLogWeight;
  //1..NCOLONIES vector stores pup counts as a double
  double *dblPupCount;
  //shortcuts
  double dblTruePup;
  double dblSqrtPsi;
  double dblSE;
  //set to true if any true state is zero
  boolean blnZeroStateWithNonZeroCount, blnKnownSEs;

  blnKnownSEs=pControlPF->blnKnownSEs;

  //Load pup counts into a double vector to save time later.
  //Also, check for missing values.
  dblPupCount=dvector(1,NCOLONIES);
  for(intColony=1; intColony<=NCOLONIES; intColony++){
    dblPupCount[intColony] = (double) pControlPF->intPupCounts[intYear][intColony];
    if (dblPupCount[intColony]<0) intNMissingValues ++;
  }
  if(intNMissingValues==NCOLONIES) {
    //All values are missing, so weights are 1 for all particles
    for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
      dblWeights[intParticle]=1.0;
    return;
  }

  //For each particle
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++) {

    //Get the square root of the observation error param
#if defined SQRTPSI
    dblSqrtPsi = pParticles[intParticle].dblParams[psi];
#else
    dblSqrtPsi = sqrt(pParticles[intParticle].dblParams[psi]);
#endif

    dblLogWeight = 0.0;
    blnZeroStateWithNonZeroCount = False;
    //For each colony
    for(intColony=1;intColony<=NCOLONIES;intColony++) {
      dblTruePup = (double) pParticles[intParticle].intNewStates[intColony][1];
      if((dblTruePup==0.0)&(dblPupCount[intColony]>0)) {
        blnZeroStateWithNonZeroCount=True;
        break;
      }
      if((dblPupCount[intColony]>=0)&(dblTruePup>0)) {
        //If observation is not missing then
        // work out the log likelihood for this observation and add to the previous
        // unless the observation is a zero count with zero true pups - then treat
        // like a missing observation (ie don't calculate lnl for this, as it
        // leads to numerical problems, and the likelihhood should be 1 in any case)
        //First, see if an SE has been provided
        dblSE = -1;
        if(blnKnownSEs) {
          dblSE = (double) pControlPF->intPupCountSE[intYear][intColony];
          //Multiply by SE inflation factor - if any
          if(dblSE>0) dblSE=dblSE*pControlPF->dblSEInflationFactor;
        }
        //If not, then use the model to make an SE up
        if (dblSE == -1) {
  #if defined PSI_IS_PRECISION
          dblSE= dblTruePup/dblSqrtPsi;
  #else
          dblSE= dblTruePup*dblSqrtPsi;
  #endif
        }

        dblLogWeight+=logdnorm(dblPupCount[intColony], dblTruePup, dblSE);
      }
    }

    if(blnZeroStateWithNonZeroCount) {
      //One of the true states was zero so set the weight to -DBL_MAX
      // -- exp of this will be zero
      dblWeights[intParticle]=-DBL_MAX;
    } else {
      //Save the log weight
      dblWeights[intParticle]=dblLogWeight;
    }
  }

  if (pControlPF->blnScaleWeights) {

    //If doing scaling...
    //...check if you have to create the scaling
    if(pMeta[intYear].dblWeightScaling1 == (double) MISSINGWEIGHT) {
      //make weight 1 the maximum of the particles    
      pMeta[intYear].dblWeightScaling1 = -DBL_MAX;
      for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
        pMeta[intYear].dblWeightScaling1 =DMAX(pMeta[intYear].dblWeightScaling1,dblWeights[intParticle]);
      //Used to make it the mean of the values (after ignoring -DBL_MAX), but that did not work well
      //Old code: pMeta[intYear].dblWeightScaling1 = 
      //  dmeanVectorExcludeMissing(dblWeights, 1, pControlPF->intNParticles,-DBL_MAX);
    }
    //...do the scaling and exponentiate
    for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
      dblWeights[intParticle]=exp(dblWeights[intParticle]-pMeta[intYear].dblWeightScaling1);

    //...now do a second scaling
    if(pMeta[intYear].dblWeightScaling2 == (double) MISSINGWEIGHT) {
      pMeta[intYear].dblWeightScaling2 =
        dmeanVector(dblWeights, 1, pControlPF->intNParticles);
      pMeta[intYear].dblWeightScaling2 = (pMeta[intYear].dblWeightScaling2>0) ?
        pMeta[intYear].dblWeightScaling2 : 1.0;
    }
    for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
      dblWeights[intParticle] /= pMeta[intYear].dblWeightScaling2;

  } else {
    //If no scaling, just exponentiate
    for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++)
        dblWeights[intParticle]=exp(dblWeights[intParticle]);
  }

  free_dvector(dblPupCount, 1, NCOLONIES);

  if (pControlPF->blnDebugFile) {
    fprintf(pControlPF->pDebugFile, "Summary of weights from data in year %i\n",intYear);
    writeDVectorSummary(pControlPF, dblWeights, 1, pControlPF->intNParticles);
  }

}

/*------------------------------------------------------------------------*/

void samplePsi (controlPFstruct * const pControlPF,
                particlestruct * const pParticles,
                Int intYear)
/* Purpose:
  For each particle, generates a sample for psi parameter from the full conditional,
    conditioning on the other model parameters and states
  Called only when you integrated out the psi parameter
  Called at present right at the end of the filtering, to get sample values for psi
  Assumes that BCurrent contains the correct value from equation n16 from Carmen's 
    5th Dec notes.
  Note that my first year is year 1, so although n16 calls for (t+1) that is just year in
    my case as Carmen uses first year is year 0.  Note also that my value of BCurrent is
    already twice that of Carmen's so I don't need to multiply the 2nd parameter of the gamma
    by 2.
*/
{

  Int intParticle;
  //Shortcuts
  particlestruct *pParticle;
  //parameter from gamma prior on psi param
  double dblPsi1;

  dblPsi1 = pControlPF->dblPriorParams1[psi];

  //For each particle
  for(intParticle=1;intParticle<=pControlPF->intNParticles;intParticle++) {
    pParticle=&pParticles[intParticle];
    //Generate a value from gamma dist
    pParticle->dblParams[psi] = dgamdev(dblPsi1 + 2*(intYear), 
      1/pParticle->dblBCurrent, &glngSeed);
  }

}

/*------------------------------------------------------------------------*/
