/*
  numeric.c - Misc numeric routines written by KBN, LJT and from Numerical Recipes in C
*/

#include "prepro.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#include <malloc.h>
#include "global.h"
#include "nrutil.h"
#include "numeric.h" // so declarations can be out of order if required

/*------------------------------------------------------------------------*/
/* beta.c routine, from Numerical Recipes, p. 216 */

double dbeta(double z, double w)

{
  extern double dgammln();

  return exp(dgammln(z) + dgammln(w) - dgammln(z+w));
}

/*------------------------------------------------------------------------*/
//dbetacf, from Numerical Recipes

#define MAXIT 200
#define EPS 3.0e-7
#define FPMIN 1.0e-30

double dbetacf(double a,double b,double x)
{
 void nrerror();
 Int m, m2;
 double aa,c,d,del,h,qab,qam,qap;

 qab=a+b;
 qap=a+1.0;
 qam=a-1.0;
 c=1.0;
 d=1.0-qab*x/qap;
 if (fabs(d) < FPMIN) d=FPMIN;
 d=1.0/d;
 h=d;
 for(m=1;m<=MAXIT;m++) {
   m2=2*m;
   aa=m*(b-m)*x/((qam+m2)*(a+m2));
   d=1.0+aa*d;
   if (fabs(d) < FPMIN) d=FPMIN;
   c=1.0+aa/c;
   if (fabs(c) <FPMIN) c=FPMIN;
   d=1.0/d;
   h *= d*c;
   aa = -(a+m)*(qab+m)*x/((a+m2)*(qap+m2));
   d=1.0+aa*d;
   if (fabs(d) < FPMIN) d=FPMIN;
   c=1.0+aa/c;
   if (fabs(c) < FPMIN) c=FPMIN;
   d=1.0/d;
   del=d*c;
   h *= del;
   if (fabs(del-1.0) < EPS) break;
  }
  if (m > MAXIT) nrerror("a or b too big, or MAXIT too small in betacf");
  return h;
}

//LJT: I added undefs for these, for safety
#undef MAXIT
#undef EPS
#undef FPMIN

/*------------------------------------------------------------------------*/
//dbetai, from Numerical Recipes

double dbetai(double a,double b,double x)
{
 extern double dbetacf();
 extern double dgammln();
 void nrerror();
 double bt;

 if (x < 0.0 || x > 1.0)  nrerror("Bad x in routine betai");
 if (x == 0.0 || x==1.0) bt=0.0;
 else
  bt = exp(dgammln(a+b) -dgammln(a) - dgammln(b) + a*log(x) + b*log(1.0-x));
 if (x < (a+1.0)/(a+b+2.0))
  return bt*dbetacf(a,b,x)/a;
 else
  return 1.0-bt*dbetacf(b,a,1.0-x)/b;
}

/*------------------------------------------------------------------------*/
//drbinom, renamed from dbnldev, from Numerical Recipes, and arguments re-arranged so
// it's in the same order as drnbinom

#define PI 3.141592654

double drbinom(Int n, double pp, long *idum)
{
	double dgammln(double xx);
  double dran1();
	/* double ran1(long *idum); */
	Int j;
	static Int nold=(-1);
	double am,em,g,angle,p,bnl,sq,t,y;
	static double pold=(-1.0),pc,plog,pclog,en,oldg;

	p=(pp <= 0.5 ? pp : 1.0-pp);
	am=n*p;
	if (n < 25) {
		bnl=0.0;
		for (j=1;j<=n;j++)
			if (dran1(idum) < p) ++bnl;
	} else if (am < 1.0) {
		g=exp(-am);
		t=1.0;
		for (j=0;j<=n;j++) {
			t *= dran1(idum);
			if (t < g) break;
		}
		bnl=(j <= n ? j : n);
	} else {
		if (n != nold) {
			en=n;
			oldg=dgammln(en+1.0);
			nold=n;
		} if (p != pold) {
			pc=1.0-p;
			plog=log(p);
			pclog=log(pc);
			pold=p;
		}
		sq=sqrt(2.0*am*pc);
		do {
			do {
				angle=PI*dran1(idum);
				y=tan(angle);
				em=sq*y+am;
			} while (em < 0.0 || em >= (en+1.0));
			em=floor(em);
			t=1.2*sq*(1.0+y*y)*exp(oldg-dgammln(em+1.0)
				-dgammln(en-em+1.0)+em*plog+(en-em)*pclog);
		} while (dran1(idum) > t);
		bnl=em;
	}
	if (p != pp) bnl=n-bnl;
	return bnl;
}
#undef PI

/*------------------------------------------------------------------------*/
//dgamdev, from Numerical Recipes
//Not using this one -- see next sub instead

/*
double dgamdev(ia,idum)
  Int ia;
  long *idum;
{
	double dran1();
	void nrerror();
	Int j;
	double am,e,s,v1,v2,x,y;

	if (ia < 1) nrerror("Error in routine gamdev");
	if (ia < 6) {
		x=1.0;
		for (j=1;j<=ia;j++) x *= dran1(idum);
		x = -log(x);
	} else {
		do {
			do {
				do {
					v1=dran1(idum);
					v2=2.0*dran1(idum)-1.0;
				} while (v1*v1+v2*v2 > 1.0);
				y=v2/v1;
				am=ia-1;
				s=sqrt(2.0*am+1.0);
				x=s*y+am;
			} while (x <= 0.0);
			e=(1.0+y*y)*exp(am*log(x/am)-s*y);
		} while (dran1(idum) > e);
	}
	return x;
}
*/

/*------------------------------------------------------------------------*/
//dgamdev - from KBN as mygamdev
// patched to propertly deal with alpha==1 30 Aug 2005

 double dgamdev(double alpha,double beta,long *seed)
  {
 double b,c,u,v,w,z,e,p,x,y;
 Int undone;

if(alpha<1.0) {

  //the following line added by LJT to avoid an infinite loop
  // in some compilers with alpha==0
  if(alpha==0) return(0.0);

  undone=1;
   while(undone==1) {
     e = exp(1.0);
     b = (e+alpha)/e;
     p = b*dran1(seed);
     if(p<1.0) {
      x = exp(1/alpha*log(p));
      if(dran1(seed) <= exp(-x))
       {
        undone=0;
        return(x*beta);
       }
     }
     else {
        x = -log((b-p)/alpha);
        if(dran1(seed) <= exp((alpha-1.0)*log(x))) {
         undone=0;
         return(x*beta);
        }
     }
    }
  } else {
    if (alpha > 1.0) {

      /* Best (1978) algo- p 410 of Devroye */
      b = alpha-1.0;
      c = 3.0*alpha - 0.75;
      undone=1;
      while(undone==1) {
        u = dran1(seed);
        v = dran1(seed);
        w = u*(1-u);
        y = sqrt(c/w)*(u-0.5);
        x = b + y;
        if(x >= 0) {
           z = 64*exp(3*log(w))*v*v;
           if(z <= 1-2*y*y/x) {
             undone=0;
             return(x*beta);
            }
           else {
            if(log(z) <= 2*(b*log(x/b)-y)) {
             undone=0;
             return(x*beta);
            }
           }
         }
      }

    } else {
      //Alpha==1
      return(-beta*log(1.0-dran1(seed)));
    }
  }

  //The following lines added by LJT 3/8/05
  printf("Error - got to end of mygamdev subroutine.\n");
  exit(1);
  return(0);
 }

/*------------------------------------------------------------------------*/
//rnbinom - from KBN (as rnbiom)

 double drnbinom(double n,double p,long *seed)
 // see Devroye pp 488-489
 {
  double Y;
  Y = dgamdev(n,(1.0-p)/p,seed);
  return(drpois(Y,seed));
 }

/*------------------------------------------------------------------------*/
//dgammln, from Numerical Recipes

double dgammln(double xx)
{
	double x,tmp,ser;
	static double cof[6]={76.18009173,-86.50532033,24.01409822,
		-1.231739516,0.120858003e-2,-0.536382e-5};
	Int j;

	x=xx-1.0;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.0;
	for (j=0;j<=5;j++) {
		x += 1.0;
		ser += cof[j]/x;
	}
	return -tmp+log(2.50662827465*ser);
}

/*------------------------------------------------------------------------*/
//drnorm - adapted from dgasdev, from Numerical Recipes
// mean and sd arguments added by LJT
// modified by LJT 5/8/06 so it doesn't occasionally return nan (as reported by KBN)


double drnorm(double mean, double sd, long *idum)
{
	static Int iset=0;
	static double gset;
	double fac,r,v1,v2,res;
	double dran1();

	if  (iset == 0) {
		do {
			v1=2.0*dran1(idum)-1.0;
			v2=2.0*dran1(idum)-1.0;
			r=v1*v1+v2*v2;
		} while (r >= 1.0);
		fac=sqrt(-2.0*log(r)/r);
		gset=v1*fac;
		iset=1;
    res=v2*fac*sd+mean;
	} else {
		iset=0;
    res=gset*sd+mean;
	}
  //use a recursive call to keep sampling values until not nan
  // only happens very rarely so you're unlikely to get stack overflow
  if(isnan(res)) res=drnorm(mean, sd, idum);
  return (res);
}

/*------------------------------------------------------------------------*/
//drpois, from dpoidev from Numerical Recipes
//returns as a double a random Poisson value

#define PI 3.141592654

double drpois(double xm, long *idum)
{
	double dgammln();
	double dran1();
	static double sq,alxm,g,oldm=(-1.0);
	double em,t,y;

	if (xm < 12.0) {
		if (xm != oldm) {
			oldm=xm;
			g=exp(-xm);
		}
		em = -1;
		t=1.0;
		do {
			++em;
			t *= dran1(idum);
		} while (t > g);
	} else {
		if (xm != oldm) {
			oldm=xm;
			sq=sqrt(2.0*xm);
			alxm=log(xm);
			g=xm*alxm-dgammln(xm+1.0);
		}
		do {
			do {
				y=tan(PI*dran1(idum));
				em=sq*y+xm;
			} while (em < 0.0);
			em=floor(em);
			t=0.9*(1.0+y*y)*exp(em*alxm-dgammln(em+1.0)-g);
		} while (dran1(idum) > t);
	}
	return em;
}
#undef PI

/*------------------------------------------------------------------------*/
//dran1, from Numerical Recipes

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

double dran1(long *idum)
{
        Int j;
        long k;
        static long iy=0;
        static long iv[NTAB];
        double temp;

        if (*idum <= 0 || !iy) {
                if (-(*idum) < 1) *idum=1;
                else *idum = -(*idum);
                for (j=NTAB+7;j>=0;j--) {
                        k=(*idum)/IQ;
                        *idum=IA*(*idum-k*IQ)-IR*k;
                        if (*idum < 0) *idum += IM;
                        if (j < NTAB) iv[j] = *idum;
                }
                iy=iv[0];
        }
        k=(*idum)/IQ;
        *idum=IA*(*idum-k*IQ)-IR*k;
        if (*idum < 0) *idum += IM;
        j=iy/NDIV;
        iy=iv[j];
        iv[j] = *idum;
        if ((temp=AM*iy) > RNMX) return RNMX;
        else return temp;
}
#undef IA
#undef IM
#undef AM
#undef IQ
#undef IR
#undef NTAB
#undef NDIV
#undef EPS
#undef RNMX

/*------------------------------------------------------------------------*/
//drbeta

/*
    Lifted and modified from R by KbN- 6 October 2004
    3 October 2005, put the v_w_from_u1_bet macro
        back in (fixes w=DBL_MAX if v > expmax)
    5 October 2005, put different bound check in the
       above macro

 *  R : A Computer Language for Statistical Data Analysis
 */

/* Reference:
 * R. C. H. Cheng (1978).
 * Generating beta variates with nonintegral shape parameters.
 * Communications of the ACM 21, 317-322.
 * (Algorithms BB and BC)
 */

// #include "nmath.h" - an R header

//#include <math.h>
//#include <float.h>

#ifndef M_LN2
#define M_LN2           0.693147180559945309417232121458        /* ln(2) */
#endif

//DMAX and DMIN defined elsewhere

double drbeta(double aa, double bb, long *seed)
{
    double a, b, alpha;
    double r, s, t, u1, u2, v, w, y, z;
    double dran1(long *seed);

    int qsame;
    /* FIXME:  Keep Globals (properly) for threading */
    /* Uses these GLOBALS to save time when many rv's are generated : */
    static double beta, gamma, delta, k1, k2;
    static double olda = -1.0;
    static double oldb = -1.0;

#define expmax (DBL_MAX_EXP * M_LN2) /* = log(DBL_MAX) */

#define v_w_from__u1_bet(AA)                    \
            v = beta * log(u1 / (1.0 - u1));    \
            w = AA * exp(v);                \
            if ((log(AA)+v) <= expmax)                    \
                w = AA * exp(v);                \
            else                                \
                w = DBL_MAX

    if (aa <= 0. || bb <= 0.) {
       printf("Error in rbeta, nonpos params a=%5.4lf b=%5.4lf \n",aa,bb);
       printf("stop ");scanf("%d",&qsame);
    }

    /* Test if we need new "initializing" */
    qsame = (olda == aa) && (oldb == bb);
    if (!qsame) { olda = aa; oldb = bb; }

    a = DMIN(aa, bb);
    b = DMAX(aa, bb); /* a <= b */
    alpha = a + b;

    if (a <= 1.0) {	/* --- Algorithm BC --- */

	/* changed notation, now also a <= b (was reversed) */

	if (!qsame) { /* initialize */
	    beta = 1.0 / a;
	    delta = 1.0 + b - a;
	    k1 = delta * (0.0138889 + 0.0416667 * a) / (b * beta - 0.777778);
	    k2 = 0.25 + (0.5 + 0.25 / delta) * a;
	}
	/* FIXME: "do { } while()", but not trivially because of "continue"s:*/
	for(;;) {
	    u1 = dran1(seed);
	    u2 = dran1(seed);
	    if (u1 < 0.5) {
		y = u1 * u2;
		z = u1 * y;
		if (0.25 * u2 + z - y >= k1)
		    continue;
	    } else {
		z = u1 * u1 * u2;
		if (z <= 0.25) {
		     v_w_from__u1_bet(b);
		    break;
		}
		if (z >= k2)
		    continue;
	    }
             v_w_from__u1_bet(b);

	    if (alpha * (log(alpha / (a + w)) + v) - 1.3862944 >= log(z))
		break;
	}
        if(isnan(a/(a+w)) || isnan(w/(a+w))) {
           printf("w=%7.5lf  a=%7.5lf b=%7.5lf \n",w,a,b);
           printf("beta=%7.5lf u1=%7.5lf v=%7.5lf expmax=%lf b*exp(v)=%lf \n",
            beta,u1,v,expmax,b*exp(v));
           printf("(log(b)+v) <= expmax %d \n",(log(b)+v)<=expmax);
           printf("stop ");scanf("%lf",&v);
         }

	return (aa == a) ? a / (a + w) : w / (a + w);

    }
    else {		/* Algorithm BB */

	if (!qsame) { /* initialize */
	    beta = sqrt((alpha - 2.0) / (2.0 * a * b - alpha));
	    gamma = a + 1.0 / beta;
	}
	do {
	    u1 = dran1(seed);
	    u2 = dran1(seed);
	    v_w_from__u1_bet(a);

	    z = u1 * u1 * u2;
	    r = gamma * v - 1.3862944;
	    s = a + r - w;
	    if (s + 2.609438 >= 5.0 * z)
		break;
	    t = log(z);
	    if (s > t)
		break;
	}
	while (r + alpha * log(alpha / (b + w)) < t);
	return (aa != a) ? b / (b + w) : w / (b + w);
    }
}



#undef expmax
#undef M_LN2
#undef v_w_from__u1_bet

/*------------------------------------------------------------------------*/

//qgaus, from Numerical Recipes

float qgaus(float (*func)(float), float a, float b)
{
	Int j;
	float xr,xm,dx,s;
	static float x[]={(float) 0.0, (float) 0.1488743389,(float) 0.4333953941,
		(float) 0.6794095682,(float) 0.8650633666,(float) 0.9739065285};
	static float w[]={(float) 0.0,(float) 0.2955242247,(float) 0.2692667193,
		(float) 0.2190863625,(float) 0.1494513491,(float) 0.0666713443};

	xm=(float) 0.5*(b+a);
	xr=(float) 0.5*(b-a);
	s=0;
	for (j=1;j<=5;j++) {
		dx=xr*x[j];
		s += w[j]*((*func)(xm+dx)+(*func)(xm-dx));
	}
	return s *= xr;
}

/*------------------------------------------------------------------------*/
//choldc, from Numerical Recipes

void choldc(double **a,Int n,double *p)

/* Given a pos. def. symmetric matrix a[1..n][1..n],
this routine constructs its Cholesky decomposition,
A=LL'.  On input, only the upper triangle of a need be given; it is
not modified.  The Cholesky factor L is returned in the lower triangle
of a, except for its diagonal elements which are returned in p[1..n].
*/

{
  extern void nrerror();
  Int i,j,k;
  double sum=0.0;

  for (i=1;i<=n;i++) {
   for (j=i;j<=n;j++) {
     for (sum=a[i][j],k=i-1;k>=1;k--) sum -= a[i][k]*a[j][k];
     if(i==j) {
       if(sum<=0.0)  /*a, with rounding errors, is not p.d. */
         nrerror("choldc failed- not p.d. ");
       p[i]=sqrt(sum);
     }
     else a[j][i]=sum/p[i];
   }
  }
}

/*------------------------------------------------------------------------*/
//Brent - Numerical recipes page 404
//Brent's routine
/* Given a funciton f, and given a bracketing triplet of abscissas ax, bx, cx
  ( such that bx is between ax and cx and f(bx) is less than both f(ax) and f(cx),
  this routine isolates the minimum to a fractional precision of about tol using
  Brent's method.  The abcissa of the minimu m is returned as xmin and the
  minimum  function value is returned as brent, the returned function value
*/
//Note - I left it as float as I'm only using it to find the number of 6+ adults --
// an integer, so float accuracy is fine! LJT 14/8/05

//Note - modified so you also pass in an array of other parameters and an integer giving the
// number of other parameters

#define ITMAX 1000
#define CGOLD 0.3819660
#define ZEPS 1.0e-10
#define SHFT(a,b,c,d) (a)=(b);(b)=(c);(c)=(d);

float brent(float ax, float bx, float cx, float (*f)(float, double *, int), float tol,
	float *xmin, double *otherparams, Int notherparams)
{
	int iter;
	float a,b,d,etemp,fu,fv,fw,fx,p,q,r,tol1,tol2,u,v,w,x,xm;
	float e=0.0;

	a=(ax < cx ? ax : cx);
	b=(ax > cx ? ax : cx);
	x=w=v=bx;
	fw=fv=fx=(*f)(x, otherparams, notherparams);
	for (iter=1;iter<=ITMAX;iter++) {
		xm=(float) 0.5*(a+b);
		tol2=(float) 2.0*(tol1=tol*(float)fabs(x)+(float)ZEPS);
		if (fabs(x-xm) <= (tol2-(float)0.5*(b-a))) {
			*xmin=x;
			return fx;
		}
		if (fabs(e) > tol1) {
			r=(x-w)*(fx-fv);
			q=(x-v)*(fx-fw);
			p=(x-v)*q-(x-w)*r;
			q=(float) 2.0*(q-r);
			if (q > (float) 0.0) p = -p;
			q=(float)fabs(q);
			etemp=e;
			e=d;
			if (fabs(p) >= fabs((float) 0.5*q*etemp) || p <= q*(a-x) || p >= q*(b-x))
				d=(float) CGOLD*(e=(x >= xm ? a-x : b-x));
			else {
				d=p/q;
				u=x+d;
				if (u-a < tol2 || b-u < tol2)
					d=(float)SIGN(tol1,xm-x);
			}
		} else {
			d=(float) CGOLD*(e=(x >= xm ? a-x : b-x));
		}
		u=(fabs(d) >= tol1 ? x+d : x+(float) SIGN(tol1,d));
		fu=(*f)(u, otherparams, notherparams);
		if (fu <= fx) {
			if (u >= x) a=x; else b=x;
			SHFT(v,w,x,u)
			SHFT(fv,fw,fx,fu)
		} else {
			if (u < x) a=u; else b=u;
			if (fu <= fw || w == x) {
				v=w;
				w=u;
				fv=fw;
				fw=fu;
			} else if (fu <= fv || v == x || v == w) {
				v=u;
				fv=fu;
			}
		}
	}
	nrerror("Too many iterations in brent");
	*xmin=x;
	return fx;
}
#undef ITMAX
#undef CGOLD
#undef ZEPS
#undef SHFT
#undef NRANSI

/*------------------------------------------------------------------------*/
//dround - from myround by KBN

 double dround(double x)
 {
  double temp,temp1, out;

  temp = modf(x,&temp1);
  if(temp <= 0.5)  out = floor(x); else out = ceil(x);
  return(out);
 }

/*------------------------------------------------------------------------*/
//iround - from myround by KBN

 Int iround(double x)
 {
  double temp,temp1, out;

  temp = modf(x,&temp1);
  if(temp <= 0.5)  out = floor(x); else out = ceil(x);
  return((Int) out);
 }

 /*------------------------------------------------------------------------*/
//dlnorm - from KBN

 double dlnorm(double y,double mu,double sigma)
 {
  double temp;

  temp = (log(y)-mu)/sigma;
  temp = 1.0/(sigma*y)*exp(-0.5*temp*temp);
  return(temp);
}

/*------------------------------------------------------------------------*/
//dnorm - from KBN
// note - constant term 1/2PI ommited as is only used in a relative sense

 double dnorm(double y,double mu,double sigma)
 {
  double temp;

  temp = (y-mu)/sigma;
  temp = (1.0/sigma)*exp(-0.5*temp*temp);
  return(temp);
}

/*------------------------------------------------------------------------*/
//lndnorm - LJT - returns the log(dnorm) - more efficient than taking log(dnorm)
// as avoids exponentiating and then logging again
// note - constant term 1/2PI ommited

 double logdnorm(double y,double mu,double sigma)
 {
  double temp;

  temp = (y-mu)/sigma;
  temp = log(1.0/sigma)-0.5*temp*temp;
  return(temp);
}

/*------------------------------------------------------------------------*/
//drunif - from KBN (as runif)

 double drunif(double a,double b,long *idum)
  {
   double out;

   out = dran1(idum)*(b-a)+a;
   return(out);
  }

/*------------------------------------------------------------------------*/

double dmeanVector(const double * const y, Int start, Int end)
//Returns the mean of a vector y[start..end]
{
  Int i;
  double dblN; //number of elements, as a double, saves converting from int
  double dblMean=0.0;

  dblN = (double)(end-start+1);
  if(dblN==0.0) return(0);

  for (i=start; i<=end; i++) dblMean+=y[i]/dblN;

  return(dblMean);
}

/*------------------------------------------------------------------------*/

double dmeanVectorExcludeMissing(const double * const y, Int start, Int end, double missingValue)
//Returns the mean of a vector y[start..end]
//  Values of y[] that are equal to missingValue are not included
{
  Int i, intNMissing=0;
  double dblN; //number of elements (excluding missing),
               // as a double, saves converting from int
  double dblMean=0.0;

  dblN = (double) (end-start+1);
  if(dblN==0.0) return(0);
  for (i=start; i<=end; i++) {
    if(y[i]==missingValue) {
      intNMissing ++;
    } else {
      dblMean+=y[i]/dblN;
    }
  }
  if(intNMissing>0) {
    if(intNMissing==(Int)dblN) return(0);
    dblMean=dblMean/(double)(end-start+1-intNMissing)*dblN;
  }
  return(dblMean);
}

/*------------------------------------------------------------------------*/

double dsumVector(const double * const y, Int start, Int end)
//Returns the sum of a vector y[start..end]
//At present, I don't do anything fancy like sort the vector before taking the
// sum -- if I did it would be more accurate (but slower)
{
  Int i;
  double dblN; //number of elements, as a double, saves converting from int
  double dblSum=0.0;

  dblN = (double)(end-start+1);
  if(dblN==0.0) return(0);

  for (i=start; i<=end; i++) dblSum+=y[i];

  return(dblSum);
}

/*------------------------------------------------------------------------*/

void dstandardizeVectorToOut(const double * const y, Int start, Int end,
  double * const out, double total)
//Returns the vector out[start..end] so that it is y[start..end] standardized
// to sum to the value in total
//At present, I don't do anything fancy like sort the vector before taking the
// sum -- if I did it would be more accurate (but slower)
//One trick, I tried however, is to standardize all except the last one, and then
// the last one gets standardized to what's left.  Didn't work, so took it out!
{
  Int i;
  double dblSum,dblStd;

  if(start>=end) return;

  dblSum=dsumVector(y, start, end);
  if(dblSum==0) return;
  dblStd = total/dblSum;
  //standardize
  for (i=start; i<=end; i++) {
    out[i]=y[i]*dblStd;
    dblSum-=out[i];
  }
  return;
}

/*------------------------------------------------------------------------*/

boolean areElementsIdentical(const double * const y, Int start, Int end)
//Returns true if all elements of the vector y are identical, to within TOL
{
#define TOL 1E-10

  Int i;
  boolean blnAreDifferent;

  blnAreDifferent = False;
  for (i=start;i<end;i++) {
    if((y[i]+TOL<y[i+1])|(y[i]-TOL>y[i+1])) {
      blnAreDifferent=True;
      break;
    }
  }
  return(!blnAreDifferent);
}

/*------------------------------------------------------------------------*/

/* K. Newman, 8 Oct 2004: Alias algorithm for generating samples
        from arbitrary discrete rv dist'ns
   amended by LJT 7/Aug/06
*/

void alias(Int numsims, Int sampsize, double *probs, Int *sample, long *seed) {

 // 1 Sept 2005- sample needs to be cleared between calls?
 //19 Aug 2006 - memory leak fixed LJT
 //30 march 2008 - added code to check if all probs are the same
 //  as was running into problems otherwise with the probs not quite
 //  adding up to 1.

  struct alias_node {
   Int value;
   struct alias_node * next;
  };
  typedef struct alias_node llists;

  Int    i=1,j=1, k=1, *A, Y;
  double *Q, Z, temp;
  llists *H, *L, *headH, *headL;

  if(areElementsIdentical(probs, 1, numsims)) {
    //use simpler algorithm
    for(i =1; i<= sampsize;i++) {
     temp = dran1(seed)*numsims+1.0;
     sample[i] = (Int) floor(temp);
    }

  } else {
    //elements not identical - do alias proper

    Q     = dvector(1,numsims);
    A     = ivector(1,numsims);
    for(i=1;i<=numsims;i++) {
    Q[i] = 0.0;
    A[i] = 0;
    //This line is here in error I think, so I commented it out
    // Was causing a crash when sampsize less than numsims - LJT
    //  sample[i] = 0;
    }

    headH = (llists *) malloc(sizeof(llists));
    headL = (llists *) malloc(sizeof(llists));
    headH = NULL; headL = NULL;

    for(i=1;i<=numsims;i++) {
    Q[i] = numsims*probs[i];
    if(Q[i] < 1.0) {
         L = (llists *) malloc(sizeof(llists));
         L->value = i;
         L->next  = headL;
         headL    = L;
     } else {
         H = (llists *) malloc(sizeof(llists));
         H->value = i;
         H->next  = headH;
         headH    = H;
     }
    }

    // while(L != NULL) {  WHAT DEVROYE's ALGO "Says"???
    while(H != NULL && L != NULL) {
     j = L->value;
     k = H->value;
     A[j] = k;
     Q[k] = Q[k]-(1.0-Q[j]);
     if(Q[k] < 1.0) {
        H = H->next;
        //if(H != NULL) prIntf(" now H=%d \n",H->value);
        //if(H == NULL) prIntf("H is empty \n");
        L->value = k;
       } else {
        L = L->next;
        // if(L != NULL) prIntf(" H=%d new L=%d \n",H->value, L->value);
        // if(L == NULL) prIntf("L is empty \n");
       }
    }


    //Len code to free memory
    H=headH;
    while(H != NULL) {
      headH = H->next;
      free(H);
      H=headH;
    }
    free(headH);
    L=headL;
    while(L != NULL) {
      headL = L->next;
      free(L);
      L=headL;
    }
    free(headL);
    //Ken code to free memory
    // /* -- free the H memory list --- */
    //  while(H != NULL) {
    //    headH = H->next;
    //    free(H);
    //    H = headH;
    //  }
    //  free(headH);
    //  free(headL);

    /* Done creating Q and A list: now take a sample */
    for(i =1; i<= sampsize;i++) {
     temp = dran1(seed)*numsims+1.0;
     Y = (Int) floor(temp);
     Z = temp - Y;
     if(Z <= Q[Y]) sample[i]=Y; else sample[i]=A[Y];
    }

    free_dvector(Q,1,numsims);
    free_ivector(A,1,numsims);

  } //end elements not identical

}

/*------------------------------------------------------------------------*/
/* 2-Pass algorithm for Covariance matrix calculation */
/* Adapted from code by Ken Newman */

void covMat(Int numpars, Int numsims, double a_wt, double **theta_star,
  double *mean_theta_star, double **mean_mat,  double **sigma_theta_star)
/*
Inputs:
  numpars is number of parameters in theta_star
  numsims is number of particles in theta_star
  a_wt is the kernal smoothing weight (calculated from the discount)
  theta_star is [1 .. numsims] [1 .. numpars]
Outputs:
  mean_theta_star is the mean vector [1 .. numpars]
  mean_mat is a matrix of theta_star - (1-a_wt)*mean_theta_star [1 .. numsims] [1 .. numpars]
  sigma_theta_star is the variance covariance matrix [1 .. numpars] [1 .. numpars]
*/
{
  Int i=1, j=1, k=1;
  // storage space
  double *sum_dev, **sum_xprod_mat;

  //reserve space
  sum_dev = dvector(1, numpars);
  sum_xprod_mat = dmatrix(1, numpars, 1, numpars);

  /* calculate Mean vector of theta star */
  for(i=1; i <= numpars; i++)
    mean_theta_star[i] = 0.0;
  for(i=1; i <= numsims; i++)
    for(j=1; j <= numpars; j++)
     mean_theta_star[j] += (1.0/numsims)*theta_star[i][j];

  /* calculate Wt'd Mean matrix of theta star */
  for(i=1; i<= numsims; i++)
    for(j=1; j<= numpars; j++)
        mean_mat[i][j] =  a_wt*theta_star[i][j] + (1.0-a_wt)*mean_theta_star[j];

  for(i=1; i <= numpars; i++) {
    sum_dev[i] = 0.0;
    for(j=1; j <= numpars; j++) {
      sigma_theta_star[i][j] = 0.0;
      sum_xprod_mat[i][j] = 0.0;
    }
  }

  for(i=1; i<= numsims; i++) for(j=1;j<=numpars;j++)
    sum_dev[j] += (theta_star[i][j]-mean_theta_star[j]);
  for(i=1; i<= numpars; i++) {
    for(j=1;j<=i;j++) {
      for(k=1;k<=numsims;k++)
        sum_xprod_mat[i][j] += (theta_star[k][i]-mean_theta_star[i])*
                               (theta_star[k][j]-mean_theta_star[j]);
    }
  }
  for(i=1; i<= numpars; i++)
    for(j=1; j<i;j++)
      sum_xprod_mat[j][i] = sum_xprod_mat[i][j];

  for(i=1; i<=numpars; i++) for(j=1; j<=i; j++)
    sigma_theta_star[i][j] = (sum_xprod_mat[i][j] - sum_dev[i]*sum_dev[j]/numsims)/
                             (numsims-1.0);
  for(i=1; i<= numpars; i++)
    for(j=1; j<i;j++)
      sigma_theta_star[j][i] = sigma_theta_star[i][j];

  //free space
  free_dvector(sum_dev, 1, numpars);
  free_dmatrix(sum_xprod_mat, 1, numpars, 1, numpars);
}

/*------------------------------------------------------------------------*/

/* Algorithm for weighted covariance matrix calculation */
/* From code by Ken Newman */
void wt_covMat(Int numpars, Int numsims, double a_wt, double **theta_star,
   double *mean_theta_star, double **mean_mat, double **sigma_theta_star,
   double *wts)
/*
Inputs:
  numpars is number of parameters in theta_star
  numsims is number of particles in theta_star
  a_wt is the kernal smoothing weight (calculated from the discount)
  theta_star is [1 .. numsims] [1 .. numpars]
  wts is a vector [1..numsims] of weights for each particle
 Outputs:
  mean_theta_star is the mean vector [1 .. numpars]
  mean_mat is a matrix of theta_star - (1-a_wt)*mean_theta_star [1 .. numsims] [1 .. numpars]
  sigma_theta_star is the variance covariance matrix [1 .. numpars] [1 .. numpars]

*/
{
  Int i=1, j=1, k=1;
  double *scale_wts;
  double temp=0.0;

  // reserve space
  scale_wts = dvector(1,numsims);

  // scaled wts
  for(i=1;i<=numsims;i++) temp += wts[i];
  for(i=0;i<=numsims;i++) scale_wts[i] = wts[i]/temp;

  /* Calculate (weighted) mean vector of theta star */
  for(i=1;i<=numpars; i++)
    mean_theta_star[i] = 0.0;
  for(i=1;i<=numsims; i++)
    for(j=1;j<=numpars;j++)
      mean_theta_star[j] += scale_wts[i]*theta_star[i][j];

  /* Calculate wt'd mean matrix of theta star */
  for(i=1; i<= numsims; i++)
    for(j=1; j <= numpars; j++)
      mean_mat[i][j] = a_wt*theta_star[i][j] + (1.0-a_wt)*mean_theta_star[j];

  // Weighted covariance matrix calculation
  for(i=1;i<=numpars;i++) for(j=1;j<=numpars;j++) sigma_theta_star[i][j]=0.0;
  for(i=1;i<=numpars;i++)
    for(j=1;j<=i;j++)
      for(k=1;k<=numsims;k++)
        sigma_theta_star[i][j] += (theta_star[k][i]-mean_theta_star[i])*
                                  (theta_star[k][j]-mean_theta_star[j])*
                                  scale_wts[k];

  for(i=1;i<=numpars;i++)
    for(j=1;j<i;j++) sigma_theta_star[j][i] = sigma_theta_star[i][j];

  // free space
  free_dvector(scale_wts, 1, numpars);
}

 /*------------------------------------------------------------------------*/

