/*
  util.c - Utility routines for SealPDF
*/

#include "prepro.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
#include <float.h>
#include <math.h>
#include "global.h"
#include "nrutil.h"
#include "numeric.h"
#include "util.h"

/*------------------------------------------------------------------------*/

void SkipLines (FILE *pFile, long NLines)
//Moves pointer along NLines in pFile
{

  char line[MAXSTR];
  long i;

  for(i=0;i<NLines;i++){
    fgets(line,MAXSTR,pFile);
  }
}

/*------------------------------------------------------------------------*/

Int GetIntFromControlFile (FILE *pFile)
/* Purpose: Returns the integer value that occurs starting in column
     CONTROL_FILE_INPUT_COLUMN of the next row of pFile */
{
  char line[MAXSTR];
  Int i;

  fgets(line,MAXSTR,pFile);
  i=atoi(&line[CONTROL_FILE_INPUT_COLUMN-1]);
  return i;
}

/*------------------------------------------------------------------------*/

long GetLongFromControlFile (FILE *pFile)
/* Purpose: Returns the long value that occurs starting in column
     CONTROL_FILE_INPUT_COLUMN of the next row of pFile */
{
  char line[MAXSTR];
  long i;

  fgets(line,MAXSTR,pFile);
  i=atol(&line[CONTROL_FILE_INPUT_COLUMN-1]);
  return i;
}

/*------------------------------------------------------------------------*/

boolean GetBooleanFromControlFile (FILE *pFile)
/* Purpose: Returns the boolean value that occurs starting in column
     CONTROL_FILE_INPUT_COLUMN of the next row of pFile.
     Booleans in the file are actually strings that start with T,t,Y,y (1) or
     F,f,N or n (0). boolean value returned is actually an unsigned int,
     just aliased using typedef in util.h
*/
{
  char line[MAXSTR];

  fgets(line,MAXSTR,pFile);
  switch (tolower(line[CONTROL_FILE_INPUT_COLUMN-1])) {
    case 't': return(1);
    case 'y': return(1);
    case 'f': return(0);
    case 'n': return(0);
    default: printf("The following boolean value not recognized:\n%s\n",line);exit(1);
  }

}

/*------------------------------------------------------------------------*/

double GetDoubleFromControlFile (FILE *pFile)
/* Purpose: Returns the long value that occurs starting in column
     CONTROL_FILE_INPUT_COLUMN of the next row of pFile */
{
  char line[MAXSTR];
  double d;

  fgets(line,MAXSTR,pFile);
  d=atof(&line[CONTROL_FILE_INPUT_COLUMN-1]);
  return d;
}

/*------------------------------------------------------------------------*/

void CopyStringFromControlFile (FILE *pFile, char * const strPutInto)
/* Purpose: copies the string that occurs starting in column
     CONTROL_FILE_INPUT_COLUMN of the next row of pFile to strPutInto*/
{
  char line[MAXSTR];
  char chrLineEnd;
  char *result;
  Int i;

  fgets(line,MAXSTR,pFile);

  /* Check for any trailing new line chars */
  chrLineEnd='\n';
  result=strchr(&line[CONTROL_FILE_INPUT_COLUMN-1], chrLineEnd);
  if(!(result==NULL)) {
    /* If found, remove them */
    i=result-&line[CONTROL_FILE_INPUT_COLUMN-1]+1;
    strncpy(strPutInto,&line[CONTROL_FILE_INPUT_COLUMN-1], i);
    strPutInto[i-1]='\0';
  } else {
    /* Not found, so no problem */
    strcpy(strPutInto, &line[CONTROL_FILE_INPUT_COLUMN-1]);
  }
}

/*------------------------------------------------------------------------*/

particlestruct * mallocParticleStruct (Int intNParticles, Int intNAges,
                                 Int intNColonies, Int intNParams)
/* Purpose: Reserves memory for a created particle structure */
{
    particlestruct *pParticles;
    Int i;

    pParticles = particlevector(1, intNParticles);
    for(i=1;i<=intNParticles;i++) {
      pParticles[i].intOldStates=imatrix(1,intNColonies,1,intNAges);
      pParticles[i].intNewStates=imatrix(1,intNColonies,1,intNAges);
      pParticles[i].intTauPups=ivector(1,intNColonies);
      //Note - dblParams is base 0, so it's the same as the constants
      // cstrParamNames, the enum paramNames, etc
      pParticles[i].dblParams=dvector(0,intNParams-1);
    }
    return(pParticles);
}

/*------------------------------------------------------------------------*/

void freeParticleStruct (particlestruct *pParticles, Int intNParticles,
                         Int intNAges, Int intNColonies,
                         Int intNParams)
// Purpose: Frees memory for a created particle structure
{

  Int i;

  for(i=1;i<=intNParticles;i++) {
    free_imatrix(pParticles[i].intOldStates,1,intNColonies,1,intNAges);
    free_imatrix(pParticles[i].intNewStates,1,intNColonies,1,intNAges);
    free_ivector(pParticles[i].intTauPups,1,intNColonies);
    //Note - dblParams is base 0, so it's the same as the constants
    // cstrParamNames, the enum paramNames, etc
    free_dvector(pParticles[i].dblParams,0,intNParams-1);
  }
  free_particlevector(pParticles,1,intNParticles);

}

/*------------------------------------------------------------------------*/

char *cdifftime (time_t finish, time_t start)
//Given input start and finish times, returns the elapsed time as a string
// in the format x minutes y seconds
{
  //strDiff needs to be declared as static so it isn't overwritten when
  // the routine returns
  static char strDiff[MAXSTR];
  long diff, minutes, seconds;
  diff = (long) difftime(finish, start);
  minutes = diff / 60;
  seconds = diff % 60;
  sprintf(strDiff, "%ld minutes %ld seconds", minutes, seconds);
  return(strDiff);
}

/*------------------------------------------------------------------------*/

void writeParamsSummary(controlPFstruct * const pControlPF,
                      particlestruct * const pParticles)
/* Purpose: Writes a summary of the current parameter values in pParticles to
     the debug file
*/
{
  Int intParticle, intParam, intNParticles;
  double dblParamMean[NPARAMS];
  double dblParamMax[NPARAMS];
  double dblParamMin[NPARAMS];
  double dblAdj[NPARAMS], dblSSE[NPARAMS], dblParamSD[NPARAMS];
  double dblParamValue, dblDiff; //shortcuts

  //don't do anything if there's no debug file
  if (pControlPF->pDebugFile==NULL) return;

  //initialize
  intNParticles = pControlPF->intNParticles;
  if(intNParticles==0) {
    fprintf(pControlPF->pDebugFile, "  No elements to summarize");
    return;
  }
  for(intParam=0;intParam<NPARAMS;intParam++) {
    dblParamMean[intParam]=0.0;
    dblAdj[intParam]=0.0;
    dblSSE[intParam]=0.0;
    dblParamMax[intParam]=-DBL_MAX;
    dblParamMin[intParam]=DBL_MAX;
  }

  //Get the mean, max and min of each particle's value
  for(intParticle=1;intParticle<=intNParticles;intParticle++) {
    for(intParam=0;intParam<NPARAMS;intParam++) {
      dblParamValue = pParticles[intParticle].dblParams[intParam];
      dblParamMean[intParam] += dblParamValue/(double)intNParticles;
      dblParamMax[intParam] = DMAX(dblParamMax[intParam], dblParamValue);
      dblParamMin[intParam] = DMIN(dblParamMin[intParam], dblParamValue);
    }
  }
  for(intParticle=1;intParticle<=intNParticles;intParticle++) {
    for(intParam=0;intParam<NPARAMS;intParam++) {
    dblDiff = pParticles[intParticle].dblParams[intParam] - dblParamMean[intParam];
    dblAdj[intParam] += dblDiff;
    dblSSE[intParam] += dblDiff*dblDiff;
    }
  }
  for(intParam=0;intParam<NPARAMS;intParam++) {
    dblParamSD[intParam] = sqrt((dblSSE[intParam]-dblAdj[intParam]/(double)intNParticles)/
      (double)(intNParticles-1));
  }

  //Print results
  for(intParam=0;intParam<NPARAMS;intParam++) {
    fprintf(pControlPF->pDebugFile, "  %s   min: %e   mean: %e   max: %e  sd: %e\n",
      cstrParamNames[intParam], dblParamMin[intParam],
      dblParamMean[intParam], dblParamMax[intParam], dblParamSD[intParam]);
  }
  fflush(pControlPF->pDebugFile);
}

/*------------------------------------------------------------------------*/

void writeStatesSummary(controlPFstruct * const pControlPF,
                      particlestruct * const pParticles,
                      Int intYear, Int blnShowNewState)
/*
  Purpose: Writes a summary of the state values in pParticles
    to the debug file.  If blnShowNewState is True shows pParticles[].intNewState, if
    False, uses intOldState.  For pups, also writes out the observed value for year intYear.
  Implementation note:
    Had to declare blnShowNewState as integer as can't use an enum in declaration
*/
{
  Int intParticle, intColony, intAge, intNParticles;
  double **dblStateMean;
  Int **intStateMax;
  Int **intStateMin;
  Int intStateValue; //shortcut

  //don't do anything if there's no debug file
  if (pControlPF->pDebugFile==NULL) return;

  //initialize
  intNParticles = pControlPF->intNParticles;
  if(intNParticles==0) {
    fprintf(pControlPF->pDebugFile, "  No elements to summarize");
    return;
  }
  dblStateMean = dmatrix(1, NCOLONIES, 1, NAGES);
  intStateMax = imatrix(1, NCOLONIES, 1, NAGES);
  intStateMin = imatrix(1, NCOLONIES, 1, NAGES);
  for (intColony=1;intColony<=NCOLONIES;intColony++) {
    for (intAge=1;intAge<=NAGES;intAge++) {
      dblStateMean[intColony][intAge]=0.0;
      intStateMax[intColony][intAge]=INT_MIN;
      intStateMin[intColony][intAge]=INT_MAX;
    }
  }

  //Get the mean, max and min of each particle's value
  for(intParticle=1;intParticle<=intNParticles;intParticle++) {
    for (intColony=1;intColony<=NCOLONIES;intColony++) {
      for (intAge=1;intAge<=NAGES;intAge++) {
        if(blnShowNewState) {
          intStateValue = pParticles[intParticle].intNewStates[intColony][intAge];
        } else {
          intStateValue = pParticles[intParticle].intOldStates[intColony][intAge];
        }
        dblStateMean[intColony][intAge] += (double) intStateValue / (double) intNParticles;
        intStateMax[intColony][intAge] = IMAX(intStateMax[intColony][intAge], intStateValue);
        intStateMin[intColony][intAge] = IMIN(intStateMin[intColony][intAge], intStateValue);
      }
    }
  }

  //Print results
  for (intColony=1;intColony<=NCOLONIES;intColony++) {
    fprintf(pControlPF->pDebugFile, "  Colony %i\n",intColony);
    for (intAge=1;intAge<=NAGES;intAge++) {
      if(intAge==1) {
        //For pups (fitting only), also put out the true value
        if(pControlPF->blnFitModel) {
          fprintf(pControlPF->pDebugFile, "   Observed pups: %i",
            pControlPF->intPupCounts[intYear][intColony]);
          if(pControlPF->blnKnownSEs)
            fprintf(pControlPF->pDebugFile, " SE: %i",
              pControlPF->intPupCountSE[intYear][intColony]);
          fprintf(pControlPF->pDebugFile, "\n");
        }
        fprintf(pControlPF->pDebugFile, "   Pup       min: %i   mean: %i   max: %i\n",
          intStateMin[intColony][intAge],
          iround(dblStateMean[intColony][intAge]),intStateMax[intColony][intAge]);
      } else {
        fprintf(pControlPF->pDebugFile, "   Adult %i   min: %i   mean: %i   max: %i\n",
          (intAge-1),intStateMin[intColony][intAge],
          iround(dblStateMean[intColony][intAge]),intStateMax[intColony][intAge]);
      }
    }
  }
  fflush(pControlPF->pDebugFile);

  free_dmatrix(dblStateMean, 1, NCOLONIES, 1, NAGES);
  free_imatrix(intStateMax, 1, NCOLONIES, 1, NAGES);
  free_imatrix(intStateMin, 1, NCOLONIES, 1, NAGES);
}

/*------------------------------------------------------------------------*/

void writeStateSummaryForOneAge(controlPFstruct * const pControlPF,
                                particlestruct * const pParticles,
                                Int intYear, Int intAge, Int blnShowNewState)
/*
  Purpose: Similar to writeStateSummary, but just outputs summary for one age
*/
{
  Int intParticle, intColony, intNParticles;
  double *dblStateMean;
  Int *intStateMax;
  Int *intStateMin;
  Int intStateValue; //shortcut

  //don't do anything if there's no debug file
  if (pControlPF->pDebugFile==NULL) return;

  //initialize
  intNParticles = pControlPF->intNParticles;
  if(intNParticles==0) {
    fprintf(pControlPF->pDebugFile, "  No elements to summarize");
    return;
  }
  dblStateMean = dvector(1, NCOLONIES);
  intStateMax = ivector(1, NCOLONIES);
  intStateMin = ivector(1, NCOLONIES);
  for (intColony=1;intColony<=NCOLONIES;intColony++) {
      dblStateMean[intColony]=0.0;
      intStateMax[intColony]=INT_MIN;
      intStateMin[intColony]=INT_MAX;
  }

  //Get the mean, max and min of each particle's value
  for(intParticle=1;intParticle<=intNParticles;intParticle++) {
    for (intColony=1;intColony<=NCOLONIES;intColony++) {
      if(blnShowNewState) {
        intStateValue = pParticles[intParticle].intNewStates[intColony][intAge];
      } else {
        intStateValue = pParticles[intParticle].intOldStates[intColony][intAge];
      }
      dblStateMean[intColony] += (double) intStateValue / (double) intNParticles;
      intStateMax[intColony] = IMAX(intStateMax[intColony], intStateValue);
      intStateMin[intColony] = IMIN(intStateMin[intColony], intStateValue);
    }
  }

  //Print results
  for (intColony=1;intColony<=NCOLONIES;intColony++) {
    fprintf(pControlPF->pDebugFile, "  Colony %i",intColony);
    //For pups, also put out the true value
    if(intAge==1) {
      fprintf(pControlPF->pDebugFile, "   Observed pups: %i\n",
        pControlPF->intPupCounts[intYear][intColony]);
      fprintf(pControlPF->pDebugFile, " Pup       min: %i   mean: %i   max: %i\n",
        intStateMin[intColony],
        iround(dblStateMean[intColony]),intStateMax[intColony]);
    } else {
      fprintf(pControlPF->pDebugFile, " Adult %i   min: %i   mean: %i   max: %i\n",
        (intAge-1),intStateMin[intColony],
        iround(dblStateMean[intColony]),intStateMax[intColony]);
    }
  }
  fflush(pControlPF->pDebugFile);


  free_dvector(dblStateMean, 1, NCOLONIES);
  free_ivector(intStateMax, 1, NCOLONIES);
  free_ivector(intStateMin, 1, NCOLONIES);
}

/*------------------------------------------------------------------------*/

void writeMeta(controlPFstruct * const pControlPF,
               metastruct * const pMeta,
               int blnWriteToDebugFile,
               int blnWriteToOutputFile)
/* Purpose: Writes a summary of the current values of the meta struct to the
   debug file and/or a file *Meta.txt where * is a prefix such as Res
*/
{

  Int intYear, i;
  FILE *pFile;

  for (i=1;i<=2;i++) {
    pFile = NULL;
    //first time write to debug file
    if((i==1)&&(blnWriteToDebugFile)&&(pControlPF->blnDebugFile))
      pFile = pControlPF->pDebugFile;
    //second time write to output file
    if((i==2)&&(blnWriteToOutputFile))
      pFile = openParticleFile("Meta", pControlPF->strResultsPrefix, TXTFILE, 0, "wt");

    if(!(pFile==NULL)) {
      if(i==1) fprintf(pFile, "Meta structure:\n"); //Just put this line our if it's to the output file
      fprintf(pFile, "      Year  Itrtns   Tot ptcls   Ptcls svd       ESS  Wt scaling1   Wt scaling2  Wt scaling3\n");
      for(intYear=1;intYear<=pControlPF->intNYears;intYear++) {
        fprintf(pFile, "  %2i%6i %7i %11i %11i %9.2f %9.6e %9.6e %9.6e\n", intYear,
          pMeta[intYear].intYear, pMeta[intYear].intIterations, pMeta[intYear].intTotalParticles,
          pMeta[intYear].intNParticlesSaved, pMeta[intYear].dblESS, pMeta[intYear].dblWeightScaling1,
          pMeta[intYear].dblWeightScaling2, pMeta[intYear].dblWeightScaling3);
      }
      //flush the debug fiile
      if(i==1) fflush(pFile);
      //close the output file
      if(i==2) fclose(pFile);
    }
  }

}

/*------------------------------------------------------------------------*/

void readMeta(controlPFstruct * const pControlPF,
               metastruct * const pMeta)
/* Purpose: Reads values of the meta struct from the file *Meta.txt where * is a prefix such as Res
   If the file doesn't exist, returns without reading any values
*/
{

  Int intYear, i=0;
  FILE *pFile;

  pFile = openParticleFile("Meta", pControlPF->strResultsPrefix, TXTFILE, 0, "rt");
  if(pFile==NULL) return;

  //Skip header line
  SkipLines(pFile, 1);
  //Read in the other lines
  for(intYear=1;intYear<=pControlPF->intNYears;intYear++) {
    fscanf(pFile, "  %i %i %i %i %i %lg %lg %lg %lg\n", &i,
      &pMeta[intYear].intYear, &pMeta[intYear].intIterations, &pMeta[intYear].intTotalParticles,
      &pMeta[intYear].intNParticlesSaved, &pMeta[intYear].dblESS, &pMeta[intYear].dblWeightScaling1,
      &pMeta[intYear].dblWeightScaling2, &pMeta[intYear].dblWeightScaling3);
  }
  fclose(pFile);

}

/*------------------------------------------------------------------------*/

void writeDVectorSummary(controlPFstruct * const pControlPF,
                         double * const dblVector, Int intStart, Int intEnd)
/* Purpose: Writes a summary of the contents of the dblVector
*/
{
  Int i;
  double dblN; //Number of elements - as a double, saves converting from int
  double dblMean=0.0;
  double dblMax=-DBL_MAX;
  double dblMin=DBL_MAX;

  //don't do anything if there's no debug file
  if (pControlPF->pDebugFile==NULL) return;

  dblN=(double) (intEnd-intStart+1);
  if(dblN==0.0) {
    fprintf(pControlPF->pDebugFile, "  No elements to summarize");
    return;
  }

  //Get the mean, max and min
  for(i=intStart;i<=intEnd;i++) {
    dblMean += dblVector[i] / dblN;
    //Can't use the DBL_MAX and DBL_MIN macros here as they cause a crash
    // on linux for some reason
    dblMax = (dblMax>dblVector[i]) ? dblMax : dblVector[i];
    dblMin = (dblMin<dblVector[i]) ? dblMin : dblVector[i];
  }

  //Print results
  fprintf(pControlPF->pDebugFile, "  min: %e   mean: %e   max: %e\n",
    dblMin,dblMean,dblMax);
  fflush(pControlPF->pDebugFile);

}
/*------------------------------------------------------------------------*/

double getESS(controlPFstruct * const pControlPF,
              particlestruct * const pParticles,
              Int *intIndex, Int intNIndex,
              Int blnWriteOut)
/*
Purpose:
  Returns the effective sample size of the weights in pParticles
  If pControlPF->blnDebugFile and blnWriteOut, then writes them to the debug file
  If intNIndex>0 then uses the values in intIndex to find the particles to
    use in the calculation; if intNIndex<0, it uses all the particles; 
    if intNIndex==0 it returns 0 for the outputs

*/
{
  Int i, intN=0;
  double dblMean=0, dblDiff, dblAdj=0, dblSSE=0, dblVariance=0, dblCVsq=0, dblESS=0;
  double dblMax=-DBL_MAX;
  double dblMin=DBL_MAX;
  double dblWeight;

  //Only calculate ESS if there are not 0 particles!
  if(intNIndex!=0){
    //Calculate the ESS using all particles (i.e., ignore NIndex)
    if (intNIndex<0) {
      intN = pControlPF->intNParticles;
      for(i=1;i<=intN;i++) {
        dblWeight = pParticles[i].dblWeight;
        dblMean += dblWeight/(double)intN;
        //Can't use the DBL_MAX and DBL_MIN macros here as they cause a crash
        // on linux for some reason
        dblMax = (dblMax>dblWeight) ? dblMax : dblWeight;
        dblMin = (dblMin<dblWeight) ? dblMin : dblWeight;
      }
      for(i=1;i<=intN;i++) {
        dblDiff = pParticles[i].dblWeight-dblMean;
        dblAdj += dblDiff;
        dblSSE += dblDiff*dblDiff;
      }
    } else {
    //Calculate the ESS using just the NIndex particles
      intN=intNIndex;
      for(i=1;i<=intN;i++) {
        dblWeight = pParticles[intIndex[i]].dblWeight;
        dblMean += dblWeight/(double)intN;
        //Can't use the DBL_MAX and DBL_MIN macros here as they cause a crash
        // on linux for some reason
        dblMax = (dblMax>dblWeight) ? dblMax : dblWeight;
        dblMin = (dblMin<dblWeight) ? dblMin : dblWeight;
      }
      for(i=1;i<=intN;i++) {
        dblDiff = pParticles[intIndex[i]].dblWeight-dblMean;
        dblAdj += dblDiff;
        dblSSE += dblDiff*dblDiff;
      }
    }
    dblVariance = (dblSSE-dblAdj/(double)intN)/(double)(intN-1);
    dblCVsq = dblVariance/(dblMean*dblMean);
    dblESS = (double)intN/(1.0+dblCVsq);
  }

  //Print it out
  if (pControlPF->blnDebugFile && blnWriteOut) {
    fprintf(pControlPF->pDebugFile, "N particles: %i  min: %e  mean: %e  max: %e  CV: %f ESS: %i\n",
      intN, dblMin, dblMean, dblMax, sqrt(dblCVsq), iround(dblESS));
      fflush(pControlPF->pDebugFile);
  }

  return(dblESS);
}

/*------------------------------------------------------------------------*/

FILE *openParticleFile(char *strRootName, char *strPrefix, char *strSuffix,
                       Int intYear, char *chrMode)
/* Purpose: opens the particle file of the form *Yr*** - e.g., ResYr1States.tmp
   If intYear is 0, then it has the form **** - e.g., ResMeta.txt
  */
{
  FILE *pFile;
  char strFileName[MAXSTR];

  if (intYear>0) {
    sprintf(strFileName, "%sYr%i%s%s", strPrefix, intYear, strRootName, strSuffix);
  } else {
    sprintf(strFileName, "%s%s%s", strPrefix, strRootName, strSuffix);
  }

  if ((pFile = fopen(strFileName, chrMode))==NULL) {
    printf("File %s could not be opened for writing",strFileName);
    exit(1);
  }
  return(pFile);
}

/*------------------------------------------------------------------------*/

Int saveParticles(controlPFstruct * const pControlPF,
                  particlestruct * const pParticles,
                  Int *intIndex,
                  Int intNIndex,
                  Int intYear,
                  Int blnSaveAsBinary,
                  Int blnSaveNewState,
                  Int blnSaveParamsWeightsIDs,
                  Int blnSaveParentB,
                  Int blnSaveNParticles,
                  Int blnAppend,
                  Int blnCompact,
                  Int intMaxNToSave,
                  char *strSuffix)
/*
Purpose:
  Saves the particles to binary or text files of the form *Yr*States.*, *Yr*Params.*,
    *Yr*Weights.*, *Yr*ID.*, *Yr*Parent.*, *Yr*NParticles.* where the *s are bits that are
    filled in (e.g., ResYr1States.tmp)
  Note that the NParticles file just contains one entry - the number of particles saved
Implementation note:
  This is a lot of similar code repeated 4 times, for single precision binary,
    double precision binary, single precision text, double precision text.  The alternative
    would be to put them all together, but then there would be lots of "if...else" statements
    within each particle, which will be much slower... (although an optimizing compiler should
    in theory take care of that...)
Inputs:
  pControlPF - control struct
  pParticles - vector or particles to save
  intIndex - if intNIndex>0 then this should be a vector containing the numbers of the
    particles to save
  intNIndex - number of particles indexed in intIndex.  If positive, takes particles indexed by intIndex,
              if zero, saves nothing, if negative ignores intIndex and saves everything
  intYear - year to save file into
  blnSaveAsBinary - if true saves as a binary file, else a text file
  blnSaveNewState - if true saves pParticles.intNewStates, if false, saves pParticles.intOldStates
  blnSaveParamsWeightsIDs - if true saves the Params, Weights and IDs to file as well as states
  blnSaveParentB - if true saves parent ID and B as well as states
                    note - B only saved if integrating out Psi
  blnSaveNParticles - if true saves the number of particles saved
  blnAppend - if true, appends results to any existing file
  blnCompact - if true, saves the parameter values as single precision, and for the states
                saves them as integers and only saves states 1 (pups) (2-NAGES-1) (ie.adults)
                and NAGES (ie breeders), rather than saving each age separately
               if false, saves all values in their native precision and as normal values
  intMaxNToSave - maximum number of particles to save
  strSuffix - the suffix to add to the files
Outputs:
  Returns the number of particles saved
*/
{
  //Whether to open in append or write mode -- depends on blnAppend
  char chrMode[MAXSTR];
  //Pointers to the files
  FILE *pStatesFile, *pParamsFile, *pWeightsFile, *pIDFile, *pParentFile, *pBFile, *pNParticlesFile;
  Int intNParticles=0,  intNStates = NCOLONIES * NAGES;
  //CountingVars
  Int i, j, intN, intColony, intAge, intParam;
  //Shortcut to current particle in loop
  particlestruct *pParticle;
  //Shortcut to whether integrating out psi
  boolean blnIntegrateOutPsi;
  //temporary storage locations for values to be written out in single precision formats
  int intValue;
  float sngValue;

  blnIntegrateOutPsi = pControlPF->blnIntegrateOutPsi;

  //Check that you're not saving any particles!
  if ((intMaxNToSave == 0)|(intNIndex == 0)) return (0);

  //Set open mode
  if (blnSaveAsBinary) {
    strcpy(chrMode, (blnAppend) ? "a+b":"wb");
  } else {
    strcpy(chrMode, (blnAppend) ? "a+t":"wt");
  }

  //Open the files
  pStatesFile = openParticleFile("States", pControlPF->strResultsPrefix,
    strSuffix, intYear, chrMode);
  if (blnSaveParamsWeightsIDs) {
    pParamsFile = openParticleFile("Params", pControlPF->strResultsPrefix,
      strSuffix, intYear, chrMode);
    pWeightsFile = openParticleFile("Weights", pControlPF->strResultsPrefix,
      strSuffix, intYear, chrMode);
    pIDFile = openParticleFile("ID", pControlPF->strResultsPrefix,
      strSuffix, intYear, chrMode);
  }
  if (blnSaveParentB) {
    pParentFile = openParticleFile("Parent", pControlPF->strResultsPrefix,
      strSuffix, intYear, chrMode);
    if (blnIntegrateOutPsi)
      pBFile = openParticleFile("B", pControlPF->strResultsPrefix,
        strSuffix, intYear, chrMode);
  }
  if(blnSaveNParticles) {
    pNParticlesFile = openParticleFile("NParticles", pControlPF->strResultsPrefix,
      strSuffix, intYear, chrMode);
  }

  //If intNINdex is negative then save all particles
  intN = (intNIndex>=0) ? intNIndex : pControlPF->intNParticles;

  if (blnSaveAsBinary) {

    if (blnCompact) {
      //Save in binary form - and compact
      for (i = 1; i <= intN; i++) {

        pParticle = (intNIndex>0) ? &pParticles[intIndex[i]] : &pParticles[i];
        intNParticles ++;

        //Save the states
        if(blnSaveNewState) {
          for(intColony=1;intColony<=NCOLONIES;intColony++) {
            //pups
            intValue= (int) pParticle->intNewStates[intColony][1];
            fwrite(&intValue, sizeof(int), 1, pStatesFile);
            //adult non-breeders - add 'em up and write 'em out as one number
            intValue = (int) 0;
            for(intAge=2;intAge<NAGES;intAge++) {
              intValue += (int) pParticle->intNewStates[intColony][intAge];
            }
            fwrite(&intValue, sizeof(int), 1, pStatesFile);
            //breeders
            intValue = (int) pParticle->intNewStates[intColony][NAGES];
            fwrite(&intValue, sizeof(int), 1, pStatesFile);
          }
        } else {
          for(intColony=1;intColony<=NCOLONIES;intColony++) {
            //pups
            intValue= (int) pParticle->intOldStates[intColony][1];
            fwrite(&intValue, sizeof(int), 1, pStatesFile);
            //adult non-breeders - add 'em up and write 'em out as one number
            intValue = (int) 0;
            for(intAge=2;intAge<NAGES;intAge++) {
              intValue += (int) pParticle->intOldStates[intColony][intAge];
            }
            fwrite(&intValue, sizeof(int), 1, pStatesFile);
            //breeders
            intValue = (int) pParticle->intOldStates[intColony][NAGES];
            fwrite(&intValue, sizeof(int), 1, pStatesFile);
          }
        }

        if (blnSaveParamsWeightsIDs) {
          //Save the params - single precision
          for(j=0;j<NPARAMS;j++) {
            sngValue = (float) pParticle->dblParams[j];
            fwrite(&sngValue, sizeof(float), 1, pParamsFile);
          }
          //Save the weight - as single
          sngValue = (float) pParticle->dblWeight;
          fwrite(&sngValue, sizeof(float), 1, pWeightsFile);
          //Save the ID - as an int
          intValue = (int) pParticle->intID;
          fwrite(&intValue, sizeof(int), 1, pIDFile);
        }
        if (blnSaveParentB) {
          //Save the ParentID - as int
          intValue = (int) pParticle->intParent;
          fwrite(&intValue, sizeof(int), 1, pParentFile);
          //Save BCurrent or BPrev - as single
          if (blnIntegrateOutPsi) {
            if(blnSaveNewState) {
              sngValue = (float) pParticle->dblBCurrent;
              fwrite(&sngValue, sizeof(float), 1, pBFile);
            } else {
              sngValue = (float) pParticle->dblBPrev;
              fwrite(&sngValue, sizeof(float), 1, pBFile);
            }
          }
        }
        //Check if you've saved enough particles now
        if(intNParticles==intMaxNToSave) break;
      }
      if (blnSaveNParticles) {
        intValue = (int) intNParticles;
        fwrite(&intValue, sizeof(int), 1, pNParticlesFile);
      }
      //End of compact binary

    } else {

      //Save in binary form - double precision
      for (i = 1; i <= intN; i++) {

        pParticle = (intNIndex>0) ? &pParticles[intIndex[i]] : &pParticles[i];
        intNParticles ++;

        //Save the states
        if(blnSaveNewState) {
          fwrite(&pParticle->intNewStates[1][1], sizeof(Int), intNStates, pStatesFile);
        } else {
          fwrite(&pParticle->intOldStates[1][1], sizeof(Int), intNStates, pStatesFile);
        }

        if (blnSaveParamsWeightsIDs) {
          //Save the params
          fwrite(&pParticle->dblParams[0], sizeof(double), NPARAMS, pParamsFile);
          //Save the weight
          fwrite(&pParticle->dblWeight, sizeof(double), 1, pWeightsFile);
          //Save the ID
          fwrite(&pParticle->intID, sizeof(Int), 1, pIDFile);
        }
        if (blnSaveParentB) {
          //Save the ParentID
          fwrite(&pParticle->intParent, sizeof(Int), 1, pParentFile);
          //Save BCurrent or BPrev
          if (blnIntegrateOutPsi) {
            if(blnSaveNewState) {
              fwrite(&pParticle->dblBCurrent, sizeof(double), 1, pBFile);
            } else {
              fwrite(&pParticle->dblBPrev, sizeof(double), 1, pBFile);
            }
          }
        }
        //Check if you've saved enough particles now
        if(intNParticles==intMaxNToSave) break;
      }
      if (blnSaveNParticles)
        fwrite(&intNParticles, sizeof(Int), 1, pNParticlesFile);
      //End of double precision binary

    }

  } else {

    if (blnCompact) {

      //Save in text form - and compact
      for (i = 1; i <= intN; i++) {

        pParticle = (intNIndex>0) ? &pParticles[intIndex[i]] : &pParticles[i];
        intNParticles ++;

        //Save the states
        if(blnSaveNewState) {
          for(intColony=1;intColony<=NCOLONIES;intColony++) {
            //pups
            fprintf(pStatesFile, "%i ", (int) pParticle->intNewStates[intColony][1]);
            //adult non-breeders - add 'em up and write 'em out as one number
            intValue = (int) 0;
            for(intAge=2;intAge<NAGES;intAge++) {
              intValue += (int) pParticle->intNewStates[intColony][intAge];
            }
            fprintf(pStatesFile, "%i ", intValue);
            //breeders
            fprintf(pStatesFile, "%i ", (int) pParticle->intNewStates[intColony][NAGES]);
          }
        } else {
          for(intColony=1;intColony<=NCOLONIES;intColony++) {
            //pups
            fprintf(pStatesFile, "%i ", (int) pParticle->intOldStates[intColony][1]);
            //adult non-breeders - add 'em up and write 'em out as one number
            intValue = (int) 0;
            for(intAge=2;intAge<NAGES;intAge++) {
              intValue += (int) pParticle->intOldStates[intColony][intAge];
            }
            fprintf(pStatesFile, "%i ", intValue);
            //breeders
            fprintf(pStatesFile, "%i ", (int) pParticle->intOldStates[intColony][NAGES]);
          }
        }
        fprintf(pStatesFile, "\n");

        if (blnSaveParamsWeightsIDs) {
          //Save the params - single precision
          for(intParam=0;intParam<NPARAMS;intParam++)
            fprintf(pParamsFile, "%e ", (float) pParticle->dblParams[intParam]);
          fprintf(pParamsFile, "\n");
          //Save the weight - as single
          fprintf(pWeightsFile, "%e\n", (float) pParticle->dblWeight);
          //Save the ID - as int
          fprintf(pIDFile, "%i\n", (int) pParticle->intID);
        }
        if (blnSaveParentB) {
          //Save the ParentID - as int
          fprintf(pParentFile, "%i\n", (int) pParticle->intParent);
          //Save BCurrent or BPrev - as single
          if (blnIntegrateOutPsi) {
            if(blnSaveNewState) {
              fprintf(pBFile, "%e\n", (float) pParticle->dblBCurrent);
            } else {
              fprintf(pBFile, "%e\n", (float) pParticle->dblBPrev);
            }
          }
        }
        //Check if you've saved enough particles now
        if(intNParticles==intMaxNToSave) break;
      }
      if (blnSaveNParticles)
        fprintf(pNParticlesFile, "%i\n", (int) intNParticles);
      //End of compact text format

    } else {

      //Save in text form - double precision
      for (i = 1; i <= intN; i++) {

        pParticle = (intNIndex>0) ? &pParticles[intIndex[i]] : &pParticles[i];
        intNParticles ++;

        //Save the states
        if(blnSaveNewState) {
          for(intColony=1;intColony<=NCOLONIES;intColony++) {
            for(intAge=1;intAge<=NAGES;intAge++) {
              fprintf(pStatesFile, "%i ", pParticle->intNewStates[intColony][intAge]);
            }
          }
        } else {
          for(intColony=1;intColony<=NCOLONIES;intColony++) {
            for(intAge=1;intAge<=NAGES;intAge++) {
              fprintf(pStatesFile, "%i ", pParticle->intOldStates[intColony][intAge]);
            }
          }
        }
        fprintf(pStatesFile, "\n");

        if (blnSaveParamsWeightsIDs) {
          //Save the params
          for(intParam=0;intParam<NPARAMS;intParam++)
            fprintf(pParamsFile, "%e ", pParticle->dblParams[intParam]);
          fprintf(pParamsFile, "\n");
          //Save the weight
          fprintf(pWeightsFile, "%e\n", pParticle->dblWeight);
          //Save the ID
          fprintf(pIDFile, "%i\n", pParticle->intID);
        }
        if (blnSaveParentB) {
          //Save the ParentID
          fprintf(pParentFile, "%i\n", pParticle->intParent);
          //Save BCurrent or BPrev
          if (blnIntegrateOutPsi) {
            if(blnSaveNewState) {
              fprintf(pBFile, "%e\n", pParticle->dblBCurrent);
            } else {
              fprintf(pBFile, "%e\n", pParticle->dblBPrev);
            }
          }
        }
        //Check if you've saved enough particles now
        if(intNParticles==intMaxNToSave) break;
      }
      if (blnSaveNParticles)
        fprintf(pNParticlesFile, "%i\n", intNParticles);
      //End of double precision text format

    }

  }

  fclose(pStatesFile);
  if (blnSaveParamsWeightsIDs) {
    fclose(pParamsFile);
    fclose(pWeightsFile);
    fclose(pIDFile);
  }
  if (blnSaveParentB) {
    fclose(pParentFile);
    if (blnIntegrateOutPsi) fclose(pBFile);
  }
  if (blnSaveNParticles) {
    fclose(pNParticlesFile);
  }

  return(intNParticles);
}

/*------------------------------------------------------------------------*/

Int loadParticlesBinary(particlestruct * const pParticles,
                  Int *intIndex,
                  Int intNIndex,
                  Int intYear,
                  Int intMaxNToLoad,
                  boolean blnLoadIDAndParent,
                  boolean blnLoadB,
                  char *strPrefix,
                  char *strSuffix)
/*
Purpose:
  Loads the particles from binary files of the form *Yr*StatesRep*, *Yr*ParamsRep*,
    *Yr*WeightsRep*, *Yr*IDRep*, *Yr*ParentRep*, where the *s are bits that are
    filled in (e.g., ResYr1StatesRep1).
  If intNIndex<=0 then all the records there, up to intMaxNToLoad;
    if intNIndex>0 then does direct access to get all the particles pointed to by intIndex
  States are loaded into intOldStates (as opposed to intNewStates)
Inputs:
  pParticles - vector of particles - needs to have space allocated to it already at present
  intIndex - if intNIndex>0 then this should be a vector containing the numbers of the
    particles to load
  intNIndex - number of particles indexed in intIndex
  intYear - year to load from
  intMaxNToLoad - maximum number of particles to load - at present equal to size of pParticles
  blnLoadIDAndParent - if Y loads the ID and parent files too
  blnLoadB - if Y loads the B file used when integrating out Psi
  strPrefix - prefix of files to load
  strSuffix - suffix of files to load
Outputs:
  Returns the number of particles loaded
*/
{
  //Pointers to the files
  FILE *pStatesFile, *pParamsFile, *pWeightsFile, *pIDFile, *pParentFile, *pBFile;
  Int  intNStates = NCOLONIES * NAGES;
  //CountingVar
  Int intParticle=0;
  //Shortcut to current particle in loop
  particlestruct *pParticle;
  char chrMode[]="rb";
//  //Shortcut to whether integrating out psi
//  boolean blnIntegrateOutPsi;

//  blnIntegrateOutPsi = pControlPF->blnIntegrateOutPsi;

  //Open the files
  pStatesFile = openParticleFile("States", strPrefix,
    strSuffix, intYear, chrMode);
  pParamsFile = openParticleFile("Params", strPrefix,
    strSuffix, intYear, chrMode);
  pWeightsFile = openParticleFile("Weights", strPrefix,
    strSuffix, intYear, chrMode);
  if(blnLoadIDAndParent) {
    pIDFile = openParticleFile("ID", strPrefix,
      strSuffix, intYear, chrMode);
    pParentFile = openParticleFile("Parent", strPrefix,
      strSuffix, intYear, chrMode);
  }
  if(blnLoadB)
    pBFile = openParticleFile("B", strPrefix,
    strSuffix, intYear, chrMode);

  if(intNIndex<=0) {
    //Load all the records available
    while (!feof(pStatesFile)) {

      intParticle ++;
      //Set shortcut
      pParticle = &pParticles[intParticle];

      //Load the states
      fread(&pParticle->intOldStates[1][1], sizeof(Int), intNStates, pStatesFile);
      //Load the params
      fread(&pParticle->dblParams[0], sizeof(double), NPARAMS, pParamsFile);
      //Load the weight
      fread(&pParticle->dblWeight, sizeof(double), 1, pWeightsFile);
      if(blnLoadIDAndParent) {
        //Load the ID
        fread(&pParticle->intID, sizeof(Int), 1, pIDFile);
        //Load the ParentID
        fread(&pParticle->intParent, sizeof(Int), 1, pParentFile);
      }
      //Load the B
      if(blnLoadB)
        fread(&pParticle->dblBPrev, sizeof(double), 1, pBFile);

      //Check if you've loaded enough particles
      if(intParticle==intMaxNToLoad) break;
    }
  } else {
    //Do direct file access
    for(intParticle=1;intParticle<=intNIndex; intParticle++) {

      //Set shortcut
      pParticle = &pParticles[intParticle];

      //Load the states
      fseek(pStatesFile, sizeof(Int)*(intIndex[intParticle]-1)*intNStates, SEEK_SET);
      fread(&pParticle->intOldStates[1][1], sizeof(Int), intNStates, pStatesFile);
      //Load the params
      fseek(pParamsFile, sizeof(double)*(intIndex[intParticle]-1)*NPARAMS, SEEK_SET);
      fread(&pParticle->dblParams[0], sizeof(double), NPARAMS, pParamsFile);
      //Load the weight
      fseek(pWeightsFile, sizeof(double)*(intIndex[intParticle]-1), SEEK_SET);
      fread(&pParticle->dblWeight, sizeof(double), 1, pWeightsFile);
      if(blnLoadIDAndParent) {
        //Load the ID
        fseek(pIDFile, sizeof(Int)*(intIndex[intParticle]-1), SEEK_SET);
        fread(&pParticle->intID, sizeof(Int), 1, pIDFile);
        //Load the ParentID
        fseek(pParentFile, sizeof(Int)*(intIndex[intParticle]-1), SEEK_SET);
        fread(&pParticle->intParent, sizeof(Int), 1, pParentFile);
      }
      //Load the B
      if(blnLoadB) {
        fseek(pBFile, sizeof(double)*(intIndex[intParticle]-1), SEEK_SET);
        fread(&pParticle->dblBPrev, sizeof(double), 1, pBFile);
      }

      //Check if you've loaded enough particles
      if(intParticle==intMaxNToLoad) break;

    }
    intParticle=IMIN(intNIndex,intMaxNToLoad);
  }

  fclose(pStatesFile);
  fclose(pParamsFile);
  fclose(pWeightsFile);
  if (blnLoadIDAndParent) {
    fclose(pIDFile);
    fclose(pParentFile);
  }
  if (blnLoadB) fclose(pBFile);

  return(intParticle);
}

/*------------------------------------------------------------------------*/

Int getNParticlesBinary(Int intYear,
                  char *strPrefix,
                  char *strSuffix)
/*
Purpoose:
  Returns the number of particles in the weights file associated with
  strSuffix and intYear (0 for no year).
  Assumes file is **not** in compact format (ie weights are stored as double).
Implementation note:
  Seems like the only ANSI standard way to do this is to read the file -- which is
  a bit wasteful, but there we are...
*/
{

  //number of weights to try to read each go
  #define READ_AT_ONCE 10000

  char strMode[]="rb";
  FILE *pWeightsFile;
  Int intN; //total number read
  Int intNRead;  //number read each time
  double *dblWeights;


  //Open the Weights file
  pWeightsFile = openParticleFile("Weights", strPrefix, strSuffix, intYear, strMode);
  if((pWeightsFile==NULL)) {
    printf("Weights file could not be opened for reading");
    exit(1);
  } else {

    intN = 0;
    dblWeights = dvector(1, READ_AT_ONCE);
    while (!feof(pWeightsFile)) {
      //Read next 10000 weights
      intNRead = fread(dblWeights, sizeof(double), READ_AT_ONCE, pWeightsFile);
      intN += intNRead;
    }
    fclose(pWeightsFile);
    free_dvector(dblWeights, 1, READ_AT_ONCE);
  }
  return(intN);
}

/*------------------------------------------------------------------------*/

Int loadWeightsBinary(double *dblWeights,
                  Int *intIndex,
                  Int intNIndex,
                  Int intYear,
                  Int intMaxNToLoad,
                  char *strPrefix,
                  char *strSuffix)
/*
Purpose:
  Loads the weights from binary files of the form *Yr*WeightsRep*,
  where the *s are bits that are
    filled in (e.g., ResYr1WeightsRep1).
  If Yr is 0 then that bit is missed out; ditto for zero suffix.
  If intNIndex<=0 then all the records there, up to intMaxNToLoad;
    if intNIndex>0 then does direct access to get all the particles pointed to by intIndex
Inputs:
  dblWeights - vector of weights - needs to have space allocated to it already at present
    assumes index starts at 1 (not 0)
  intIndex - if intNIndex>0 then this should be a vector containing the numbers of the
    particles to load
  intNIndex - number of particles indexed in intIndex
  intYear - year to load from
  intMaxNToLoad - maximum number of particles to load - at present equal to size of pParticles
  strPrefix - prefix of files to load
  strSuffix - suffix of files to load
Outputs:
  Returns the number of particles loaded
*/
{
  //Pointers file
  FILE *pWeightsFile;
  //CountingVar
  Int intParticle=0;
  char chrMode[]="rb";

  //Open the files
  pWeightsFile = openParticleFile("Weights", strPrefix,
    strSuffix, intYear, chrMode);

  if(intNIndex<=0) {
    //Load all the records available
    while (!feof(pWeightsFile)) {

      //Load the weights
      intParticle = fread(&dblWeights[1], sizeof(double), intMaxNToLoad, pWeightsFile);

    }
  } else {
    //Do direct file access
    for(intParticle=1;intParticle<=intNIndex; intParticle++) {

      fseek(pWeightsFile, sizeof(double)*(intIndex[intParticle]-1), SEEK_SET);
      fread(&dblWeights[intParticle], sizeof(double), 1, pWeightsFile);

      //Check if you've loaded enough particles
      if(intParticle==intMaxNToLoad) break;

    }
    intParticle=IMIN(intNIndex,intMaxNToLoad);
  }

  fclose(pWeightsFile);
  return(intParticle);
}

/*------------------------------------------------------------------------*/

void removeParticleFiles(controlPFstruct * const pControlPF, char *strSuffix)
/*
Purpose:
  Use to remove temp particle files - call after they have been saved elsewhere (e.g.,
    to text files for importing elsewhere)
*/
{
  Int intYear;
  char strFileName[MAXSTR];

  for(intYear = 1; intYear <= pControlPF->intNYears; intYear ++) {
    sprintf(strFileName, "%sYr%i%s%s", pControlPF->strResultsPrefix, intYear, "States", strSuffix);
    remove(strFileName);
    sprintf(strFileName, "%sYr%i%s%s", pControlPF->strResultsPrefix, intYear, "Params", strSuffix);
    remove(strFileName);
    sprintf(strFileName, "%sYr%i%s%s", pControlPF->strResultsPrefix, intYear, "Weights", strSuffix);
    remove(strFileName);
    sprintf(strFileName, "%sYr%i%s%s", pControlPF->strResultsPrefix, intYear, "ID", strSuffix);
    remove(strFileName);
    sprintf(strFileName, "%sYr%i%s%s", pControlPF->strResultsPrefix, intYear, "Parent", strSuffix);
    remove(strFileName);
    sprintf(strFileName, "%sYr%i%s%s", pControlPF->strResultsPrefix, intYear, "B", strSuffix);
    remove(strFileName);
  }
}

/*------------------------------------------------------------------------*/

void writeCovMat(controlPFstruct * const pControlPF,
  particlestruct * const pParticles,
  double a_wt, double **theta_star,
  double *mean_theta_star, double **mean_mat,  double **sigma_theta_star)
 /* Purpose: writes output from CovMat routine to debug file */
 {

  Int intNumPars, intNumSims, i, j;

  intNumSims =pControlPF->intNParticles;
  intNumPars =pControlPF->intNModelParams;

  fprintf(pControlPF->pDebugFile, "\ndebugCovMat\n-----------\n\n");
  fprintf(pControlPF->pDebugFile, "a_wt%e\n",a_wt);

  fprintf(pControlPF->pDebugFile, "params (only those in model)\n");
  for(i=1; i<=pControlPF->intNParticles; i++) {
    for (j=0; j<NPARAMS;j++) {
      if(pControlPF->intParamNumber[j]>0)
        fprintf(pControlPF->pDebugFile, "%e, ", pParticles[i].dblParams[j]);
    }
    fprintf(pControlPF->pDebugFile, "\n");
  }


  fprintf(pControlPF->pDebugFile, "theta_star\n");
  for(i=1;i<=intNumSims;i++) {
    for(j=1;j<=intNumPars;j++) {
      fprintf(pControlPF->pDebugFile, "%e, ", theta_star[i][j]);
    }
    fprintf(pControlPF->pDebugFile, "\n");
  }

  fprintf(pControlPF->pDebugFile, "mean_theta_star\n");
  for(i=1;i<=intNumPars;i++) {
    fprintf(pControlPF->pDebugFile, "%e\n", mean_theta_star[i]);
  }

  fprintf(pControlPF->pDebugFile, "mean_mat\n");
  for(i=1;i<=intNumSims;i++) {
    for(j=1;j<=intNumPars;j++) {
      fprintf(pControlPF->pDebugFile, "%e, ", mean_mat[i][j]);
    }
    fprintf(pControlPF->pDebugFile, "\n");
  }

  fprintf(pControlPF->pDebugFile, "sigma_theta_star\n");
  for(i=1;i<=intNumPars;i++) {
    for(j=1;j<=intNumPars;j++) {
      fprintf(pControlPF->pDebugFile, "%e, ", sigma_theta_star[i][j]);
    }
    fprintf(pControlPF->pDebugFile, "\n");
  }

  fflush(pControlPF->pDebugFile);

 }
