#RStates.r -- R code to read in files containing state values generated 
# by SealPF program, and print/plot them

#Author: Len Thomas, CREEEM
#Last updated: 1 August 2016

#To run with the additional year of data:
# - Run with adult data (n.state=-1) and use.add.info.for.weight==T
#  this will output a new weight file AddInfoWeights and a new 
#  file containing the total population size AddInfoTotPop
# - Back up the old weights file and copy the new one in its place
# - Then run with pup data (n.state=1) and use.add.info.for.weight==F and
#  use.add.info.for.lnl==T.  This will produce an AIC that uses the
#  new weights (assuming you did the above step), but also uses the 
#  total population size estimate also
#Note: weights are rescaled during this process so their mean is 1 

RStates<-function(read.results=TRUE,max.particles=0,prefix="NewRes",n.state=-1,weight.file="Weights",input.files.compact=0,
  obs.file="PupData_1984_2011.txt",state.estimates=NULL,
  plot.results=TRUE,plot.target="png",print.results=TRUE,
  file.results=TRUE,print.lnl=TRUE,use.add.info.for.lnl=FALSE,
  use.add.info.for.weight=FALSE){
#Purpose: Function to read in states, compute summary statistics on them, and display/print
# these
#Inputs:
#  read.results - if T then reads results, and optionally other stuff (see below)
#    if F then assumes that some vars are already in global workspace
#  max.particles - if 0 reads in all the particles available; otherwise reads at most
#    this many (fewer if fewer are available). Useful for testing the routine before reading in 
#    huge particle files!  Ignored if read.results is F.
#  prefix - if reading in results then the prefix for results files
#  n.state - state (i.e., age) to read in and process
#    If 0, then calculate total population size, including males and pups.
#    If -1, then calculate total population size, including males, excluding pups.
#    Note that 1 is pups, 2 is age 1 adults, etc...
#  weight.file - weight file to read from - this is the middle portion of the name
#    can be changed to AddInfoWeights to make inferences after additional info (i.e., the independent
#    population count) has been added
#  input.files.compact - whether the input files are compact: 0 means compact, 1 means not compact, 
#    2 means compact except last year.  Note in the case of 2, this routine will not calculate the correct 
#    number of animals in the last year except when n.state = 1, 0 or -1
#  obs.file - name of the file containing pupcounts - used when n.state is 1 for plotting and calculating LnL
#    Header row gives region names, header column gives year names, -1 is missing data.
#    Also used for other years to work out the number of colonies, number of years etc - so always needs to be
#    passed in -- it's even read if read.results is F
#  state.estimates - if read.results is F then pass in state estimates - a list containing mean.est, lcl.est, ucl.est
#  plot.results - if T then plots the trajectories
#  plot.target - png, pdf or screen
#  print.results - if T displays the results
#  file.results - if T then saves to csv
#  print.lnl - if T then prints negative log integrated likelihood - only if n.state is 1 (pups)
#    assumes CV is known (value given below in code) and a normal measurement error model
#    Only makes sense if state is 1 (i.e., pups)
#  use.add.info.for.lnl - set this to T when using weight file AddInfoWeights, so that the LnL
#    is calculated including the independent population count
#  use.add.info.for.weight - if this is true, then the additional independent population estimate(s)
#    of adults is/are used to calculate and
#    write out a new weights file, and a file containing the total population size estimated for that year
#    (files AddInfoWeights and AddInfoTotPop) and then the routine stops - so not plots etc are produced
#    You need to say n.state=-1 for this one, so you're calculating adult states
#    as it needs to know the adult numbers for teh years with indep estimates to calculate the likelihood

  # Requires the Hmisc R library
  library(Hmisc)
  
  #Options on what action is required - not usually set, so not parameters
  #-----------------------------------------------------------------------

  #first year label - set to NA to use the one from the input file instead
  start.year<-1984
  #If T, prints the results
  round.printed.results<-TRUE
  #If T, only reads in the final year of data -- useful to save time
  # when it's just the last year that's of interest
  only.final.year<-FALSE
  
  #Some checks
  if((print.lnl)&(n.state!=1))
    stop ("You asked to print LnL, but this is only possible if state is 1 (i.e., pups)")
  if((use.add.info.for.lnl)&(!print.lnl))
    stop("You asked to use the additional info in the LnL, but you did not ask to calculate the LnL!")
  if ((use.add.info.for.weight)&(n.state!=-1))
     stop ("Can only use the use.add.info.for.weights option when doing adult counts\n")
  if((!read.results)&is.null(state.estimates))
     stop("You set read.results to FALSE but did not pass in any state estimates!")
  
  #This is the info on the independent counts, and the weight file and population count info to save
  #2015: Now scaled so it accounts for the 0.0766 non-surveyed pup production
  #2016: Now able to cope with multiple years -- earliest year first
  add.info.years<-c(2008,2014)
  n.add.info.years<-length(add.info.years)
  add.info.shift<-c(56753.92447, 55731.10604) #55745.17532) #51989.50306,51989.50306) #57949.67 #old 44482.15
  add.info.scale<-c(2406.20071, 2413.67866) #2405.77625)  #1622.56156,1622.56156) #1861.16 #old 8506.336
  add.info.shape<-c(19.77998, 33.83813) #33.29523) #18.20586,18.20586) #16.3035 #old 3.888958
  add.info.weight.file<-"AddInfoWeights"
  add.info.tot.pop.file<-"AddInfoTotPop"

  #This is the info about sex ratio
  sex.ratio.file<-"SexRatio"
  
  if(n.state==0) state.filename<-paste("TotalPop")
  if(n.state==-1) state.filename<-paste("Adults")
  if(n.state>0) state.filename<-paste("Age",n.state-1,"",sep="")
  
  #Options about the data, model, etc
  #----------------------------------
  #Set to T if the first column of obs file is year - this gets ignored
  obs.file.first.col.year<-T
  #Directory to read results files from.  If "" then reads from current directory.
  # if this is a vector of directory paths, then reads from multiple directories
  # and collates the results
  dir="" 
  #Whether input files are text (T) or binary (F) 
  input.files.text<-F 

  #as an aside, last time I checked, all integers are stored as 16 bytes per number, which means
  # we should be able to cope with seal numbers up to 2^(16*8-1)-1 = 1.701412e+38!
  size.real.compact<-4
  size.real.double<-8
  n.ages.compact<-3
  n.ages.double<-7
  if(input.files.compact==0){
    size.real<-size.real.last.year<-size.real.double
    n.ages<-n.ages.last.year<-n.ages.double
  }
  if(input.files.compact==1){
    size.real<-size.real.last.year<-size.real.compact
    n.ages<-n.ages.last.year<-n.ages.compact
  }
  if(input.files.compact==2){
    size.real<-size.real.compact
    size.real.last.year<-size.real.double
    n.ages<-n.ages.compact
    n.ages.last.year<-n.ages.double
  }
  
  #Measurement error CV - used in calculating AIC
  CV<-1/sqrt(125)
  #Alpha level
  alpha<-0.05
  
  #Read in observed pup values
  #----------------------------
  #(used to get number of years, number of colonies, etc)
  temp <- read.table(obs.file,header=T,na.strings="-1")
  if(obs.file.first.col.year) temp<-temp[,-1]
  if(length(dim(temp))==1) {
    #If there's only one colony, have to make sure it's an array
    obs.state <-array(temp[1:n.years,],dim=dim(temp))
    dimnames(obs.state)<-dimnames(temp)
    K<-1
  } else {
    obs.state<-temp
    K<-dim(obs.state)[2]
  }
  rm(temp)
  #Set start and end year
  n.years<-dim(obs.state)[1]
  if(is.na(start.year)) {
    start.year<-as.numeric(dimnames(obs.state)[[1]][1])
  } else {
    #set the dimnames
    dimnames(obs.state)[[1]]<-start.year:(start.year+n.years-1)
  }
  end.year<-start.year+n.years-1
  if(only.final.year) {
    #start with the final year
    final.year<-n.years
  } else {
    #start with the first year
    final.year<-1
  }
  
  if(read.results) {
    
    #Initialize stuff
    #--------------------
  
    #Initialize variables
    #Results matrices
    mean.est <- lcl.est <- ucl.est <- matrix(NA,n.years,K)
    dimnames(mean.est)<-dimnames(lcl.est)<-dimnames(ucl.est)<-list(as.character(start.year:(start.year+n.years-1)),dimnames(obs.state)[[2]])
    ci <- 1-alpha
    #number of directories of results to read in
    n.dirs<-length(dir)
    #will store the number of particles per directory
    n.particles<-numeric(n.dirs)
    #total particles
    total.particles<-0
    
    #Read in the weights files
    #--------------------
    for(i in 1:n.dirs) {
      weight.file.name<-paste(dir[i],prefix,"Yr",n.years,weight.file,ifelse(input.files.text,".txt",""),sep="")
      if(input.files.text) {
        weights.temp<-scan(weight.file.name)
        n.particles[i]<-length(weights)
        if(max.particles>0) {
          if(total.particles+n.particles[i]>max.particles) {
            n.particles[i]<-max.particles-total.particles
            weights.temp<-weights.temp[1:n.particles[i]]
          }
        }
      } else {
        size<-file.info(weight.file.name)$size
        n.particles[i]<-size/size.real.last.year
        if(max.particles>0) {
          if(total.particles+n.particles[i]>max.particles){
            n.particles[i]<-max.particles-total.particles
          }
        }      
        #open and read in
        weights.file<-file(description=weight.file.name,open="rb")
        weights.temp<-readBin(weights.file,"numeric",size=size.real.last.year,n=n.particles[i])
        close(weights.file)
      }
      if(i==1) {
        weights<-weights.temp
      } else {
        #Note - this part is not efficient -- if I do go back to reading from multiple directories, I should
        # probably start by creating a matrix of the right size, and then filling in, rather than this!
        weights<-c(weights, weights.temp)
      }
      total.particles=total.particles+n.particles[i]
    }
    rm(weights.temp,weights.file)
    gc()
    
    #Read in the additional info total population files
    #--------------------------------------------------
    #(Note - now extended so it can cope with multiple input files)
    if(use.add.info.for.lnl){
      total.adults<-matrix(0,total.particles,n.add.info.years)
      for(j in 1:n.add.info.years){
        start.particle<-1 
        for(i in 1:n.dirs) {
          end.particle<-start.particle+n.particles[i]-1
          total.pop.file.name<-paste(dir[i],prefix,"Yr",(add.info.years[j]-start.year+1),add.info.tot.pop.file,ifelse(input.files.text,".txt",""),sep="")
          if(input.files.text) {
            total.adults[start.particle:end.particle,j]<-scan(total.pop.file.name)
          } else {
            #open and read in
            tot.pop.file<-file(description=total.pop.file.name,open="rb")
            total.adults[start.particle:end.particle,j]<-readBin(tot.pop.file,"integer",n=n.particles[i])
            close(tot.pop.file)
          }
          start.particle<-end.particle+1
        }
      }
    }
  
    #Read in the sex ratio file (if retrieving adult numbers)
    #---------------------------------------------
    if(n.state<=0){
      sex.ratio<-numeric(total.particles)
      start.particle<-1 
      for(i in 1:n.dirs) {
        end.particle<-start.particle+n.particles[i]-1
        sex.ratio.file.name<-paste(dir[i],prefix,"Yr",n.years,sex.ratio.file,ifelse(input.files.text,".txt",""),sep="")
        if(input.files.text) {
          sex.ratio[start.particle:end.particle]<-scan(sex.ratio.file.name)
        } else {
          #open and read in
          sex.ratio.file<-file(description=sex.ratio.file.name,open="rb")
          sex.ratio[start.particle:end.particle]<-readBin(sex.ratio.file,"numeric",size=size.real.last.year,n=n.particles[i])
          close(sex.ratio.file)
        }
        start.particle<-end.particle+1
      }
    }
      
    #Read in and process the states files, one year at a time
    #--------------------------------------------------------
    #sum.lnl will contain the log likelihood for each particle
    if((n.state==1)&(print.lnl)) sum.lnl=numeric(total.particles)
    
    if (use.add.info.for.weight) {

#      final.year<-add.info.year-start.year+1
#      init.year<-final.year
      years<-add.info.years-start.year+1
      
    } else {
      #Start at the end, and work backwards through the years, unless just doing 
      #the additional info calculation, in which case just do the add.info.year
      init.year<-n.years
      years<-seq(init.year, final.year,by=-1)
    }
    if(use.add.info.for.weight | use.add.info.for.lnl){
      #Start a counter for each add info year that's been processed
      add.info.counter<-1
    }
    for (year in years) {
  
      cat("Processing year",year,"\n")
      n.ages.this.year<-ifelse(year==n.years,n.ages.last.year,n.ages)
  
      #will contain the posterior state values for all particles for this year
      post.state<-matrix(0,K,total.particles)
    
      #Read in data for the current year from each directory
      for(i in 1:n.dirs) {
        if (i==1) {
          start.i = 1
        } else {
          start.i = start.i + n.particles[i-1]
        }
        if(n.particles[i]>0){
          state.file.name <- paste(dir[i],prefix,"Yr",year,"States",ifelse(input.files.text,".txt",""),sep="")
          #read 'em in
          if(input.files.text) {
              post.states<-array(scan(state.file.name,n=n.particles[i]*n.ages.this.year*K),c(n.ages.this.year,K,n.particles[i]))
          } else {
            state.file<-file(description=state.file.name,open="rb")
            post.states<-array(readBin(state.file,"integer",n=n.particles[i]*n.ages.this.year*K),c(n.ages.this.year,K,n.particles[i]))
            close(state.file)    
          }
          if(n.state>0) {
            #just read in the state you want
            post.state[,start.i:(start.i+n.particles[i]-1)]<-post.states[n.state,,]
          } else {
            #combine all adult states
            for(age in 2:n.ages.this.year) {
              for(k in 1:K) {
                #add in this component of the population - note "sex.ratio" is number of adults per female
                post.state[k,start.i:(start.i+n.particles[i]-1)]<-
                  post.state[k,start.i:(start.i+n.particles[i]-1)]+(post.states[age,k,]*sex.ratio)
              }
            }
            if(n.state==0) {
              #also add in the pups
              for(k in 1:K) {
                #add in this component of the population
                post.state[k,start.i:(start.i+n.particles[i]-1)]<-
                  post.state[k,start.i:(start.i+n.particles[i]-1)]+(post.states[1,k,])
              }
            }
          }
        } #end if n.particles[i]>0
      }
    
      #add in the additional information
      if(use.add.info.for.weight){
        
        #don't have to test for whether it's an add info year as only add info
        #years are used in the loop when use.add.info.for.weight is true
        #calculate total pop size
        total.adults<-apply(post.state,2,sum)

        #Initialize if you're on the first additional year
        if(add.info.counter == 1) add.weight<-rep(1,length(total.adults))
        #work out likelihood
        add.weight<-add.weight * dgamma(total.adults-add.info.shift[add.info.counter],shape=add.info.shape[add.info.counter],scale=add.info.scale[add.info.counter])

        #Write out the tot pop size file for this year
        start.particle<-1
        for(i in 1:n.dirs){
          total.pop.file.name<-paste(dir[i],prefix,"Yr",year,add.info.tot.pop.file,ifelse(input.files.text,".txt",""),sep="")
          end.particle<-start.particle+n.particles[i]-1
          if(input.files.text) {
            cat(total.adults[start.particle:end.particle],file=total.pop.file.name,sep="\n",append=F)
          } else {
            #write out
            tot.pop.file<-file(description=total.pop.file.name,open="wb")
            writeBin(as.integer(total.adults[start.particle:end.particle]),tot.pop.file)
            close(tot.pop.file)
          }
          #get ready to do the next lot
          start.particle<-end.particle+1
        }
        
        #If you're on the last year then rescale and write out the new weights file
        if(add.info.counter == n.add.info.years) {
          #rescale so the new weights have a mean of 1 - so don't upset the old weights too much
          add.weight<-add.weight/mean(add.weight)
          #now use this to adjust the weights
          new.weights<-weights*add.weight
          #rescale again
          new.weights<-new.weights/mean(new.weights)
    
          #now write the weights out to file
          start.particle<-1
          for(i in 1:n.dirs) {
            weight.file.name<-paste(dir[i],prefix,"Yr",n.years,add.info.weight.file,ifelse(input.files.text,".txt",""),sep="")
            end.particle<-start.particle+n.particles[i]-1
            if(input.files.text) {
              cat(new.weights[start.particle:end.particle],file=weight.file.name,sep="\n",append=F)
            } else {
              #write out
              weights.file<-file(description=weight.file.name,open="wb")
              writeBin(new.weights[start.particle:end.particle],weights.file,size=size.real.last.year)
              close(weights.file)
            }
            #get ready to do the next lot
            start.particle<-end.particle+1
          }
          
          #and exit
          cat("New weights calculated as required.  Exiting function now.\n")
          return(NULL)
        } else {
          #Add another year to the counter
          add.info.counter <- add.info.counter + 1
        }
      }
    
      #Work out mean and confidence limits for this year
      for (k in 1:K) {
        mean.est[year,k]<-wtd.mean(post.state[k,],weights=weights,normwt=T)
        cl<-wtd.quantile(post.state[k,],probs<-c((1-ci)/2,1-(1-ci)/2),
          weights=weights,normwt=T)
        lcl.est[year,k] <- cl[1]
        ucl.est[year,k] <-cl[2]
      }
      
      #Work out the log-likihood for each particle for this year and add to total
      if((n.state==1)&(print.lnl)) {
        lnl.mat<-dnorm(as.vector(obs.state[year,],mode="numeric"),post.state,CV*post.state,log=T)
        lnl<-apply(lnl.mat,2,sum)
        if(use.add.info.for.lnl) {
          #Check if it's an add info year
          #Note -- we are working backwards from the last year to the first
          #There was possibly a bug here previously as the add info was added to the lnl in all years!
          if(add.info.counter<=n.add.info.years) {
            tmp.index<-n.add.info.years-add.info.counter+1
            if(year == (add.info.years[tmp.index])-start.year+1) {
                #It's an add info year
                lnl<-lnl+dgamma(total.adults[,tmp.index]-add.info.shift[tmp.index],shape=add.info.shape[tmp.index],scale=add.info.scale[tmp.index],log=T)
              add.info.counter<-add.info.counter+1
            }
          }
        }
        lnl[!is.finite(lnl)]<--500
        sum.lnl=sum.lnl+lnl
      }
    
      rm(post.state, post.states)
    } #end of the year loop
    
    if((n.state==1)&(print.lnl)) {
      #Print out negative log integrated likelihood
      max.lnl<-max(sum.lnl)
      tmp<-exp(sum.lnl-max.lnl)
      nlnil<- - (log(wtd.mean(tmp,weights=weights,normwt=T)) + max.lnl)
      cat("negative log integrated likelihood is ", nlnil,"\n")
     
    }
  
  } else {
    #dont' read in results
    cat("Results not being read from file\n")
    #make the passed in state estimates available
    attach(state.estimates)
  }
  
  if(print.results){
    for(k in 1:K) {
      cat("\nColony", k, "\n")
      for (year in final.year:n.years) {
        if(round.printed.results) {
          cat(round(mean.est[year,k]/1000,1), " (", round(lcl.est[year,k]/1000,1), " ", round(ucl.est[year,k]/1000,1), ")\n", sep="")
        } else {
          cat(mean.est[year,k], " (", lcl.est[year,k], " ", ucl.est[year,k], ")\n", sep="")
        }
      }
    }
    cat("\nTotal", "\n")
    for (year in final.year:n.years) {
      if(round.printed.results) {
        cat(round(sum(mean.est[year,])/1000,1), " (", round(sum(lcl.est[year,])/1000,1), " ", round(sum(ucl.est[year,])/1000,1), ")\n", sep="")
      } else {
        cat(sum(mean.est[year,]), " (", sum(lcl.est[year,]), " ", sum(ucl.est[year,]), ")\n", sep="")
      }
    }
  }
  
  if(file.results) {
    write.csv(mean.est,file=paste("Mean",state.filename,weight.file,".csv",sep=""))
    write.csv(lcl.est,file=paste("Lcl",state.filename,weight.file,".csv",sep=""))
    write.csv(ucl.est,file=paste("Ucl",state.filename,weight.file,".csv",sep=""))
  }
  
  if(plot.results) {
  
    #turn on the printing device
    if (plot.target=="pdf") pdf(paste("states",state.filename,weight.file,".pdf",sep=""))
    if (plot.target=="png") png(paste("states",state.filename,weight.file,".png",sep=""),width=1000,height=1000 ,pointsize=18)
    if (plot.target=="bitmap") bitmap(paste("states",state.filename,weight.file,".bmp",sep=""),width=7,height=7,units="in",pointsize=10,res=300)#,cex=1.0,cex.axis=1.3,cex.lab=1.3,cex.main=2.0)
    
    if(K>1) par(mfrow=c(2,2),ask=F,cex=1.0,cex.axis=1.3,cex.lab=1.3,cex.main=1.4)
    if(K==1) par(mfrow=c(1,1),ask=F)
    for(k in 1:K) {
      if (n.state==1) {
        min.x<- min(obs.state[,k],mean.est[,k],ucl.est[,k],lcl.est[,k],na.rm=T)
        max.x<- max(obs.state[,k],mean.est[,k],ucl.est[,k],lcl.est[,k],na.rm=T)
        if(sum(is.na(obs.state[,k]))<length(obs.state[,k])) {
          #at least some non-null pup counts
          plot(dimnames(obs.state)[[1]],obs.state[,k],
            xlab="Year",ylab="Pups",type="p",xlim=c(start.year,end.year),ylim=c(min.x,max.x))
          lines((start.year):end.year,mean.est[,k])
        } else {
          #all observations are null
          plot((start.year):end.year,mean.est[,k],type="l",
            xlab="Year",ylab="Pups",xlim=c(start.year,end.year),ylim=c(min.x,max.x))
        }
        lines((start.year):end.year,lcl.est[,k],lty=3)
        lines((start.year):end.year,ucl.est[,k],lty=3)
      } else {
        min.x<- min(mean.est[,k],ucl.est[,k],lcl.est[,k])
        max.x<- max(mean.est[,k],ucl.est[,k],lcl.est[,k])
        plot((start.year):end.year,mean.est[,k],type="l",
          xlab="Year",ylab="State",xlim=c(start.year,end.year),ylim=c(min.x,max.x))
        lines((start.year):end.year,lcl.est[,k],lty=3)
        lines((start.year):end.year,ucl.est[,k],lty=3)
      }
      title(dimnames(obs.state)[[2]][k])
    }
    #par(mfrow=c(1,1))
  
    #turn off the printing device
    if(!(plot.target=="screen")) dev.off()
  
  }

  if(read.results){
    state.estimates<-list(mean.est=mean.est,lcl.est=lcl.est,ucl.est=ucl.est)
    if((n.state==1)&(print.lnl)) state.estimates$nlnil<-nlnil
  } else {
    detach(state.estimates)
  }
  invisible(state.estimates)
}
